﻿define(["jquery","underscore","backbone","localStorage","device","settings","globalize","preferences","helper",'logger'],
        function ($, _, Backbone, storage, device, settings, globalize, preferences, helper, logger) {

            function ApiAdapter() {
                function initSync() {
                    // override sync method, so that we can inject out stuff
                    var originalSync = Backbone.sync;

                    var customSync = function (method, model, options) {
                        return originalSync(method, model, options).fail(function (xhr) {

                            // stop further redirects if not online
                            //if (!device.isOnline()) {
                            //    return xhr;
                            //}

                            logger.log("ajax fail: " + xhr.status + " " + xhr.statusText);
                            var unauthorized = (xhr.status == 401);

                            if (unauthorized) {
                                authTicket(null);
                                Backbone.Notifications.trigger("action", "unauthorized");
                                return xhr;
                            }
                        });
                    }
                    Backbone.sync = customSync;
                }

                function _ajaxBeforeSend(xhr, options) {
                    if (!device.isOnline()) {
                        return false;
                    }

                    var lang = globalize.culture();

                    if (options.url.indexOf("/") == 0) {
                        options.contentType = 'application/json';
                        options.url = settings.apiUrl + options.url;

                        options.url += ((options.url.indexOf("?") >= 0) ? "&" : "?") + "lang=" + lang.name;
                    }

                    xhr.crossDomain = true;

                    var ticket = authTicket();
                    if (ticket) {
                        xhr.setRequestHeader("X-Ticket", ticket);
                    }

                    //// add language header

                    //xhr.setRequestHeader("Accept-Language", lang.name);

                    logger.log("ajax call: " + options.url);

                    return true;
                }

                function initAjax() {
                    $.ajaxSetup({
                        timeout: settings.apiTimeout,
                        beforeSend: function (xhr, options) {
                            return _ajaxBeforeSend(xhr, options);
                        }
                    });

                }

                function init() {
                    initAjax();
                    initSync();
                }

                function call(url, data) {
                    data = $.extend({
                        type:"GET"
                    }, data);
                    return $.ajax(url, data);
                }

                 function authTicket(ticket) {
                    if (typeof ticket === "undefined") {
                        return storage.get("auth");
                    }
                    storage.set("auth", ticket);
                }

                function isAuthenticated() {
                    return (authTicket());
                }

                var self = this;

                this.call = call;
                this.authTicket = authTicket;
                this.isAuthenticated = isAuthenticated;

                this.collection = {};
                
                this.collection.location = function (data) {
                    var apiData = {
                        data:$.extend({},{
                            country: settings.countryIdIran
                        },data)
                    }
                    
                    return call("/collection/location", apiData);
                }

                this.collection.category = function (data) {
                    var apiData = {
                        data: $.extend({}, {
                        }, data)
                    }
                    return call("/collection/category", apiData);
                }

                this.collection.getCategoryAttributes = function (id) {
                    var apiData = {
                        data: {
                            category:id
                        }
                    }
                    return call("/collection/category-attributes", apiData);
                }

                this.collection.doping = function (id) {
                    var deferred = $.Deferred();

                    var data = [
                        {
                            id: 0,
                            name: globalize.localize("collections.doping.free.name"),
                            description: globalize.localize("collections.doping.free.description")
                        },
                        {
                            id: 11,
                            name: globalize.localize("collections.doping.bronze.name"),
                            description: globalize.localize("collections.doping.bronze.description")
                        },
                        {
                            id: 12,
                            name: globalize.localize("collections.doping.silver.name"),
                            description: globalize.localize("collections.doping.silver.description")
                        },
                        {
                            id: 13,
                            name: globalize.localize("collections.doping.diamond.name"),
                            description: globalize.localize("collections.doping.diamond.description")
                        }
                    ];
                    if (typeof id === "undefined") {
                        deferred.resolve(data);
                    } else {
                        deferred.resolve(_.find(data, function (o) { return o.id == id }));
                    }

                    return deferred;
                };

                this.collection.offerFilter = {};
                this.collection.offerFilter.sortPreset = function (id) {
                    var deferred = $.Deferred();

                    var data = [
                      //  { id: 0, name: globalize.localize("collections.offerFilter.sortPreset.default") },
                        { id: 1, title: globalize.localize("collections.offerFilter.sortPreset.mostrecent") },
                        { id: 2, title: globalize.localize("collections.offerFilter.sortPreset.lowestprice") },
                        { id: 3, title: globalize.localize("collections.offerFilter.sortPreset.highestprice") }
                    ];
                    if (typeof id === "undefined") {
                        deferred.resolve(data);
                    } else {
                        deferred.resolve(_.find(data, function (o) { return o.id==id }));
                    }
                    
                    return deferred;
                };
                this.collection.offerFilter.datePreset = function (id) {
                    var deferred = $.Deferred();

                    var data = [
                       { id: 0, name: globalize.localize("collections.offerFilter.datePreset.all") },
                       { id: 1, name: globalize.localize("collections.offerFilter.datePreset.1day") },
                       { id: 2, name: globalize.localize("collections.offerFilter.datePreset.2days") },
                       { id: 3, name: globalize.localize("collections.offerFilter.datePreset.7days") }
                    ];

                    if (typeof id === "undefined") {
                        deferred.resolve(data);
                    } else {
                        deferred.resolve(_.find(data, function (o) { return o.id == id }));
                    }

                    return deferred;
                };

                this.me = {};
                this.me.offer = {};
                this.me.offer.doping = {};

                this.me.get = function () {
                    var apiData = {};
                    if (!isAuthenticated()) {
                        var result = $.Deferred();
                        result.resolve(null);
                        return result;
                    };
                    return call("/user/me", apiData);
                }

                this.me.register = function (data) {
                    var apiData = {
                        type:"POST",
                        data: $.extend({}, {

                        }, data)
                    }
                    return call("/register", apiData);
                }

                this.me.forgotPassword = function (data) {
                    var apiData = {
                        type: "POST",
                        data: $.extend({}, {

                        }, data)
                    }
                    return call("/forgot-password-request", apiData);
                }

                this.me.login = function (data) {
                    var apiData = {
                        method: "POST",
                        data: $.extend({}, {

                        }, data)
                    }
                    return call("/login", apiData).done(function (o) {
                        authTicket(o.ticket);
                    });
                }

                this.me.logout = function () {
                    // clear cache
                    authTicket(null);
                }

                this.me.siteLoginRequest = function (returnUrl) {
                    var apiData = {
                        method: "POST",
                        data: {
                            returnUrl: returnUrl
                        }
                    }
                    return call("/user/site-login-request", apiData)
                }

                this.me.offer.get = function (data) {
                    if (_.isNumber(data)) {
                        return call("/user/offer/"+data);
                    }

                    return call("/user/offer", {
                        data: data
                    });
                }

                this.me.offer.add = function (data) {
                    return call("/user/offer", {
                        method: "POST",
                        data: data
                    });
                }

                this.me.offer.update = function (id, data) {
                    return call("/user/offer/" + id, {
                        method: "PUT",
                        data:data
                    });
                }

                this.me.offer.remove = function (id) {
                    return call("/user/offer?id=" + id, {
                        method: "DELETE"
                    });
                }

                this.me.offer.complete = function (id) {
                    return call("/user/offer/" + id + "/complete", {
                        method: "POST"
                    });
                }

                this.me.offer.doping.list = function (id) {
                    return call("/user/offer/"+ id+ "/doping", {
                        method: "GET"
                    });
                }

                this.me.offer.doping.prepare = function (id, data) {

                    return call("/user/offer/" + id + "/doping/prepare", {
                        method: "POST",
                        data:data
                    });
                }

                

                this.me.offer.imageUpload = function (file, callbacks) {
                    var deferred = $.Deferred();
                    if (!file) {
                        deferred.reject();
                        return deferred;
                    }

                    var formData = new FormData();

                    // HTML file input user's choice...
                    formData.append("file", file);

                    //logger.log("Upload array length: " + rawData.length);
                    
                    var apiData = {
                        method: "POST",
                        contentType: false,
                        beforeSend: function (xhr, options) {
                            if (!_ajaxBeforeSend(xhr, options)) return false;
                            if (file.name) {
                                xhr.setRequestHeader('X-Content-Name', encodeURIComponent(file.name));
                            }
                        },
                        data: formData,
                        cache: false,
                        processData: false,
                        xhr: function () {
                            myXhr = $.ajaxSettings.xhr();
                            if (myXhr.upload && callbacks && callbacks.fileProgress) {
                                myXhr.upload.addEventListener('progress', callbacks.fileProgress, false);
                            }
                            return myXhr;
                        }
                    }
                    return call("/user/offer/image/upload-alt", apiData).done(function (o) {
                        deferred.resolve(o);
                    }).fail(function (data) {
                        deferred.reject(data);
                    });

                    return deferred;
                }

                this.me.offer.imageUploadAlt = function (file, callbacks) {
                    var deferred = $.Deferred();
                    if (!file) {
                        deferred.reject();
                        return deferred;
                    }

                    var options = new FileUploadOptions();
                    options.fileName = file.name;
                    options.mimeType = file.type;
                    options.chunkedMode = false;
                    options.params = {};

                    var fileFullPath = file.fullPath;
                    if (!fileFullPath && typeof file.toURL !== 'undefined') fileFullPath = file.toURL();

                    var ticket = authTicket();
                    if (ticket) {
                        var headers = { 'X-Ticket': ticket };
                        options.headers = headers;
                    }

                    var ft = new FileTransfer();
                    ft.upload(fileFullPath, encodeURI(settings.apiUrl + "/user/offer/image/upload-alt"), function (res) {
                       // alert("bytes: " + res.bytesSent + " responseCode: " + res.responseCode + " response: " + res.response + " isString: " + _.isString(res.response));
                        if (_.isString(res.response)) {
                            deferred.resolve(JSON.parse(res.response));
                        } else {
                            deferred.resolve(res.response);
                        }
                    }, function (err) {
                      //  var errorMessage = "code: " + err.code + " source: " + err.source + " target: " + err.target + " httpcode: " + err.http_status + " body: "+err.body + " url: "+ settings.apiUrl;
                        
                     //   alert("ft.upload.err: " + errorMessage);
                        deferred.reject(err);
                    }, options);

                    return deferred;
                }

                this.offer = {
                };

                this.offer.imageUrl = function (key, options) {
                    var params = [];

                    if (key && key.indexOf(".") == -1) {
                        params[params.length] = "process=always";
                    }

                    if (options && options.mode) params[params.length] = "mode=" + options.mode;
                    if (options && options.width) params[params.length] = "width=" + Math.floor(options.width);
                    if (options && options.maxWidth) params[params.length] = "maxwidth=" + Math.floor(options.maxWidth);
                    if (options && options.height) params[params.length] = "height=" + Math.floor(options.height);
                    if (options && options.maxHeight) params[params.length] = "maxheight=" + Math.floor(options.maxHeight);

                    var imageUrl = settings.imageUrl;
                    var paramString = key;
                    if (params.length > 0) {
                        paramString = paramString + "?" + params.join("&");
                    }
                    var hash = helper.hashCode(paramString);
                    imageUrl = imageUrl.replace("[]", Math.abs(hash % settings.staticResourceVariations));

                    return imageUrl + "/o/orig/" + paramString;
                }

                this.offer.addFavorite = function (id, current) {
                    if (typeof current === "undefined") {
                        current = self.offer.getFavorites();
                    }
                    if (!current) {
                        current = [id];
                    } else {
                        // if exists, do nothing
                        if (self.offer.isFavorite(id)) return current;
                        current[current.length] = id;
                    }

                    storage.set("offer-favorites", current);

                    return current;
                }

                this.offer.removeFavorite = function (id, current) {
                    if (typeof current === "undefined") {
                        current = self.offer.getFavorites();
                    }
                    if (!current) {
                        return current;
                    } else {
                        var len = current.length;
                        current = _.without(current, id);
                        if (current.length != len) {
                            storage.set("offer-favorites", current);
                        }
                    }

                    return current;
                }

                this.offer.isFavorite = function (id,current) {
                    if (typeof current==="undefined"){
                        current = self.offer.getFavorites();
                    }
                    
                    if (!current) return false;
                    for (var i=0;i<current.length;i++) {
                        // if exists, do nothing
                        if (current[i] == id) return true;
                    }

                    return false;
                }

                this.offer.getFavorites = function () {
                    return storage.get("offer-favorites");
                }

                this.offer.filterSettings = function () {
                    return preferences.offerFilter.apply(this,arguments);
                }

                this.offer.get = function (data) {
                    return call("/v2/offer", {
                        data: data
                    });
                }

                this.offer.sendMessage = function (id,data) {
                    return call("/offer/" + id + "/message", {
                        type: "POST",
                        data: data
                    });
                }

                this.offer.category = function (data) {
                    return call("/offer/category", {
                        data:data
                        });
                }

                this.offer.getCacheKey = function (data) {
                    if (!data) return null;
                    var keyItems = [
                    data.q || '',
                    data.withImage || '',
                    data.categoryId || '',
                    data.p || '',
                    data.datePreset || '',
                    data.priceFrom || '',
                    data.priceTo || '',
                    data.regionId || '',
                    data.cityId || '',
                    data.neighbourhoodId || '',
                    data.filterPreset || '',
                    data.sortPreset || '',
                    data.take || '',
                    data.skip || ''
                    ];

                    return "offer.search." + keyItems.join(".");
                }

                this.image = {};
                this.image.preload = function (url) {
                    var img = new Image();
                    var deferred = $.Deferred();

                    img.onload = function () {
                        deferred.resolve(url);
                    }

                    img.onerror = function (e) {
                        logger.error("error preloading image: " + e);
                        deferred.reject(url);
                    }
                    img.src = url;

                    return deferred;
                }


                this.application = {};
                this.application.upgrade = {};

                this.application.info = function () {
                    var platform = device.platform();
                    return call("/info", {
                        data: {
                            platform: 3,
                            source: settings.source.id
                        }
                    }).done(function (o) {
                        if (o && o.apiVersion) settings.apiVersion = o.apiVersion;
                        if (o && o.appVersion) settings.appVersion = o.appVersion;
                        if (o && o.source) settings.source.location = o.source;
                    });
                }

                var upgradeSettings = {
                    interval: null,
                    lastChecked: null,
                    consentVersion:false
                }
                this.application.upgrade.monitorUpgrade = function (val) {
                    if (typeof val === 'undefined') return (upgradeSettings.interval != null);
                    
                    if (val && settings.updateCheckIntervalSeconds > 0) {
                        if (!upgradeSettings.interval) {
                            upgradeSettings.interval = window.setInterval(function () {
                                self.application.upgrade.check();
                            }, settings.updateCheckIntervalSeconds*1000);
                        }
                    } else {
                        if (upgradeSettings.interval) {
                            window.clearInterval(upgradeSettings.interval);
                        }
                    }
                }

                this.application.upgrade.check = function () {
                    
                    self.application.info().done(function () {
                        
                        if (helper.isVersionGreater(settings.version, settings.appVersion) && upgradeSettings.consentVersion != helper.toVersionString(settings.appVersion)) {
                            upgradeSettings.consentVersion = helper.toVersionString(settings.appVersion);
                            Backbone.Notifications.trigger("updateready", settings);
                        }
                    });
                }

                this.feedback = {};
                this.feedback.send = function (data) {
                    data = $.extend({}, {
                        device: device.asString() + "\n\nVersion: " + helper.toVersionString(settings.version) + "\n\nSource: " + settings.source.id,
                        url: window.location.href
                    },data);
                    return call("/feedback", {
                        type: "POST",
                        data: data
                    })
                }

                init();
            }

            return new ApiAdapter();
        }
);