﻿define(["jquery","localStorage",'logger'],
        function ($,storage,logger) {

            // global instance
            var cleanupTimer = null;

            function CacheAdapter(options) {

                // defaults
                options = $.extend({}, {
                    maxItems: 100
                },options);

                function set(key, data) {
                    var holder = storage.get("cache");

                    // check if max
                    if (options.maxItems && holder.length >= options.maxItems) {
                        if (!cleanupTimer) {
                            cleanupTimer = window.setInterval(function () {
                                cleanup(holder);
                            }, 60000);
                        }
                    }

                    if (typeof holder[key] === 'undefined') {
                        holder.length++;
                    }

                    holder[key] = {
                        data: data,
                        added:new Date()
                    };

                    storage.set("cache", holder);

                    logger.log("added to cache with length: " + holder.length);
                }

                function cleanup(holder) {
                    holder = holder || storage.get("cache");
                    
                    // check if max
                    if (options.maxItems && holder.length >= options.maxItems) {
                        // remove items
                        logger.log("cleaning up cache");
                        var i = holder.length;
                        $.each(holder, function (key, o) {
                            if (key == "length") return;
                            if (i >= options.maxItems && i > 0)
                            {
                                delete o;
                                holder.length--;
                                logger.log("removed from cache. new length: " + holder.length);
                            }
                            i--;
                        });

                        storage.set("cache", holder);
                    }

                }

                function get(key, getter) {
                    var holder = storage.get("cache");

                    if (!holder) {
                        holder= {
                            length: 0
                        };
                        storage.set("cache", holder);
                    }

                    var val = holder[key];
                    if (typeof val === 'undefined' || typeof val.data === 'undefined' || (val.added && options.expiresSeconds && ((new Date()).getTime() - val.added.getTime()) >= options.expiresSeconds * 1000)) {
                       
                        if (typeof getter === 'undefined') {
                            return null;
                        }
                        return getter().done(function (data) {
                            set(key, data);
                        });
                    }

                    if (typeof getter === 'undefined') {
                        return val.data;
                    }

                    var def = $.Deferred();

                    def.resolve(val.data);

                    return def;
                }

                function clear() {
                   
                    if (storage) {
                        var holder = storage.get("cache");

                        if (holder) {
                            $.each(holder, function (i, o) {
                                delete o;
                            });
                        }
                        holder = null;

                        storage.remove("cache");

                        if (cleanupTimer) {
                            window.clearInterval(cleanupTimer);
                            cleanupTimer = null;
                        }
                    }
                }

                this.clear = clear;
                this.get = get;
                this.set = set;
            }

            return CacheAdapter;
        }
);

