﻿define(["logger"],
        function (logger) {

            function TrackingAdapter() {
                var _ga;
                function start() {
                    ensureGA();
                    if (!_ga) {
                        logger.error("GA plugin is not available!");
                        return;
                    }

                    logger.log("GA plugin is available!");

                    _ga.init(function () {
                        logger.log("GA plugin initialized!");
                    }, function (data) {
                        logger.error("Error while initializing GA plugin:" + data);
                    }, "UA-32233635-2", 60);
                }

                function end() {
                    ensureGA();
                    if (!_ga) return;
                    _ga.exit(function () {
                        logger.log("GA plugin exited!");
                        _ga = null;
                    }, function (data) {
                        logger.error("Error while exiting GA plugin:" + data);
                        _ga = null;
                    });

                }

                function trackPage(url) {
                    ensureGA();
                    if (!_ga) return;

                    if (!url) url = window.location.hash;

                    _ga.trackPage(function () {
                        logger.log("Page track executed ok");
                    }, function (data) {
                        logger.error("Error while executing page track:" + data);
                    },url);
                }

                function trackEvent(category,action,label,value) {
                    ensureGA();
                    if (!_ga) return;


                    _ga.trackEvent(function () {
                        logger.log("Event track executed ok");
                    }, function (data) {
                        logger.error("Error while executing event track:" + data);
                    }, category,action,label,value||0);
                }

                function ensureGA() {
                    if (!_ga && window && window.plugins && window.plugins.gaPlugin) _ga = window.plugins.gaPlugin;
                }

                this.trackPage = trackPage;
                this.trackEvent = trackEvent;

                this.start = start;
                this.end = end;
                
            }

            return new TrackingAdapter();
        }
);

