﻿define(["localStorage"],
        function (storage) {
                        
            function UserStorageAdapter() {

                function offerFilter(val) {
                    var KEY = "user.offerfilter";
                    if (typeof val === "undefined") return storage.get(KEY);

                    if (!val) {
                        storage.remove(KEY);
                    } else {
                        storage.set(KEY, val);
                    }

                    /*
                    {
	                    region:[int],
                        city:[int],
                        neighbourhood:[int],
                        category:[int],
                        price:{
	                        from:[int],
                            to:[int]
                        },
                        withImage:[bool],
                        datePreset:[int],
                        sortPreset:[int],
                        keywords:[string]
                    }
                    */
                }

                function hasOfferFilter(filter) {
                    filter = filter || offerFilter();

                    if (!filter) return false;
                    if (hasOfferFilterSort(filter.sortPreset)) return true;
                    if (filter.price && (filter.price.from || filter.price.to)) return true;
                    if (filter.keywords) return true;
                    if (filter.withImage) return true;
                    if (hasOfferFilterLocation(filter.location)) return true;
                    if (hasOfferFilterCategory(filter.category)) return true;

                    return false;
                };

                function hasOfferFilterLocation(val) {
                    var LEVEL_REGION = 2;
                    if (!val) return false;

                    if (val.level > LEVEL_REGION) return true;
                    var currentLocation = location();

                    if (currentLocation && currentLocation.id != val.id) return true;

                    return false;
                };
                function hasOfferFilterCategory(val) {
                    if (!val) return false;
                    if (val > 0 || val.id > 0) return true;

                    return false;
                };

                function hasOfferFilterSort(val) {
                    var DEFAULT_SORT = 1;
                    if (!val) return false;

                    var id = val;
                    if (val.id) id = val.id;
                    
                    return (id > 0 && id != DEFAULT_SORT);
                };

                function clearOfferFilter() {
                    offerFilter(null);
                }

                function clear() {
                    storage.remove("user.offerfilter");
                    storage.remove("user.language");
                    storage.remove("user.location");
                    storage.remove("user.lastContactInfo");
                }

                function language(val) {
                    if (typeof val === "undefined") return storage.get("user.language");
                    storage.set("user.language", val);
                }

                function location(val) {
                    if (typeof val === "undefined") return storage.get("user.location");
                    storage.set("user.location", val);
                }

                function lastContactInfo(val) {
                    if (typeof val === "undefined") return storage.get("user.lastContactInfo");
                    storage.set("user.lastContactInfo", val);
                }

                this.offerFilter = offerFilter;
                this.hasOfferFilter = hasOfferFilter;
                this.hasOfferFilterLocation = hasOfferFilterLocation;
                this.hasOfferFilterSort = hasOfferFilterSort;
                this.hasOfferFilterCategory = hasOfferFilterCategory;
                this.clearOfferFilter = clearOfferFilter;
                this.language = language;
                this.lastContactInfo = lastContactInfo;
                this.location = location;
                this.clear = clear;
            }

            return new UserStorageAdapter();
        }
);

