define(["jquery", "underscore", "backbone","api", "router", "uiContainers", "uiValidation", "uiToast", 'globalize', 'uiGlobal', 'uiModal',
    'uiDialog',
    'preferences',
    'helper',
    'tracking',
    'views/location',
    "logger"],
function ($, _, Backbone,api, router, uiContainers, validation, uiToast, globalize, uiGlobal, uiModal, uiDialog, preferences,helper,tracking,ViewLocation,logger) {
    function initialize() {

        var pageDepth = 0;
      
        helper.prepareApplication();

        // create global event bus
        Backbone.Notifications = {};
        _.extend(Backbone.Notifications, Backbone.Events);

        uiGlobal.init();
        uiContainers.init();
        validation.init();       
        tracking.start();

        // Globalize.start = function () {
        {
            var culture = globalize.culture();
            uiGlobal.html.setAttribute("lang", culture.language);
            uiGlobal.$body.toggleClass("rtl", culture.isRTL).toggleClass("ltr", !culture.isRTL);

            // force redraw - Android 2.3 bug
            uiGlobal.$body[0].offsetWidth;
        }
       // }

        router.on("route", function (route, x) {
            logger.log("Router on route");
            pageDepth++;
            
        });

        //$(document).on("keypress", function (e) {
        //    if (e.keyCode == 27) {
        //        Backbone.Notifications.trigger("navigateback");
        //    }
        //});

        // global events
        Backbone.Notifications.on("navigateback", function (defaultRoute) {
            
            // if dialog options are opened, close them first
            if (uiDialog.options.isOpen()) {
                logger.log("dialog options opened!");
                uiDialog.options.close();
                return;
            }

            if (uiModal.isOpen()) {
                logger.log("modal opened!");
                uiModal.close();
                return;
            }
            if (uiContainers.snap.isOpen())
            {
                uiContainers.snap.close();
                logger.log("closing snap ...");
                return;
            }

            if (pageDepth <= 1) {
                logger.log("should exiting app");
                if (navigator && navigator.app && navigator.app.exitApp) {
                    Backbone.Notifications.trigger("exitapp");
                    return;
                }
            }
            pageDepth = Math.max(pageDepth - 1, 0);

            logger.log("page depth: " + pageDepth);
            if (navigator && navigator.app) {
                navigator.app.backHistory();
            } else {
                uiGlobal.window.history.go(-1);
            }

        }).on("navigate", function (route, data) {
            //var deffers = [];
            //if (uiModal.isOpen()) {
            //    deffers.push(uiModal.close());
            //}
            //if (uiContainers.snap) {
            //    deffers.push(uiContainers.snap.close());
            //}

            //$.when(deffers).always(function () {
            //    router.navigate(route, { trigger: true });
            //});
            if (helper.isHashEqual(route, window.location.hash)) {
                router.prepareForShowView();
                router.reload();
            } else {
                router.navigate(route, { trigger: true });
            }

            
        }).on("action", function (e, action, data) {

            if (action == "unauthorized") {
                uiToast.open(globalize.localize("views.app.sign-out"));
                router.navigate("login", { trigger: true });
                return;
            }

        }).on("reload", function () {
            tracking.trackEvent("navigation", "reload");
            router.reload();
        }).on("updateready", function (data) {
            tracking.trackEvent("update", "ready", helper.toVersionString(data.version));
            uiDialog.confirm(globalize.localize("views.app.upgrade.message"),
                globalize.localize("views.app.upgrade.title"),
                [globalize.localize("views.app.upgrade.button.yes"), globalize.localize("views.app.upgrade.button.no")]).done(function () {
                    tracking.trackEvent("update", "accepted", helper.toVersionString(data.version));
                    window.open(data.source.location, "_system");
                }).fail(function () {
                    tracking.trackEvent("update", "canceled", helper.toVersionString(data.version));
                });
        }).on("locationselect", function (view) {
            var location = preferences.location();
            uiContainers.header.loading(true);
            uiModal.open({
                location: location,
                rootLevelName: globalize.localize("views.location.any"),
                maxLevel: 2, // region
                minLevel:2,
                onSelect: function (data) {
                    $(".title-content", uiContainers.header.get()).html("<i class=\"icon icon-location\"></i> " + data.name);
                    preferences.location({
                        id: data.id,
                        name: data.name,
                        level: data.level
                    });

                    var offerFilter = api.offer.filterSettings() || {};
                    offerFilter.location = null;
                    api.offer.filterSettings(offerFilter);

                    uiToast.open(globalize.localize("views.location.success.info"));

                    tracking.trackEvent("locationChange", "select", "country", data.id);

                    if (view && view.reload) view.reload();
                }
            }, ViewLocation, view).always(function () {
                uiContainers.header.loading(false);
            });
        }).on("exitapp", function () {
            if (navigator && navigator.app && navigator.app.exitApp) {
                tracking.trackEvent("native", "exitapp", "click");
                tracking.end();
                api.application.upgrade.monitorUpgrade(false);
                navigator.app.exitApp();
            }
        });
        
        // cordova native events
        uiGlobal.document.addEventListener("online", function (e) {
            api.application.upgrade.monitorUpgrade(true);
            tracking.trackEvent("native", "network", "online");
            uiToast.success(globalize.localize("views.app.online.info"));
        }, false);

        uiGlobal.document.addEventListener("offline", function (e) {
            api.application.upgrade.monitorUpgrade(false);
            tracking.trackEvent("native", "network", "offline");
            uiToast.danger(globalize.localize("views.app.offline.info"));
        }, false);

        uiGlobal.document.addEventListener("backbutton", function (e) {
            logger.log("back pressed");
            tracking.trackEvent("native", "backbutton", "click");
            Backbone.Notifications.trigger("navigateback");
        }, false);

        uiGlobal.document.addEventListener("menubutton", function (e) {
            tracking.trackEvent("native", "menubutton", "click");
            uiContainers.snap.toggleLeft();
        }, false);

        //uiGlobal.document.addEventListener("resume", function (e) {
            
        //}, false);

        uiGlobal.document.addEventListener("searchbutton", function (e) {
            tracking.trackEvent("native", "searchbutton", "click");
            router.navigate("#offer/search", { trigger: true });
        }, false);

        router.start();

        // hide splash
        if (navigator && navigator.splashscreen) {
            navigator.splashscreen.hide();
        }

        // if device is offline,  show notification
       // uiToast.info("wellcome!");

        api.application.upgrade.monitorUpgrade(true);
    }

    return {
        initialize: initialize
    };
}
);


