﻿define(["jquery", "underscore", "globalize", "logger"],
function ($, _, globalize, logger) {

    var LEVEL_COUNTRY = 1;
    var LEVEL_REGION = 2;
    var LEVEL_CITY = 3;
    var LEVEL_NEIGHBOURHOOD = 4;

    return {
        parseDate: function (iso8601) {
            if (iso8601 === null) return null;

            // try default constructor
            var val = new Date(iso8601);

            // if date can't be parsed, convert it to legacy format
            if (val instanceof Date && isNaN(val.getTime())) {
                return this.parseLegacyDate(iso8601);
            }

            return val;
        },

        parseLegacyDate: function (iso8601) {
            if (iso8601 === null) return null;
            var s = $.trim(iso8601);
            s = s.replace(/\.\d+/, ""); // remove milliseconds
            s = s.replace(/-/, "/").replace(/-/, "/");
            s = s.replace(/T/, " ").replace(/Z/, " UTC");
            s = s.replace(/([\+\-]\d\d)\:?(\d\d)/, " $1$2"); // -04:00 -> -0400
            s = s.replace(/([\+\-]\d\d)$/, " $100"); // +09 -> +0900
            return new Date(s);
        },

        hashCode: function (str) {
            var hash = 0;
            if (str.length == 0) return hash;
            for (i = 0; i < str.length; i++) {
                char = str.charCodeAt(i);
                hash = ((hash << 5) - hash) + char;
                hash = hash & hash; // Convert to 32bit integer
            }
            return hash;
        },

        handleNavigation: function (el) {
            if (!el) return;
            el.toggleClass("active");

            var route = el.data("href");
            if (!route || route.indexOf("javascript:") == 0) {
                return;
            }

            logger.log("data navigate to: " + route);
            if (route == "back") {
                Backbone.Notifications.trigger("navigateback");
            } else {
                Backbone.Notifications.trigger("navigate", route);
            }
        },

        toCategoryCssClass: function (cat) {
            if (!cat && !cat.id) return null;

            if (cat.id == 38663) return "icon icon-cat-home-appliances";
            if (cat.id == 36413) return "icon icon-cat-real-easte";
            if (cat.id == 33121) return "icon icon-cat-vehicles";

            if (cat.id == 39378) return "icon icon-cat-clothing";
            if (cat.id == 38457) return "icon icon-cat-computer";
            if (cat.id == 38546) return "icon icon-cat-mobiles";

            if (cat.id == 39306) return "icon icon-cat-photo";
            if (cat.id == 38783) return "icon icon-cat-video";
            if (cat.id == 38991) return "icon icon-cat-baby";

            if (cat.id == 40178) return "icon icon-cat-jewelry";
            if (cat.id == 39085) return "icon icon-cat-health";
            if (cat.id == 39690) return "icon icon-cat-games";

            if (cat.id == 39600) return "icon icon-cat-toys";
            if (cat.id == 38908) return "icon icon-cat-antiques";
            if (cat.id == 39756) return "icon icon-cat-watches";

            if (cat.id == 42730) return "icon icon-cat-industrial";
            if (cat.id == 39166) return "icon icon-cat-books";
            if (cat.id == 40262) return "icon icon-cat-sports";

            if (cat.id == 38456) return "icon icon-cat-other";
            if (cat.id == 40677) return "icon icon-cat-services";
            if (cat.id == 43384) return "icon icon-cat-jobs";

            if (!cat.parent) return null;

            return this.toCategoryCssClass(cat.parent);
        },

        toDopingIcon: function (id) {

            if (id == 0) return "icon icon-vitamin-free";
            if (id == 11) return "icon icon-vitamin-bronze";
            if (id == 12) return "icon icon-vitamin-silver";
            if (id == 13) return "icon icon-vitamin-diamond";

            return null;
        },
        getNodeLevel: function (data) {
            var level = 0;
            var current = data;
            while (current) {
                level++;
                current = current.parent;
            }

            return level;
        },
        getNodeLevel: function (data) {
            var level = 0;
            var current = data;
            while (current) {
                level++;
                current = current.parent;
            }

            return level;
        },
        applyNodeLevel: function (data) {
            var current = data;
            while (current) {
                if (typeof current.level === 'undefined') current.level = this.getNodeLevel(current);
                current = current.parent;
            }
        },
        toCategoryParentList: function (data) {
            if (!data) return [];
            var cat = null;
            if (_.isArray(data)) {
                if (data.length == 0) return [];

                cat = data[data.length - 1];
            } else {
                cat = data;
            }

            if (cat.value) cat = cat.value;

            var result = [];
            while (cat) {
                result.unshift(cat);
                cat = cat.parent;
            }

            return result;
        },

        isOfferEyeCatching: function (o) {
            if (!o || !o.vitamins) return false;
            if (o.vitamins.isEyeCatching || o.vitamins.isSilverPackage || o.vitamins.isGoldPackage || o.vitamins.isDiamondPackage) return true;

            return false;
        },

        hasOfferVitamin: function (o) {
            if (!o) return false;

            if (isAdvanceSearch) return true;
            if (isCategoryHomePage) return true;
            if (isDiamondPackage) return true;
            if (isDiscount) return true;
            if (isEyeCatching) return true;
            if (isGoldPackage) return true;
            if (isHomePage) return true;
            if (isHotSale) return true;
            if (isSilverPackage) return true;
            if (isTopList) return true;

            return false;
        },

        isVersionGreater: function (current, latest) {

            var isNew = false;
            if (latest && current) {
                if (latest.major > current.major) isNew = true;
                if (latest.major == current.major && latest.minor > current.minor) isNew = true;
                if (latest.major == current.major && latest.minor == current.minor && latest.build > current.build) isNew = true;
            }
            return isNew;
        },
        toVersionString: function (o) {
            if (!o) return null;
            return o.major + "." + o.minor + "." + o.build;
        },

        fromVersionString: function (o) {
            var result = { major: 0, minor: 0, build: 0 };
            if (!o) return result;

            var val = o.split(".");
            for (var i = 0; i < val.length; i++) {
                if (i == 0) {
                    result.major = parseInt(val[i]);
                    if (_.isNaN(result.major)) result.major = 0;
                }
                if (i == 1) {
                    result.minor = parseInt(val[i]);
                    if (_.isNaN(result.minor)) result.minor = 0;
                }
                if (i == 2) {
                    result.build = parseInt(val[i]);
                    if (_.isNaN(result.build)) result.build = 0;
                }

            }
            return result;
        },
        isHashEqual: function (val1, val2) {
            if (val1 == val2) return true;
            val1 = val1 || "#";
            val2 = val2 || "#";
            if (val1.indexOf("#") != 0) val1 = "#" + val1;
            if (val2.indexOf("#") != 0) val2 = "#" + val2;
            return (val1 == val2);;
        },
        isLocationEqual: function (val1, val2) {
            if (val1 == null && val2 == null) return true;
            if (val1 == null || val2 == null) return false;
            return (val1.id == val2.id && val1.level == val2.level);
        },

        getLocationRegion: function (location) {
            if (!location) return null;
            var current = location;
            while (current) {
                if (current.level == LEVEL_REGION) return current;
                current = location.parent;
            }

            return;
        },
        getLocationRegionId: function (location) {
            var val = this.getLocationRegion(location);
            if (val) return val.id;
            return null;
        },
        prepareOfferAttributeList: function (list, exclude) {
            if (typeof exclude === 'undefined') exclude = [];

            var TYPE_NUMBER = 1;
            var TYPE_BOOL = 2;
            var TYPE_DATETIME = 3;
            var TYPE_STRING = 4;
            var TYPE_COLLECTION = 5;

            return $.map(list, function (o) {
                if (exclude.indexOf(o.id) > -1) return null;

                var val = o.values;

                if ((o.id == -1 || o.id == -2) && val) {
                    val = globalize.format(val, "C0");
                }
                if (o.options && _.isArray(o.options)) {
                    var valArr = o.values;
                    if (!_.isArray(valArr)) valArr = [valArr];
                    var arr = $.map(o.options, function (oo) {
                        if (_.contains(valArr, oo.id)) return oo.name;
                        return null;
                    });
                    if (!arr || arr.length == 0) return null;

                    val = arr;
                }

                if (o.type == TYPE_BOOL) {
                    if (val == 'true') val = true;
                    if (val == 'false') val = false;

                    if (val) {
                        val = globalize.localize("views.shared.yes");
                    } else {
                        val = globalize.localize("views.shared.no");
                    }
                }

                return {
                    name: o.name,
                    values: val
                };
            });
        },
        // converts new line to <br>
        nl2br: function (str, is_xhtml) {
            if (!str) return str;
            var breakTag = (is_xhtml || typeof is_xhtml === 'undefined') ? '<br />' : '<br>';
            return (str + '').replace(/([^>\r\n]?)(\r\n|\n\r|\r|\n)/g, '$1' + breakTag + '$2');
        },
        prepareApplication: function () {

            // ensure console is accessible
            if (typeof window.console === "undefined") {
                window.console = {
                    log: function (text) {

                    }
                };
            }

            // ensure Array.indexOf

            if (!Array.indexOf) {
                Array.prototype.indexOf = function (obj) {
                    for (var i = 0; i < this.length; i++) {
                        if (this[i] == obj) {
                            return i;
                        }
                    }
                    return -1;
                }
            }

            String.prototype.toFarsiDigits = function () {
                var tbl = {
                    '0': '०',
                    '1': '१',
                    '2': '२',
                    '3': '३',
                    '4': '४',
                    '5': '५',
                    '6': '६',
                    '7': '७',
                    '8': '८',
                    '9': '९'
                };

                var out = "";
                for (i = 0; i < this.length; i++) {
                    var ch = this[i];
                    var map = tbl[ch];
                    if (typeof map === 'undefined') map = ch;
                    out += map;
                }

                return out;
            }

            String.prototype.toLatinDigits = function () {
                var tbl = {
                    '०': '0',
                    '१': '1',
                    '२': '2',
                    '३': '3',
                    '४': '4',
                    '५': '5',
                    '६': '6',
                    '७': '7',
                    '८': '8',
                    '९': '9'
                };

                var out = "";
                for (i = 0; i < this.length; i++) {
                    var ch = this[i];
                    var map = tbl[ch];
                    if (typeof map === 'undefined') map = ch;
                    out += map;
                }

                return out;
            }

        }

    };
});

