﻿define([
  'models/base', 'globalize'
], function (ModelBase, globalize) {

    var translate = globalize.localizeBase("views.me.offer-edit.validation");

    var EDITOR_TEXTBOX = 1;
    var EDITOR_TEXTAREA = 2;
    var EDITOR_DROPDOWNLIST = 3;
    var EDITOR_LISTBOX = 4;
    var EDITOR_RANGE = 5;
    var EDITOR_CHECKBOX = 6;
    var EDITOR_RADIOBUTTONLIST = 7;
    var EDITOR_CHECKBOXLIST = 8;

    var TYPE_NUMBER = 1;
    var TYPE_BOOL = 2;
    var TYPE_DATETIME = 3;
    var TYPE_STRING = 4;
    var TYPE_COLLECTION = 5;

    var ATTRIBUTE_PRICE = -1;
    var ATTRIBUTE_PRICE_SEC = -2;

    var Model = ModelBase.extend({
        category:false,
        defaults: {
            attributes: null,
            attributesDummy: null
        },
        validation: {
            attributesDummy: {
                fn: function (valueDummy, attr, context) {
                    if (!this.category) {
                        return translate("attributes.category");
                    }

                    var self = this;
                    var errors = [];
                    var attributes = context.attributes;
                    _.each(attributes, function (o) {
                        var attrErrors = self._validateAttribute(o);
                        if (attrErrors) {
                            if (_.isArray(attrErrors)) {
                                errors.push({
                                    value: o,
                                    errors: attrErrors
                                });
                            } else {
                                errors.push({
                                    value: o,
                                    errors: [attrErrors]
                                });
                            }
                        }
                    });

                    if (errors.length > 0) return errors;
                }
            }
        },
        _validateAttribute: function (val) {
            if (!val) return false;
            
            if (val.required) {
                if (this._isEmpty(val)) return translate("attributes.value.required");
            } else {
                if (this._isEmpty(val)) return null;
            }

            // check by content type
            if (!this._isValidType(val)) {
                if (val.type == TYPE_NUMBER) return translate("attributes.value.number.invalid"); 
                return translate("attributes.value.invalid");
            }

            // TODO: AHOO-1845

            return null;
        },
        _isEmpty: function (val) {
            if (!val) return true;
            if (!val.values) return true;
            if (_.isArray(val.values) && val.values.length == 0) return true;
            if (_.isString(val.values) && $.trim(val.values).length == 0) return true;

            return false;
        },
        _isValidType: function (val) {
            if (!val) return true;
            if (val.type == TYPE_NUMBER) {
                if (_.isString(val.values)) {
                    val.values = val.values.toLatinDigits();
                }

                if (val.id == ATTRIBUTE_PRICE || val.id == ATTRIBUTE_PRICE_SEC) {
                    return this._isNumber(val.values);
                }

                return this._isDigit(val.values);
            }
            if (val.type == TYPE_BOOL) {
                if (_.isString(val.values)) {
                    val.values = (val.values.toLowerCase() == "true");
                }
                return _.isBoolean(val.values);
            }
            if (val.type == TYPE_DATETIME) {

            }
            if (val.type == TYPE_STRING) {
                return _.isString(val.values) || this._isNumber(val.values);
            }
            if (val.type == TYPE_COLLECTION && (val.editor == EDITOR_CHECKBOXLIST || val.editor == EDITOR_LISTBOX)) {
                return _.isArray(val.values);
            }

            return true;

        },
        _isNumber: function (value) {
            // value.match(/^-?(?:\d+|\d{1,3}(?:,\d{3})+)(?:\.\d+)?$/)
            return _.isNumber(value) || (_.isString(value) && !isNaN(globalize.parseFloat(value)));
        },
        _isDigit: function (value) {
            return _.isNumber(value) || (_.isString(value) && value.match(/^\d+$/));
        }

    });

    return Model;
});