﻿define(["jquery","api","uiToast", "backbone", "preferences","uiModal","uiContainers","globalize",
    'views/home',
    'views/login',
    'views/register',
    'views/feedback',
    'views/tos',
    'views/privacy-policy',
    'views/forgot-password',
    'views/me/profile',
    'views/me/change-password',
    'views/me/change-email',
    'views/me/change-language',
    'views/me/about-device',
    'views/me/offer',
    'views/me/offer-edit',
    'views/me/offer-edit-doping',
    'views/me/offer-edit-doping-duration',
    'views/me/offer-edit-doping-bank',
    'views/me/offer-edit-doping-confirm',
    'views/me/offer-view',
    'views/me/message',
    'views/me/message-view',
    'views/me/payment',
    'views/offer/view',
    'views/offer/search',
    'views/offer/search-filter',
    'views/offer/favorites',
    'views/offer/hot',
    'views/offer/category',
    'views/_empty',
    'logger'
], function ($,api,uiToast, Backbone, preferences, uiModal,uiContainers,globalize,
    ViewsHome,
    ViewsLogin,
    ViewsRegister,
    ViewsFeedback,
    ViewsTOS,
    ViewsPrivacyPolicy,
    ViewsForgotPassword,
    ViewsMeProfile,
    ViewsMeChangePassword,
    ViewsMeChangeEmail,
    ViewsMeChangeLanguage,
    ViewsMeAboutDevice,
    ViewsMeOffer,
    ViewsMeOfferEdit,
    ViewsMeOfferEditDoping,
    ViewsMeOfferEditDopingDuration,
    ViewsMeOfferEditDopingBank,
    ViewsMeOfferEditDopingConfirm,
    ViewsMeOfferView,
    ViewsMeMessage,
    ViewsMeMessageView,
    ViewsMePayment,
    ViewsOfferView,
    ViewsOfferSearch,
    ViewsOfferSearchFilter,
    ViewsOfferFavorites,
    ViewsOfferHot,
    ViewsOfferCategory,
    ViewsEmpty,
    logger) {

    var AppRouter = Backbone.Router.extend({
        goto: null,
        routes: {
            'login': 'login',
            'logout': 'logout',
            'me': 'me',
            'me/change-password': 'me-change-password',
            'me/change-email': 'me-change-email',
            'me/change-language': 'me-change-language',
            'me/about-device': 'me-about-device',
            'me/offer/create': 'me-offer-create',
            'me/offer/edit/:id/doping': 'me-offer-edit-doping',
            'me/offer/edit/:id/doping/:doping/duration/:duration': 'me-offer-edit-doping-duration',
            'me/offer/edit/:id/doping/:doping/duration': 'me-offer-edit-doping-duration-new',
            'me/offer/edit/:id/doping/:doping/duration/:duration/bank/:bank': 'me-offer-edit-doping-bank',
            'me/offer/edit/:id/doping/:doping/duration/:duration/bank': 'me-offer-edit-doping-bank-new',
            'me/offer/edit/:id/doping/:doping/duration/:duration/bank/:bank/confirm': 'me-offer-edit-doping-confirm',
            'me/offer/edit/:id/doping/confirm': 'me-offer-edit-doping-confirm-free',
            'me/offer/edit/:id': 'me-offer-edit',
            'me/offer/:id': 'me-offer-view',
            'me/offer': 'me-offer',
            'me/message': 'me-message',
            'me/message/:id': 'me-message-view',
            'me/payment': 'me-payment',
            'register': 'register',
            'feedback': 'feedback',
            'tos': 'tos',
            'privacy-policy': 'privacy-policy',
            'forgot-password': 'forgot-password',
            'offer/hot': 'offer-hot',
            'offer/category': 'offer-category',
            'offer/category/:id': 'offer-category',
            'offer/search': 'offer-search',
            'offer/search-filter': 'offer-search-filter',
            'offer/favorites': 'offer-list-favorites',
            'offer/add': 'offer-edit',
            
            'offer/:id': 'offer-view',
            '': 'home',
            '*actions': 'defaultAction'
        },
        appView: null,
        showView:function(View,options){
            // this.appView = view;

            // if there are any models, close then
            var result = $.Deferred();
            var self = this;
            this.prepareForShowView().always(function () {
                var inst =null;
                
                // if view requires authenticated user, show login for annonimous
                if (View.access == "auth" && !api.isAuthenticated()) {
                    uiToast.danger(globalize.localize("views.base.not-authenticated-message"));
                    inst = new ViewsLogin({
                        onSuccess: function () {
                            self.showView(View, options);
                        }
                    });
                } else {
                    inst = new View(options || {});
                }

                self.appView = inst;
                result.resolve(inst);
            })

            return result;
        },
        prepareForShowView:function(){
            var result = $.Deferred();

            uiModal.closeAll().always(function () {
                uiContainers.snap.close().always(function () {
                    result.resolve();
                });
            })

            return result;
        },

        reload: function () {
            if (this.appView && this.appView.reload) {
                // do reload
                this.appView.reload();
            }
        },
        start: function () {

            var router = this;

            this.on('route:login', function () {
               
                router.showView(ViewsLogin);
            });
            this.on('route:logout', function () {
            });
            this.on('route:home', function () {
                preferences.clearOfferFilter();

             //   router.showView(ViewsPrivacyPolicy);
                //router.navigate("offer/search", { trigger: true })

                router.showView(ViewsOfferSearch);
            });
            this.on('route:register', function () {
                router.showView(ViewsRegister);
            });
            this.on('route:feedback', function () {
                router.showView(ViewsFeedback);
            });
            this.on('route:tos', function () {
                router.showView(ViewsTOS);
            });
            this.on('route:privacy-policy', function () {
                router.showView(ViewsPrivacyPolicy);
            });
            this.on('route:forgot-password', function () {
                router.showView(ViewsForgotPassword);
            });

            this.on('route:me', function () {
                router.showView(ViewsMeProfile);
            });

            this.on('route:me-change-password', function () {
                router.showView(ViewsMeChangePassword);
            });

            this.on('route:me-change-email', function () {
                router.showView(ViewsMeChangeEmail);
            });

            this.on('route:me-change-language', function () {
                router.showView(ViewsMeChangeLanguage);
            });

            this.on('route:me-about-device', function () {
                router.showView(ViewsMeAboutDevice);
            });

            this.on('route:me-offer', function () {
                router.showView(ViewsMeOffer);
            });

            this.on('route:me-offer-create', function () {
                router.showView(ViewsMeOfferEdit);
            });

            this.on('route:me-offer-edit', function (id) {
                router.showView(ViewsMeOfferEdit,{
                    id: id
                });
            });

            this.on('route:me-offer-edit-doping', function (id) {
                router.showView(ViewsMeOfferEditDoping,{
                    id: id
                });
            });
            this.on('route:me-offer-edit-doping-duration-new', function (id, doping) {
                router.showView(ViewsMeOfferEditDopingDuration, {
                    id: id,
                    duration: null,
                    doping: doping
                });
            });
            this.on('route:me-offer-edit-doping-duration', function (id,doping,duration) {
                router.showView(ViewsMeOfferEditDopingDuration, {
                    id: id,
                    duration: duration,
                    doping:doping
                });
            });
            this.on('route:me-offer-edit-doping-bank-new', function (id, doping, duration) {
                router.showView(ViewsMeOfferEditDopingBank, {
                    id: id,
                    duration: duration,
                    doping: doping
                });
            });
            this.on('route:me-offer-edit-doping-bank', function (id, doping, duration, bank) {
                router.showView(ViewsMeOfferEditDopingBank, {
                    id: id,
                    duration: duration,
                    bank: bank,
                    doping: doping
                });
            });
            this.on('route:me-offer-edit-doping-confirm', function (id, doping, duration, bank) {
                router.showView(ViewsMeOfferEditDopingConfirm, {
                    id: id,
                    duration: duration,
                    bank: bank,
                    doping: doping
                });
            });
            this.on('route:me-offer-edit-doping-confirm-free', function (id) {
                router.showView(ViewsMeOfferEditDopingConfirm, {
                    id: id,
                    doping: 0
                });
            });

            this.on('route:me-offer-view', function (id) {
                router.showView(ViewsMeOfferView,{
                    id: id
                });
            });

            this.on('route:me-message', function () {
                router.showView(ViewsMeMessage);
            });

            this.on('route:me-message-view', function (id) {
                router.showView(ViewsMeMessageView,{
                    id: id
                });
            });

            this.on('route:me-payment', function () {
                router.showView(ViewsMePayment,{
                });
            });
           
            this.on('route:offer-view', function (id) {
                router.showView(ViewsOfferView,{
                    id: id
                });
            });

            this.on('route:offer-search', function () {
                router.showView(ViewsOfferSearch);
            });

            this.on('route:offer-search-filter', function () {
                router.showView(ViewsOfferSearchFilter);
            });

            this.on('route:offer-list-favorites', function () {
                router.showView(ViewsOfferFavorites);
            });

            this.on('route:offer-hot', function () {
                router.showView(ViewsOfferHot);
            });

            this.on('route:offer-category', function (id) {
                router.showView(ViewsOfferCategory,{
                    id:id
                });
            });

            this.on('route:defaultAction', function (actions) {
                logger.log('No route:', actions);
            });
            
            this.on('route', function (route) {
                logger.log('Route:' + route);
            });

            Backbone.history.start();
        }
    });

    return new AppRouter();
});
