﻿define(["jquery", "backbone", "underscore", "libSnap", "api", "uiDialog", "globalize", "template", 'uiGlobal', 'preferences', 'libHammer', 'helper', 'logger'],
function ($, Backbone, _, Snap, api, uiDialog, globalize, templates, uiGlobal, preferences, Hammer, helper, logger) {

    var translateSideMenu = globalize.localizeBase("views.side-menu");
    var translateHeader = globalize.localizeBase("views.header");
    var translateFooter = globalize.localizeBase("views.footer");
    var translateShared = globalize.localizeBase("views.shared");

    function render(container,val) {
        if (!val) {
            container.empty();
            return false;
        }
        
        if (typeof val === "string") {
            container.html(val);
        }

        return true;
    };

    function Main(el) {
        var el = el;

        this.get = function () {
            return el;
        };
    }

    function Header(container) {
        var el = el;
        var container = container;
        var _visible = false;

        function create() {
            if (el || !container) return;

            //uiGlobal.$body.toggleClass("hide-nav-header", true);
            
            el = $("<div />").addClass("navbar navbar-top");
            uiGlobal.transform(el, "translate3d(0, -100%, 0)");
            container.prepend(el);
        }

        function destroy() {
            if (el) {;
                el.off(".header");
                el.remove();
                el = null;
            }
            _visible = false;
            uiGlobal.toggleClass(uiGlobal.$body, "has-fixed-header", false);
            //uiGlobal.$body.toggleClass("has-fixed-header", false);
        }

        this.view = function (val) {
            // ensure we containers are created
            create();

            var isVisible = render(el, val);
                        
            //  el.toggleClass("hidden", !isVisible);
            uiGlobal.toggleClass(uiGlobal.$body, "has-fixed-header", isVisible);
            //uiGlobal.$body.toggleClass("has-fixed-header", isVisible);
            if (isVisible) {
                this.show();
            } else {
                this.hide();
            }
        }

        this.viewNone = function (val) {
            this.view(null);
        }

        this.viewDefault = function (data) {
            var self = this;

            data = $.extend({},{
                settings: {
                    visible:true
                },
                back: {
                    visible: false,
                    title: translateShared("back")
                },
                refresh: {
                    visible: false
                },
                filter:{
                    visible:true,
                    enabled: preferences.hasOfferFilter(),
                    title: globalize.localize("views.header.filter")
                },
                title: null,
                subtitle: null
            },data);

            if (el) el.off(".viewdefault");
            this.view(templates._headerDefault(data));
           
            if (data.onSettings && el) {
                $(".action-settings", el).hammer({
                   // prevent_default: true
                }).on(uiGlobal.event.tap + ".viewdefault", function (e) {
                    //uiGlobal.touch(e, function (e) {
                        e.preventDefault();
                        data.onSettings(e);
                    //});
                });
            }

            if (data.onBack && el) {
                $(".action-back", el).hammer({
                   // prevent_default: true
                }).on(uiGlobal.event.tap + ".viewdefault", function (e) {
                    //   uiGlobal.touch(e, function (e) {
                        e.preventDefault();
                        data.onBack(e);
                   // });
                });
            }

            if (data.onRefresh && el) {
                $(".action-refresh", el).hammer({
                  //  prevent_default: true
                }).on(uiGlobal.event.tap + ".viewdefault", function (e) {
                    //uiGlobal.touch(e, function (e) {
                        e.preventDefault();
                        data.onRefresh(e);
                    //});
                });
            }

            if (data.onFilter && el) {
                $(".action-filter", el).hammer({
                   // prevent_default: true
                }).on(uiGlobal.event.tap + ".viewdefault", function (e) {
                    //uiGlobal.touch(e, function (e) {
                    e.preventDefault();
                    data.onFilter(e);
                    //});
                });
            }

            if (data.onTitle && el) {
                $(".title", el).hammer({
                  //  prevent_default: true
                }).on(uiGlobal.event.tap + ".viewdefault", function (e) {
                    //  uiGlobal.touch(e, function (e) {
                        e.preventDefault();
                        data.onTitle(e);
                 //   });
                });
            }

            isDefault = true;
        }


        this.viewOkCancel = function (data) {
            data = $.extend({
                okEnabled:true,
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                okCssClass:"primary",
                cancelEnabled:true,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-chevron-inverse",
                cancelCssClass:"",
                titleType: "text",
                titleSearchValue: "",
                titleSearchPlaceholder:"start typing",
                title: "",
                subtitle: "",
                onCancel: function (e) {
                    Backbone.Notifications.trigger("navigateback");
                }
            },data);
            if (el) el.off(".okcancel");
            var self = this;
            this.view(templates._headerOkCancel(data));
            if (data.onCancel && data.cancelEnabled && el) {
                $(".action-cancel", el).hammer({
                   // prevent_default: true
                }).on(uiGlobal.event.tap + ".okcancel", function (e) {
                    // uiGlobal.touch(e, function (e) {
                        e.preventDefault();
                        data.onCancel(e)
                  //  });
                });
            }
            if (data.onOk && data.okEnabled && el) {
                $(".action-ok", el).hammer({
                   // prevent_default: true
                }).on(uiGlobal.event.tap + ".okcancel", function (e) {
                    //uiGlobal.touch(e, function (e) {
                        e.preventDefault();
                        data.onOk(e);
                    //});
                });
            }
        }

        this.viewSaveCancel = function (data) {
            data = $.extend({
                okText: translateShared("accept"),
                okIcon: "icon icon-accept",
            }, data);

            this.viewOkCancel(data);
        }

        this.get = function () {
            return el;
        };

        this.hide = function () {
            var result = $.Deferred();

            if (!el || !_visible) {
                result.resolve();
                return result;
            }
            _visible = false;
            uiGlobal.transform(el, "translate3d(0, 0, 0)");
            container[0].offsetWidth;
            var transition = "transform 0.4s ease";
            result = uiGlobal.transition(el, transition, 400);
            uiGlobal.transform(el, "translate3d(0, -100%, 0)");

            return result;
        }

        this.show = function () {
            var result = $.Deferred();

            if (!el || _visible) {
                result.resolve();
                return result;
            }
            _visible = true;
            uiGlobal.transform(el, "translate3d(0, -100%, 0)");
            container[0].offsetWidth;
            var transition = "transform 0.4s ease";
            result = uiGlobal.transition(el, transition, 400);
            uiGlobal.transform(el, "translate3d(0, 0, 0)");

            return result;
        }

        
        this.searchText = function (val) {
            if (!el) return null;

            var searchElement = $("input[type='search']",el);

            if (typeof val === 'undefined') {
                if (!el) return null;
                return searchElement.val();
            }

            searchElement.val(val||'');
        },
        this.loading = function (val) {
            if (!el) return this;
            uiGlobal.toggleClass(el, "loading", val);
            //el.toggleClass("loading", val);
        },
        this.title = function (val) {
            if (!el) return this;

            $(".title-content", el).html(val);
        }
    }

    function HeaderSec(container) {
        var el = el;
        var container = container;
        var _visible = false;

        function create() {
            if (el || !container) return;

            el = $("<div />").addClass("navbar navbar-top-sec");
            //uiGlobal.transform(el, "translate3d(0, -200%, 0)");
            _visible = false;
            container.append(el);
        }

        function destroy() {
            if (el) {;
                el.off(".headersec");
                el.remove();
                el = null;
                uiGlobal.toggleClass(uiGlobal.$body, "has-fixed-header-sec", false);
                //uiGlobal.$body.toggleClass("has-fixed-header-sec", false);
            }
            _visible = false;
        }

        this._view = function (options) {
            // ensure we containers are created
            create();

            var html = options.render();

            var isVisible = render(el, html);

            if (options.onCreate) options.onCreate(el);
            uiGlobal.toggleClass(uiGlobal.$body, "has-fixed-header-sec", isVisible);
           // uiGlobal.$body.toggleClass("has-fixed-header-sec", isVisible);
            if (isVisible) {
                this.show();
            } else {
                this.hide();
            }
        }

        this.view = function (data) {
            var self = this;

            if (el) el.off(".headersec");
           
            this._view(data);
        }

        this.viewNone = function () {
            this.hide().always(function () {
                destroy();
            });
        }

        this.get = function () {
            return el;
        };

        this.hide = function () {
            var result = $.Deferred();

            if (!el || !_visible) {
                result.resolve();
                return result;
            }
            _visible = false;
            //uiGlobal.transform(el, "translate3d(0, 0, 0)");
            //container[0].offsetWidth;
            //var transition = "transform 0.4s ease";
            //result = uiGlobal.transition(el, transition, 400);
            //uiGlobal.transform(el, "translate3d(0, -300%, 0)");

            el.hide();
            result.resolve();
            return result;
        }

        this.show = function () {
            var result = $.Deferred();
           
            if (!el || _visible) {
                result.resolve();
                return result;
            }
            _visible = true;
            //uiGlobal.transform(el, "translate3d(0, -100%, 0)");
            //container[0].offsetWidth;
            //var transition = "transform 0.4s ease";
            //result = uiGlobal.transition(el, transition, 400);
            //uiGlobal.transform(el, "translate3d(0, 0, 0)");

            el.show();
            result.resolve();
            
            return result;
        }
    }

    function Footer(container) {
        var container = container;
        var el = null;
        var _visible = false;

        function create() {
            if (el || !container) return;

            el = $("<div />").addClass("navbar navbar-justify navbar-bottom");
            container[0].offsetWidth;
        //    uiGlobal.transform(el, "translate3d(0, 100px, 0)");
            _visible = false;
            container.append(el);
            logger.log("Footer create");
        }

        function destroy() {
            logger.log("Footer destroy");
            if (el) {;
                el.off(".footer");
                el.remove();
                el = null;
                uiGlobal.toggleClass(uiGlobal.$body, "has-fixed-footer", false);
                //uiGlobal.$body.toggleClass("has-fixed-footer", false);
            }
            _visible = false;
        }
          
        this.view = function (val) {
            // ensure we containers are created
            
            create();

            var isVisible = render(el, val);
                       
            //   el.toggleClass("hidden", !isVisible);
            uiGlobal.toggleClass(uiGlobal.$body, "has-fixed-footer", isVisible);
           // uiGlobal.$body.toggleClass("has-fixed-footer", isVisible);

            if (isVisible) {
                var elements = $("[data-href]", el);
                elements.off(".footer").hammer({
                   // prevent_default: true
                }).on(uiGlobal.event.tap + ".footer", function (e) {
                    e.preventDefault();
                    elements.removeClass("active");
                    helper.handleNavigation($(this));
                });
            }

            if (isVisible) {
                this.show();
            } else {
                this.hide();
            }

            return this;
        }

        this.viewNone = function () {
            this.hide().always(_.bind(function () {
                this.view(null);
            },this));

            return this;
        }

        this.viewDefault = function (data,options) {
           // el.off(".viewdefaultfooter");

            if (typeof data === "undefined") {
                var resolveClass = function (o) {
                    var cssClass = "";
                    if (o.href == "#me/offer/create") cssClass = "primary";
                    
                    return cssClass+ " " + ((window.location.hash == o.href) ? "active" : "");
                }

                data = {
                    items: [
                   
                  //  { href: "#", icon: "icon-home", label: translateFooter("home"), cssClass: resolveClass },
                    { href: "#offer/category", label: translateFooter("offer-category"), icon: "icon-category", cssClass: resolveClass },
                    { href: "#offer/search", label: translateFooter("offer-search"), icon: "icon-offer-list", cssClass: resolveClass },
                 //   { href: "#offer/favorites", icon: "icon-offer-favorite", label: translateFooter("offer-favorite"), cssClass: resolveClass },
                    { href: "#me/offer/create", icon: "icon-offer-create", label: translateFooter("offer-create"), cssClass: resolveClass },
                    ]
                };
            }

            this.view(templates._footerDefault(data), options);

            return this;
        }

        this.get = function () {
            return el;
        };

        this.hide = function () {
            var result = $.Deferred();
            logger.log("Footer hide");
            if (!el || !_visible) {
                result.resolve();
                return result;
            }
            _visible = false;
            el.hide();
            result.resolve();
            //uiGlobal.transform(el, "translate3d(0, 0, 0)");
            //container[0].offsetWidth;
            //var transition = "transform 0.4s ease";
            //result = uiGlobal.transition(el, transition, 400);
            //uiGlobal.transform(el, "translate3d(0, 100px, 0)");

            return result;
        }

        this.show = function () {
            var result = $.Deferred();
            logger.log("Footer show");
            if (!el || _visible) {
                result.resolve();
                return result;
            }
            _visible = true;
            el.show();
            result.resolve();

            //uiGlobal.transform(el, "translate3d(0, 100px, 0)");
            //container[0].offsetWidth;
            //var transition = "transform 0.4s ease";
            //result = uiGlobal.transition(el, transition, 400);
            //uiGlobal.transform(el, "translate3d(0, 0, 0)");

            return result;
        }
    }

    function Snapper(main,left,right) {
        var inst = null;
        var contentMain = main;
        var left = left;
        var right = right;

       
        
        this.init = function (options) {
            options = $.extend({
                element: contentMain[0],
                hyperextensible: false,
                touchToDrag: false,
                tapToClose: true
                //transitionSpeed:3
            }, options);
            inst = new Snap(options);
            
        }

        this.settings = function (o) {
            if (inst) {
                inst.settings(o);
            }
        }

        this.viewLeft = function (val) {
            var panel = left;
            if (globalize.culture().isRTL) {
                panel = right;
            }

            render(panel, val);
            if (panel) {
                var elements = $("[data-href]", panel);
                elements.off(".snap-left").hammer({
                  //  prevent_default: true
                }).on(uiGlobal.event.tap + ".snap-left", function (e) {
                    e.preventDefault();
                    helper.handleNavigation($(this));
                });
            }

            return panel;
        }

        this.viewRightDefault = function () {
            // no template
            return this.viewRight(null);
        }

        this.viewRight = function (val) {
            var panel = right;
            if (globalize.culture().isRTL) {
                panel = left;
            }

            render(panel, val);

            if (panel) {
                var elements = $("[data-href]", panel);
                elements.off(".snap-right").hammer({
                  //  prevent_default: true
                }).on(uiGlobal.event.tap + ".snap-right", function (e) {
                    e.preventDefault();
                    helper.handleNavigation($(this));
                });
            }

            return panel;
        }

        this.viewLeftDefault = function (data) {

            data = $.extend({
                isAuthenticated: api.isAuthenticated(),
                translate:translateSideMenu
            }, data);

            var html = templates._snapPrimary(data);

           return this.viewLeft(html);
        }

        this.isOpen = function () {
            if (!inst) return false;
            var state = inst.state();

            if (state && (state.state == 'right' || state.state == 'left')) return true;

            return false;
        }

        this.close = function () {
            var deferred = $.Deferred();
            if (!inst) return deferred.resolve();

            var state = inst.state();
            
            if (state && state.state == 'closed') return deferred.resolve();
            
            inst.on("animated", function () {
                deferred.resolve();
                if (inst) inst.off("animated");
            });
            inst.close();

            return deferred;
        }
        this.openLeft = function () {

            var deferred = $.Deferred();
            if (!inst) return deferred.resolve();

            var state = inst.state();

            inst.on("animated", function () {
                deferred.resolve();
                if (inst) inst.off("animated");
            });

            if (globalize.culture().isRTL) {
                if (state && state.state == 'right') return deferred.resolve();
                inst.open('right');
            } else {
                if (state && state.state == 'left') return deferred.resolve();
                inst.open('left');
            }

            return deferred;
        }
        this.closeLeft = function () {
            var deferred = $.Deferred();
            if (!inst) return deferred.resolve();

            var state = inst.state();
            if (state && state.state == 'closed') return deferred.resolve();

            inst.on("animated", function () {
                deferred.resolve();
                if (inst) inst.off("animated");
            });
            if (globalize.culture().isRTL) {
                inst.close('right');
            } else {
                inst.close('left');
            }

            return deferred;
        }
        this.toggleLeft = function () {
            if (!inst) return $.Deferred().resolve();
            if ((inst.state().state == "left" && !globalize.culture().isRTL) || (inst.state().state == "right" && globalize.culture().isRTL)) {
                return this.closeLeft();
            } else {
                return this.openLeft();
            }
        }
        this.openRight = function () {
            var deferred = $.Deferred();
            if (!inst) return deferred.resolve();

            var state = inst.state();

            inst.on("animated", function () {
                deferred.resolve();
                if (inst) inst.off("animated");
            });
            if (globalize.culture().isRTL) {
                if (state && state.state == 'left') return deferred.resolve();
                inst.open('left');
            } else {
                if (state && state.state == 'right') return deferred.resolve();
                inst.open('right');
            }

            return deferred;
        }
        this.closeRight = function () {
            var deferred = $.Deferred();
            if (!inst) return deferred.resolve();

            var state = inst.state();
            if (state && state.state == 'closed') return deferred.resolve();

            inst.on("animated", function () {
                deferred.resolve();
                if (inst) inst.off("animated");
            });
            if (globalize.culture().isRTL) {
                inst.close('left');
            } else {
                inst.close('right');
            }

            return deferred;
            
        }
        this.toggleRight = function () {
            if (!inst) return $.Deferred().resolve();
            if ((inst.state().state == "right" && !globalize.culture().isRTL) || (inst.state().state == "left" && globalize.culture().isRTL)) {
                return this.closeRight();
            } else {
                return this.openRight();
            }
        }
    }

    var inst = {
        main: null,
        header: null,
        headerSec:null,
        footer: null,
        snap: null
    }

    inst.init = function () {
        
        this.main= new Main(uiGlobal.$content);
        this.header = new Header(uiGlobal.$header);
        this.headerSec = new HeaderSec(uiGlobal.$header);
        this.footer= new Footer(uiGlobal.$footer);
        this.snap = new Snapper(uiGlobal.$snapContent, uiGlobal.$snapDrawerLeft, uiGlobal.$snapDrawerRight);

        this.snap.init({
            addBodyClasses: true,
            hyperextensible: true,
            resistance: 0.5,
            flickThreshold: 50,
            transitionSpeed: 0.3,
            easing: 'ease',
            maxPosition: 266,
            minPosition: -266,
            tapToClose: true,
            touchToDrag: true,
            slideIntent: 10, // degrees
            minDragDistance: 5,
            disable:'left'
        });

    }
    
    return inst;
    
}
);


