﻿define(["jquery", "underscore", "template", 'uiGlobal', 'uiModal', 'globalize', 'logger'],
        function ($, _, template, uiGlobal, uiModal, globalize, logger) {

            var itemTemplate = _.template("<a data-val=\"<%=val%>\"><i class=\"icon icon-chevron\"></i><%=text%></a>");

            function EditorLocation(options) {
                var container;
                var LEVEL_REGION = 1;
                var LEVEL_CITY = 2;
                var LEVEL_NEIGHBOURHOOD = 3;
                var data = null;
                var selected = null;
                var valueContainer = null;
                var levelEnabled = LEVEL_NEIGHBOURHOOD;

                options = $.extend({}, {
                    template:template.controls.editorList
                },options);

                function render(renderData,o) {
                    data = $.extend({}, {
                        required: false,
                        label: globalize.localize("views.shared.select-location.message"),
                        icon: null
                    }, renderData);

                    var html = options.template(data);

                    if (o) {
                        var el = $(html);
                        o.append(el);
                        create(o);
                    }

                    return html;
                }

                function create(o) {
                    container = o;
                    container.hammer({
                       // prevent_default: true
                    }).on(uiGlobal.event.tap + ".editorlocation", "a", function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                      //  uiGlobal.touch(e, function (e) {
                            var trigger = $(e.target);
                            var level = trigger.data("val");
                            open(level);
                       // });
                    });
                    valueContainer = $("[data-role='text']", container);
                }

                function open(level) {
                    logger.log(level);
                    var listData = null;
                    if (level == LEVEL_REGION) {
                        // nothing
                    }
                    if (level == LEVEL_CITY) {
                        listData = {
                            region: selected.region.id
                        }
                    }
                    if (level == LEVEL_NEIGHBOURHOOD) {
                        listData = {
                            city: selected.city.id
                        }
                    }

                    var modalData = function () {
                        var deferred = $.Deferred();
                        data.data(listData).done(function (data) {
                            var result = {
                                list: [],
                                source: data
                            }

                            if (level == LEVEL_REGION) {
                                result.list = $.map(data.country.children, function (o, i) {
                                    return {
                                        id: o.id,
                                        name: o.name,
                                        childrenCount: o.childrenCount,
                                        selected: (selected && selected.region && o.id == selected.region.id)
                                    }
                                })
                            }
                            if (level == LEVEL_CITY) {
                                result.list = $.map(data.region.children, function (o, i) {
                                    return {
                                        id: o.id,
                                        name: o.name,
                                        childrenCount: o.childrenCount,
                                        selected: (selected && selected.city && o.id == selected.city.id)
                                    }
                                })
                            }
                            if (level == LEVEL_NEIGHBOURHOOD) {
                                result.list = $.map(data.city.children, function (o, i) {
                                    return {
                                        id: o.id,
                                        name: o.name,
                                        childrenCount: o.childrenCount,
                                        selected: (selected && selected.neighbourhood && o.id == selected.neighbourhood.id)
                                    }
                                })
                            }

                            deferred.resolve(result);
                        });

                        return deferred;
                    }

                    uiModal.openSelectList({
                        type: "single",
                        title: data.listTitle,
                        data: modalData,
                        onSelect: function (data,source) {
                            select(data,level,source);
                        }
                    });
                }

                function select(value, level, source) {
                    // preprocess selected
                    var location = selected || {};
                    if (level == LEVEL_REGION) {
                        
                        if (value && selected && selected.region && value.id != selected.region.id) {
                            location.city = null;
                            location.neighbourhood = null;
                        }
                        location.region = value;
                    }
                    if (level == LEVEL_CITY) {

                        if (value && selected && selected.city && value.id != selected.city.id) {
                            location.neighbourhood = null;
                        }
                        location.city = value;
                    }
                    if (level == LEVEL_NEIGHBOURHOOD) {
                        location.neighbourhood = value;
                    }

                    if (data.onSelect && data.onSelect(location, level) === false) return;
                    val(location);
                }

                function destroy() {
                    // unbind stuff
                    if (container) {
                        container.off(".editorlocation");
                    }
                    data = null;
                    valueContainer = null;
                }

                function val(value) {
                    if (typeof value === 'undefined') return selected;

                    selected = value;
                    renderVal(value);
                }

                function renderVal(value) {
                    valueContainer.empty();
                    var arr = [];

                    if (!value) {
                        arr.push({
                            text: globalize.localize("views.shared.select-region.message"),
                            val:LEVEL_REGION
                        });
                    } else {
                        if (value.region) {
                            arr.push({
                                text: value.region.name,
                                val: LEVEL_REGION,
                                childrenCount:value.region.childrenCount
                            });
                        }
                        if (value.city) {
                            arr.push({
                                text: value.city.name,
                                val: LEVEL_CITY,
                                childrenCount: value.city.childrenCount
                            });
                        }
                        if (value.neighbourhood) {
                            arr.push({
                                text: value.neighbourhood.name,
                                val: LEVEL_NEIGHBOURHOOD,
                                childrenCount:0
                            });
                        }
                    }
                    
                    if (arr.length > 0) {
                        var last = arr[arr.length - 1];
                        if (hasMore(value,last.val)) {
                            arr.push({
                                val: last.val + 1,
                                text: globalize.localize("views.shared.select.message"),
                            });
                        }
                    }

                    $.each(arr, function (i, o) {
                        if (o.val <= levelEnabled) {

                            var el = $(itemTemplate(o));

                            valueContainer.append(el);
                        }
                    });
                }

                function enable(level) {
                    if (typeof level === 'undefined') return levelEnabled;
                    if (levelEnabled == level) return;
                    levelEnabled = level;
                    renderVal(selected);
                }

                function hasMore(val, level) {
                    if (level == LEVEL_REGION) {
                        return (val && val.region && val.region.childrenCount && val.region.childrenCount > 0 && level <= levelEnabled);
                    }
                    if (level == LEVEL_CITY) {
                        return (val && val.city && val.city.childrenCount && val.city.childrenCount > 0 && level <= levelEnabled);
                    }
                    if (level == LEVEL_NEIGHBOURHOOD) {
                        return (val && val.neighbourhood && val.neighbourhood.childrenCount && val.neighbourhood.childrenCount > 0 && level <= levelEnabled);
                    }
                    
                    return false;
                }

                this.render = render;
                this.create = create;
                this.destroy = destroy;
                this.enable = enable;
                this.val = val;
                this.hasMore = hasMore;
                this.container = function () {
                    return container;
                }
                this.LEVEL_REGION = LEVEL_REGION;
                this.LEVEL_CITY = LEVEL_CITY;
                this.LEVEL_NEIGHBOURHOOD = LEVEL_NEIGHBOURHOOD;
            }

            return EditorLocation;
        }
);