﻿define(["jquery", "underscore","template",'uiGlobal','uiModal','helper','globalize'],
        function ($, _, template, uiGlobal, uiModal, helper,globalize) {

            var itemTemplate = _.template("<a data-val=\"<%=val%>\"><i class=\"icon icon-chevron\"></i><%=text%></a>");

            function EditorOfferCategory(options) {
                var container;
                var data = null;
                var selected = null;
                var valueContainer = null;

                options = $.extend({}, {
                    template:template.controls.editorList
                },options);

                function render(renderData,o) {
                    data = $.extend({}, {
                        required: false,
                        label: "Category",
                        icon: null,
                        listTitle: globalize.localize("views.shared.select-category.message"),
                        listSubtitle:""
                    }, renderData);

                    var html = options.template(data);

                    if (o) {
                        var el = $(html);
                        o.append(el);
                        create(o);
                    }

                    return html;
                }

                function create(o) {
                    container = o;
                    container.hammer({
                       // prevent_default: true
                    }).on(uiGlobal.event.tap + ".editorcategory", "a", function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                      //  uiGlobal.touch(e, function (e) {
                            var trigger = $(e.target);
                            var level = trigger.data("val");
                            open(level);
                      //  });
                    });
                    valueContainer = $("[data-role='text']", container);
                }

                function open(val) {
                    var arr = getArray(selected);
                    var item = getById(selected, val);
                    var parent = (item) ? item.parent : null;
                    if (!parent && !val) parent = selected;
                    var id = (parent) ? parent.id : null;
                    var selectedIds = _.pluck(arr, "id");
                    var modalData = function () {
                        var deferred = $.Deferred();
                        data.data(id).done(function (data) {
                            var result = {
                                list: [],
                                source: data
                            }

                            if (data && data.length > 0) {
                                result.list = $.map(data[data.length - 1].children, function (o, i) {

                                    return {
                                        id: o.id,
                                        name: o.name,
                                        icon:helper.toCategoryCssClass(o),
                                        effectiveMetaData:o.effectiveMetaData,
                                        childrenCount: o.childrenCount,
                                        selected: _.contains(selectedIds, o.id)
                                    }
                                })
                            }

                            deferred.resolve(result);
                        });

                        return deferred;
                    }

                    var title = data.listTitle;
                    var subtitle = data.listSubtitle;
                    if (_.isFunction(title)) {
                        title = title(parent);
                    }
                    if (_.isFunction(subtitle)) {
                        subtitle = subtitle(parent);
                    }

                    uiModal.openSelectList({
                        type: "single",
                        title: title,
                        subtitle: subtitle,
                        data: modalData,
                        onSelect: function (data,source) {
                            select(data,parent,source);
                        }
                    });
                }

                function select(value, parent, source) {
                    if (value) {
                        value.parent = parent;
                    }

                    if (data.onSelect && data.onSelect(value) === false) return;
                    val(value);
                }

                function destroy() {
                    // unbind stuff
                    if (container) {
                        container.off(".editorcategory");
                    }
                    data = null;
                    valueContainer = null;
                }

                function val(value) {
                    if (typeof value === 'undefined') return selected;

                    selected = value;
                    renderVal(value);
                }

                function renderVal(value) {
                    valueContainer.empty();

                    var arr = getArray(value);

                    if (!arr || arr.length == 0) {
                        var el = $(itemTemplate({
                            text: globalize.localize("views.shared.select-category.message"),
                            val:null
                        }));

                        valueContainer.append(el);
                    } else {

                        // if last has children, add empty category
                        var last = arr[arr.length - 1];
                        if (last.childrenCount > 0) {
                            arr.push({
                                id: null,
                                name: "Select category in " + last.name
                            });
                        }

                        $.each(arr, function (i, o) {
                            var el = $(itemTemplate({
                                text: o.name,
                                val:o.id
                            }));

                            valueContainer.append(el);
                        });


                    }
                }

                function hasMore(val) {
                    if (!val) return true;
                    if (val.childrenCount > 0) return true;

                    return false;
                }

                function getArray(data) {
                    var parent = null;
                    var arr = [];

                    if (!data) {
                        return arr;
                    }

                    var cat = data;
                    while (cat != null) {

                        arr.push({
                            id: cat.id,
                            name: cat.name,
                            childrenCount: cat.childrenCount
                        });

                        cat = cat.parent;
                    }

                    arr.reverse();

                    return arr;
                }

                function getById(data,id) {
                    var parent = null;

                    if (!data || !id) {
                        return null;
                    }
                  
                    var cat = data;
                    while (cat != null) {
                        if (cat.id == id) return cat;
                        cat = cat.parent;
                    }

                    return null;
                }

                function getLastNonEmpty(arr, fromIndex) {
                    if (!arr || arr.length == 0) return null;
                    if (typeof fromIndex === "undefined") fromIndex = arr.length - 1;
                    if (fromIndex < 0) return null;
                    for (var i = fromIndex; i >= 0; i--) {
                        if (arr[i].id) return arr[i];
                    }
                    return null;
                }

                this.render = render;
                this.create = create;
                this.destroy = destroy;
                this.val = val;
                this.hasMore = hasMore;
                this.container = function () {
                    return container;
                }
            }

            return EditorOfferCategory;
        }
);