﻿define(["jquery", "underscore", "template", 'uiGlobal', 'uiControlEditorOfferImage', 'api', 'logger'],
        function ($, _, template, uiGlobal, uiEditorOfferImage, api, logger) {

            function EditorOfferImageGallery(options) {
                var container;
                var self = this;
                var data = null;
                var imageControls = [];

                function createIn(o,options) {
                    data = $.extend({}, {
                        width: 90,
                        height:90
                    }, options);
                    
                    container = o;
                }

                function onSelectionChanged() {
                    if (data.onSelectionChanged) {
                        data.onSelectionChanged(selected());
                    }
                }

                function selected(items) {
                    if (typeof items === 'undefined') {
                        return _.filter(imageControls, function (o) {
                            return o.selected();
                        });
                    }

                    _.each(imageControls, function (o) {
                        var sel = false;
                        
                        sel = _.some(items||[], function (oo) {
                            return (oo == o);
                        });
                        o.selected(sel);
                    });

                    return this;
                }

                function primary(item) {
                    if (typeof item === 'undefined') {

                        var items = _.filter(imageControls, function (o) {
                            return o.primary();
                        });
                        if (items.length == 0) return null;
                        return items[0];
                    }

                    if (_.isArray(item)) item = item[0];

                    _.each(imageControls, function (o) {
                        o.primary(o == item);
                    });

                    return this;
                }

                function add(opt){
                    var imgControl = new uiEditorOfferImage();

                    imgControl.render({
                        key: opt.key,
                        loading:opt.loading||false,
                        primary:opt.primary||false,
                        width: data.width,
                        height: data.height,
                        contentType:opt.contentType,
                        onSelectionChanged: onSelectionChanged
                    }, container);

                    imageControls.push(imgControl);

                    return imgControl;
                }
                

                function load(images) {
                    if (!images) return;
                    for (var i = 0; i < images.length; i++) {
                        var img = images[i];

                        add(img);
                    }
                }

                function upload(file) {
                    //logger.log("Uploading file type: " + file.type +" size: " +file.size);
                    var fileType = file.type || "image/jpeg";
                   // alert("Uploading file type: " + fileType + " size: " + file.size);
                    var imgControl =  add({
                        loading: true,
                        contentType: fileType,
                        primary: (imageControls.length ==0)
                    });

                    var deferred = $.Deferred();
                    
                    var handler = api.me.offer.imageUpload;
                    if (typeof FileTransfer !== 'undefined') handler = api.me.offer.imageUploadAlt;
                    handler(file, {
                        fileProgress: function (e) {
                            if (e.lengthComputable) {
                                //var percentComplete = (e.loaded / e.total) * 100;
                                // logger.log(percentComplete);
                            }
                        }
                    }).done(function (o) {
                        var imgUrl = imgControl.imageUrl(o.key);
                        if (!imgUrl) return deferred.reject(null);
                        logger.log("Done uploading file: " + imgUrl);
                       // alert("Done uploading file: " + imgUrl);
                        api.image.preload(imgUrl).done(function () {
                            logger.log("Done preloading image file: " + imgUrl);
                            deferred.resolve(o.key, imgUrl);
                        }).fail(function (data) {
                            deferred.reject(data);
                        });

                    }).fail(function (data) {
                        //logger.error("Error uploading file: " + data);
                        //alert("Error uploading file: " + data);
                        deferred.reject(data);
                    });                                       

                    deferred.done(function (key, url) {
                        imgControl.loading(false);
                        imgControl.key(key);
                    }).fail(function (data) {
                        //logger.error("Error uploading image file: " + data);
                        //alert("Error uploading image file: " + data);
                        imgControl.loading(false);
                        imgControl.invalid();
                    });

                    return deferred;
                }

                function remove(items) {
                    if (!items) return this;
                    if (!_.isArray(items)) {
                        remove([items]);

                        return this;
                    }

                    var hasPrimary = false;
                    _.each(items, function (o) {
                        if (o.primary()) hasPrimary = true;
                        o.remove();
                    });
                    
                    imageControls = _.reject(imageControls, function (o) {
                        return _.contains(items, o);
                    });
                    if (hasPrimary && imageControls.length > 0) {
                        imageControls[0].primary(true);
                    }

                    return this;
                }

                function val(value) {
                    if (typeof value === 'undefined') {

                        return $.map(imageControls, function (o) {
                            if (!o || o.invalid() || o.loading()) return null;

                            var data = o.data();

                            return {
                                key:o.key(),
                                primary: o.primary(),
                                contentType: data.contentType
                            }
                        });
                    }

                    load(value);
                }

                function destroy() {
                    // unbind stuff
                    if (imageControls) {
                        $.each(imageControls, function (i, o) {
                            if (o && o.destroy) {
                                o.destroy();
                            }
                        });
                    }
                }

                this.createIn = createIn;
                this.load = load;
                this.upload = upload;
                this.destroy = destroy;
                this.primary = primary;
                this.selected = selected;
                this.remove = remove;
                this.val = val;
            }

            return EditorOfferImageGallery;
        }
);