﻿define(["jquery", "underscore", "template", 'uiGlobal', 'api', 'logger'],
        function ($, _, template, uiGlobal, api, logger) {

            var itemTemplate = _.template("<a class=\"<%-cssClass%>\" style=\"width:<%-width%>px;height:<%-height%>px\"><img src=\"<%-src%>\" width=\"<%-width%>\" height=\"<%-height%>\" /><i class=\"icon mark\"></i><i class=\"icon mark-alt\"></i></a>");

            function EditorOfferImage(options) {
                var container;
                var self = this;
                var data = null;
                var imgEl = null;
                var el = null;

                function render(options, container) {
                    data = $.extend({}, {
                        selected: false,
                        loading: false,
                        primary: false,
                        key:null,
                        width: null,
                        height: null,
                        invalid:false
                    }, options);

                    var renderData = {
                        width: data.width,
                        height: data.height,
                        src: getImageUrl(),
                        cssClass: (data.loading ? "loading" : "") + (data.selected ? " selected" : "") + (data.primary ? " primary" : "")
                    }

                    var html = itemTemplate(renderData);

                    if (container) {
                        el = $(html);
                        container.append(el);
                        create(container);
                    }

                    return html;
                }

                function getImageUrl(key) {
                    if (typeof key === "undefined" && data) key = data.key;
                    if (!key || !data) return null;

                    return api.offer.imageUrl(key, {
                        width: data.width,
                        height: data.height
                    });
                }

                function create(o, generatedEl) {
                    if (generatedEl && !el) el = generatedEl;
                    container = o;
                    el.hammer({
                      //  prevent_default: true
                    }).on(uiGlobal.event.tap + ".uiofferimage", function (e) {
                        e.preventDefault();
                        e.stopPropagation();
                      //  uiGlobal.touch(e, function (e) {
                            selected(!data.selected);
                      //  });
                    });
                    imgEl = $("img", el);
                }

                function destroy() {
                    // unbind stuff
                    logger.log("img destroy: " + data.key);
                    if (el) {
                        el.off(".uiofferimage");
                        el.remove();
                    }
                    data = null;
                    el = null;
                    imgEl = null;
                }

                function selected(value) {
                    if (!data) return false;
                    if (typeof value === 'undefined') return data.selected;

                    if (data.loading) return;
                    data.selected = value;
                    el.toggleClass("selected", value);
                    if (data.onSelectionChanged) {
                        data.onSelectionChanged(this);
                    }
                }

                function loading(value) {
                    if (!data) return false;
                    if (typeof value === 'undefined') return data.loading;

                    data.loading = value;
                    el.toggleClass("loading", value);
                }

                function primary(value) {
                    if (!data) return false;
                    if (typeof value === 'undefined') return data.primary;

                    data.primary = value;
                    el.toggleClass("primary", value);
                }

                function key(value) {
                    if (!data) return false;
                    if (typeof value === 'undefined') return data.key;

                    data.key = value;
                    imgEl.attr("src",getImageUrl());
                }

                function invalid(value) {
                    if (!data) return false;
                    if (typeof value === 'undefined') return data.invalid;

                    data.invalid = value;
                }

                function remove() {
                    this.destroy();
                }

                this.render = render;
                this.create = create;
                this.destroy = destroy;
                this.selected = selected;
                this.loading = loading;
                this.primary = primary;
                this.key = key;
                this.imageUrl = getImageUrl;
                this.invalid = invalid;
                this.remove = remove;
                this.data = function () { return data; };
            }

            return EditorOfferImage;
        }
);