﻿define([
  'jquery',
  'underscore',
  'template',
  'globalize',
  'uiControlBase',
  'uiGlobal',
  'uiContainers'
], function ($, _,template, globalize,uiControlBase, uiGlobal, uiContainers) {

    var TileControl = uiControlBase.extend({
        template: template.controls.tile,
        _loading: false,
        _value: null,
        contentEl: null,
        _contentHtml: null,
        _cssClass: null,
        _validationErrors:null,
        getRenderData: function () {

            return {
                cssClass: this._parseCssClass(),
                mediaIcon: this._parseMediaIcon(),
                content: this._parseContentHtml(),
                icon: this.options.icon || "icon mark"
            }
        },
        onBeforeDestory: function () {
            if (this.el) {
                this.el.off(uiGlobal.event.tap);
            }
            this.contentEl = null;
            this._value = null;
        },
        onRender: function () {
            if (!this.el) return this;
            var self = this;

            this.el.hammer({
                // prevent_default: false
            }).on(uiGlobal.event.tap, function (e) {
                e.preventDefault();
                e.stopPropagation();

                self.trigger("select", self,self.value());
            });

            this.contentEl = $(".tile-content", this.el);
        },
        create: function (container) {
            if (this.created()) this.destroy();
            this.createIn({}, container);
        },
        loading: function (val) {
            if (typeof val === "undefined") return this._loading;

            this._loading = !!val;

            if (this.created()) {
                this.renderContent();
                this.renderCssClass();
            }
        },
        renderContent: function () {
            if (this.contentEl) {
                var html = this._parseContentHtml();
                if (html != this._contentHtml) {
                    this._contentHtml = html;
                    this.contentEl.html(html);
                }
            }
        },
        renderCssClass: function () {
            if (this.el) {
                var cssClass = this._parseCssClass();
                if (cssClass != this._cssClass) {
                    this._cssClass = cssClass;
                    this.el.attr("class", "tile " + cssClass);
                }
            }
        },
        renderMediaIcon: function () {
            if (this.el) {
                var mediaIcon = this._parseMediaIcon();
                if (mediaIcon != this._mediaIcon) {
                    this._mediaIcon = mediaIcon;

                    var html = "<i class=\""+mediaIcon+" hidden-loading\"></i><i class=\"icon icon-spinner spin visible-loading\"></i>";
                    $(".tile-media", this.el).html(html);
                }
            }
        },
        value: function (val, validate) {
            if (typeof val === 'undefined') {
                return this._value;
            }
            this._value = val;
            if (validate) {
                this._validate();
            }
            this.renderContent();
            this.renderCssClass();
            this.renderMediaIcon();

            this.trigger("changed", self);
        },
        cssClass: function (val) {
            if (typeof val === 'undefined') {
                return this._cssClass;
            }
            this._cssClass = val;
            this.renderContent();
            this.renderCssClass();
        },
        _parseContentHtml: function () {
            var result = null;
            if (this.options && this.options.content) {
                result = this.options.content.call(this, this.value());
            }
            if (this.isValid() === false) {
                _.each(this._validationErrors, function (o) {
                    result = result + "<div class=\"error\">" + o + "</div>";
                });
            }

            return result;
        },
        _parseCssClass: function () {
            var val = "";
            if (this.options && this.options.cssClass) {
                val = this.options.cssClass.call(this, this.value());
            }

            if (this.isValid() === false) {
                val = val + " has-error";
            }
            if (this.isValid() === true) {
                val = val + " success";
            }

            if (this.loading()) val = val + " loading";

            return val;
        },
        _parseMediaIcon: function () {
            var val = "";
            if (this.options && this.options.mediaIcon) {
                if (_.isFunction(this.options.mediaIcon)) {
                    val = this.options.mediaIcon.call(this, this.value());
                } else {
                    val = this.options.mediaIcon;
                }
            }

            return val;
        },
        _validate: function () {
            var val = null;
            if (this.options && this.options.validate) {
                val = this.options.validate.call(this, this.value());
                this._setValidationErrors(val);
            }
        },
        _setValidationErrors: function (errors) {
            if (errors) {
                if (!_.isArray(errors)) errors = [errors];
                this._validationErrors = errors;
            } else {
                this._validationErrors = [];
            }
        },
        isValid:function(){
            if (this._validationErrors == null) return null;
            return this._validationErrors.length == 0;
        },
        setValidationErrors:function(errors){
            this._setValidationErrors(errors);
            this.renderContent();
            this.renderCssClass();
        },
        clearValidationErrors:function(){
            this.setValidationErrors(false);
        }
    });

    return TileControl;
});