﻿define([
  'jquery',
  'underscore',
  'globalize',
  'template',
  'uiGlobal',
  'uiContainers',
  'uiControlBase',
  'uiControlButton',
  'uiScroll',
  'logger'
], function ($, _, globalize, template, uiGlobal, uiContainers, uiControlBase, uiControlButton, uiScroll, logger) {

    var MoreButtonOld = uiControlBase.extend({
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                visible: false,
                loading: false,
                cssClass: "more-link"
            }, this.options);
        },
        template: template.controls.button,
        getRenderData: function () {
            return $.extend({}, {
                icon: "icon icon-reload",
                title: "more"
            }, this.options);
        },
        onBeforeRender: function (el, html, renderData) {
          //  el.attr("class", "more-link");
            if (!this.visible()) {
                el.hide();
            }
            if (this.loading()) {
                this.renderLoadingOn();
            } else {
                this.renderLoadingOff();
            }
        },
        visible: function (val) {
            if (typeof val === 'undefined') return this.options.visible;

            this.options.visible = val;
            if (!this.el) return this;
            if (val) {
                this.el.show();
            } else {
                this.el.hide();
            }

            return this;
        },
        loading: function (val) {
            if (typeof val === 'undefined') return this.options.loading;

            this.options.loading = val;
            if (val) {
                if (!this.visible()) this.visible(true);
                this.renderLoadingOn();
            } else {
                this.renderLoadingOff();
            }

            return this;
        },
        renderLoadingOn: function () {
            if (!this.el) return this;
            this.unbindEvents();
            this.el.html("<i class=\"icon icon-spinner spin\" />");
        },
        renderLoadingOff: function () {
            if (!this.el) return this;
            var html = this.template(this.getRenderData());
            this.unbindEvents();
            //var el = $(html);

            //this.el.html(el.html());
         //   this.el.empty();
            this.el.html(html);
            this.bindEvents();
        },
        onRender: function () {
            this.unbindEvents();
            this.bindEvents();
        },
        onBeforeDestory: function () {
            this.unbindEvents();
        },
        bindEvents: function () {
            if (!this.el) return this;
            var self = this;
            this.el.hammer({
               // prevent_default: true
            }).on(uiGlobal.event.tap + ".morebutton", function (e) {
                e.preventDefault();
                e.stopPropagation();
                if (!self.loading() && self.visible())
                 //   uiGlobal.touch(e, function () {
                        self.trigger("select", self);
                //    });
            });
        },
        unbindEvents: function () {
            if (this.el) {
                this.el.off(".morebutton");
            }
        }
    });

    var MoreButton = uiControlBase.extend({
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                visible: false,
                loading: false
            }, this.options);
        },
        visible: function (val) {
            if (typeof val === 'undefined') return this.options.visible;

            this.options.visible = val;
            
            return this;
        },
        loading: function (val) {
            if (typeof val === 'undefined') return this.options.loading;

            this.options.loading = val;
            if (val) {
                if (!this.visible()) this.visible(true);
                this.renderLoadingOn();
            } else {
                this.renderLoadingOff();
            }

            return this;
        },
        renderLoadingOn: function () {
            uiContainers.header.loading(true);
        },
        renderLoadingOff: function () {
            uiContainers.header.loading(false);
        },
        created: function () {
            return true;
        },
        createIn: function (options, container) {
            if (this.loading()) {
                this.renderLoadingOn();
            } else {
                this.renderLoadingOff();
            }
            
            return this;
        }
    });

    var ViewList = uiControlBase.extend({
        template: _.template("<div class=\"list\" />"),
        _controlContainer: [],
        _moreButton: null,
        list: [],
        _hasMore: false,
        _lastTop: 0,
        _scroller: null,
        _moreBottomOffset: 0,
        _pullToRefreshTimer:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            var self = this;
            this.options = $.extend({}, {
                enableMore: true,
                hideFooter: false,
                hideHeader: false
            }, options);

            if (this.options.container) this.container = this.options.container;
            if (this.options.template) this.template = this.options.template;
            this.createItem = this.options.createItem || this.createItem;
            this.more = this.options.more || this.more;
            this._moreBottomOffset = Math.floor(uiGlobal.dimensions.body.height * 0.8);

           // if (this.more) this.options.enableMore = true;

            this.initializeMoreButton();
        },
        initializeMoreButton:function(){
            if (this.options.enableMore) {
                var self = this;
                this._moreButton = new MoreButton();
                

                this.registerControl(this._moreButton);
                this._moreButton.on("select",_.bind(this._more,this));
            }
        },
        _more:function(){
            if (this.more && this._hasMore && this._moreButton) {
                this._moreButton.loading(true);
                var self = this;
                return this.more().always(function () {
                    if (self._moreButton) self._moreButton.loading(false);
                });
            }

            return null;
        },
        getRenderData: function () {
            return this.options;
        },
        onRender: function () {
            
            if (this._moreButton && !this._moreButton.created()) {
                this._moreButton.createIn({}, this.container);
            }

            var self = this;
            if (this._controlContainer) {
                _.each(this._controlContainer, function (o) {
                    if (o && o.created && o.created()) return;
                    o.createIn({}, self.el);
                });
            }

            // start listening to scroll
            var scrollEl = this.options.scrollElement;
            var scrollElDom = scrollEl[0];
            
            if (this.options.enableMore) {
                var offsetHeight = scrollElDom.offsetHeight;
                
                this._moreLoading = false;
                if (!this._scroller) {
                    this._scroller = new uiScroll(scrollElDom, {});
                } else {
                    this._scroller.refresh();
                    //this._scroller.destroy();
                   // this._scroller = new uiScroll(scrollElDom, {});
                }

                //var throttledCallback = _.throttle(function (e) {
                //    self._onScroll(offsetHeight, e || this);
                //}, 0);

                
                this._scroller.off("scroll").on("scroll", function (e) {
                    self._onScroll(offsetHeight, e || this);
                });
            }
        },
        _onPullToRefresh: function () {
            logger.log("pull to refresh!");
            this._pullToRefreshTimer = null;
            this.trigger("reload");
        },
        _onScroll: function (offsetHeight, e) {
            if (this._moreLoading) return;
            var self = this;
                      
            var top = 0;
            if (e && e.y) {
                top = -1*e.y;
            } else if (e && e.target && e.target.scrollTop) {
                top = e.target.scrollTop;
            }
            var scrollHeight = (e.scrollerHeight || ((e.target && e.target.firstChild)?e.target.firstChild.offsetHeight:0));

            if (this._pullToRefreshTimer && top >= 0) {
                window.clearTimeout(this._pullToRefreshTimer);
                this._pullToRefreshTimer = null;
            } else if (!this._pullToRefreshTimer && top < -10) {
                this._pullToRefreshTimer = window.setTimeout(_.bind(this._onPullToRefresh,self), 1000);
            }
            
            //   logger.log(top + " " + scrollHeight + " " + offsetHeight);
            if (scrollHeight && top && (offsetHeight + top) >= (scrollHeight - this._moreBottomOffset)) {
                //if (this._scroller) {
                //    this._scroller.off("scroll");
                //}
                var moreResult = this._more();
                if (moreResult) {
                    this._moreLoading = true;
                    moreResult.always(function () {
                        // rebind
                        self._moreLoading = false;
                        if (self._scroller) {
                            self._scroller.refresh();

                            var throttledCallback = _.throttle(function (e) {
                                self._onScroll(offsetHeight, e || this);
                            }, 0);

                            self._scroller.off("scroll").on("scroll", throttledCallback);
                        }
                        //if (this._scroller) {
                        //    this._scroller.on("scroll",function () {
                        //        self._onScroll(offsetHeight, this);
                        //    });
                        //}
                    });
                }
            }

            if (this.options.hideFooter) {
                var diff = (top - this._lastTop);
                if (Math.abs(diff) >= 100) {
                    // logger.log("hide: " + diff + " " + top + " " + this._lastTop);
                    this._lastTop = top;
                    if (diff > 0) {
                        //  uiContainers.header.hide();
                        uiContainers.footer.hide();
                    } else {
                        //  uiContainers.header.show();
                        uiContainers.footer.show();
                    }
                }
            }
        },
        onBeforeDestroy: function () {
            this.unregisterControls();
            if (this._moreButton) {
                this._moreButton.off("select");
                this._moreButton.destroy();
                this._moreButton = null;
            }
            if (this.options.scrollElement) {
                this.options.scrollElement.off(".view-list");
            }
            if (this._scroller) {
                this._scroller.destroy();
                this._scroller = null;
            }
            if (this._pullToRefreshTimer) {
                window.clearTimeout(this._pullToRefreshTimer);
                this._pullToRefreshTimer = null;
            }
        },
        empty:function(){
            this.unregisterControls();
            this.list = [];
            if (this.el) this.el.empty();
            if (this._moreButton) this._moreButton.destroy();

            this.initializeMoreButton();
        },
        load: function (data, options) {
            
            if (options && options.empty) {
                this.empty();
            }
            
            var self = this;
            var deferred = $.Deferred();
            var callbackFlags = {
                isListEmpty: (this.list.length == 0),
                
            }
            if (_.isFunction(data)) {
                this.trigger("loadstart", data, callbackFlags);
                var fnResult = data();
                if (!fnResult) {
                    deferred.resolve(null, callbackFlags)
                } else {
                    fnResult.done(function (serverData,arg0,arg1) {
                        deferred.resolve(serverData, callbackFlags,arg0,arg1);
                    }).fail(function () {
                        deferred.reject();
                    });
                }
            } else {
                this.trigger("loadstart", data, callbackFlags);
                deferred.resolve(data, callbackFlags);
            }
            
            deferred.done(function (data, opt, arg0, arg1) {
                self._load(data, options);
                self.trigger("loadend", data, opt, arg0, arg1);
            }).fail(function (data) {
                self.trigger("loaderror", data);
            });

            return deferred;
        },
        _load: function (data, options) {
            var list = data;
            var getList = options.getList || this.options.getList || this.getList;
            if (getList) list = getList(data);
            
           // this.list = this.list||[];
            //this.list.push(list);
            this.list = list;
            var self = this;
            _.each(list, function (o) {
                var item = self.createItem(o);
                if (item) {
                    self.registerControl(item);
                    item.createIn({},self.el);
                    item.on("select", function (el) {
                        self.trigger("select", el);
                    });
                }
            });

            if (this.options.enableMore) {
                var hasMore = options.hasMore || this.options.hasMore || this.hasMore;
                
                if (hasMore) {
                    this._hasMore = hasMore(data, list);
                }
                if (this._moreButton) this._moreButton.visible(this._hasMore);
            }

        },
        registerControl: function (control) {
            if (!control || !this._controlContainer) return this;
            this._controlContainer.push(control);

            return this;
        },
        createItem:function(data){
            var item = new uiControlButton();
            
            return item;
        },
        unregisterControls: function () {
           
            if (this._controlContainer && this._controlContainer.length > 0) {
                this._controlContainer = $.map(this._controlContainer, function (o) {
                    if (o || o.destroy) {
                        o.destroy();
                        o = null;
                        return null;
                    }
                    return o;
                });
            };

            return this;
        },

    });
    
    return ViewList;
});