﻿define([
  'jquery',
  'underscore',
  'api',
  'helper',
  'uiTimeAgo',
  'uiGlobal',
  'globalize',
  'template',
  'uiControlButton'
], function ($, _, api, helper, uiTimeAgo, uiGlobal, globalize, template, uiControlButton) {

    var eyeCatchingLabel = globalize.localize("views.shared.offer.eye-catching.label");
    var rentLabel = globalize.localize("views.shared.rent.label");
    var priceLabel = globalize.localize("views.shared.price.label");
    var priceVehicleLabel = globalize.localize("views.shared.priceVehicle.label");
    var prepaymentLabel = globalize.localize("views.shared.prepayment.label");

    var Control = uiControlButton.extend({
        _imageWidth: 0,
        _imageHeight: 0,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                data: null
            }, options);

            this._imageWidth = Math.floor(Math.min((uiGlobal.dimensions.body.width - 15) / 3, 200));
            this._imageHeight = Math.floor(this._imageWidth / 1.33);
        },
        template: template.offer._listItem,
        getRenderData: function () {
            var item = this.options.data;
            if (!item) return null;

            var imageDimension = {
                width: this._imageWidth,
                height: this._imageHeight
            };

            return {
                item: item,
                imageDimension: imageDimension,
                image: function (key) {
                    return api.offer.imageUrl(key, {
                        width: imageDimension.width,
                        height: imageDimension.height
                    });
                },
                format: function (val, f) {
                    return globalize.format(val, f);
                },
                renderDate: function (val) {
                    var date = helper.parseDate(val);
                    return uiTimeAgo.inWords(new Date().getTime() - date.getTime());
                },
                isFavorite: false, // api.offer.isFavorite(item.id),
                isEyeCatching: helper.isOfferEyeCatching(item),
                eyeCatchingLabel: eyeCatchingLabel,
                rentLabel: rentLabel,
                priceLabel: priceLabel,
                priceVehicleLabel: priceVehicleLabel,
                prepaymentLabel: prepaymentLabel,
                categoryCssClass: helper.toCategoryCssClass(item.category)
            };
        }
    });

    return Control;
});