﻿define(["jquery", "underscore", "uiPageTransition", "uiControlDialogOptions", "uiBackdrop", "logger"],
        function ($, _, uiPageTransition, uiControlDialogOptions, uiBackdrop, logger) {

            function Dialog() {
                var _optionControl = null;
                function confirm(message, title, buttonLabels) {

                    var result = $.Deferred();
                    if (navigator && navigator.notification && navigator.notification.confirm) {
                        logger.log("using native confirm");
                        navigator.notification.confirm(message, function (index) {
                            if (index == 1) {
                                result.resolve(index);
                            } else {
                                result.reject(index);
                            }
                        }, title, buttonLabels);
                    } else {
                        logger.log("using web confirm");
                        if (window.confirm(message)) {
                            result.resolve(1);
                        } else {
                            result.reject(2);
                        }
                    }

                    return result;
                }

                function alert(message, title, buttonName) {
                    var result = $.Deferred();
                    if (navigator && navigator.notification && navigator.notification.alert) {
                        navigator.notification.alert(message, function () {
                            result.resolve();
                        }, title, buttonName);
                    } else {
                        window.alert(message);
                        result.resolve();
                        //if (alertCallback) alertCallback();
                    }

                    return result;
                }

                function optionsOpen(title, options, callback) {
                    var result = $.Deferred();

                    // if options are opened, close them first
                    if (_optionControl) {
                        _optionControl.close().always(function () {
                            optionsShow(title, options, callback).done(function (index) {
                                result.resolve(index);
                            }).fail(function (index) {
                                result.fail(index);
                            });
                        });
                    } else {
                        result = optionsShow(title, options, callback);
                    }

                    return result.always(function () {
                        _optionControl = null;
                    });
                }

                function optionsIsOpen() {
                    return (_optionControl);
                }

                function optionsShow(title, options, callback) {
                    uiBackdrop.show();
                    _optionControl = new uiControlDialogOptions();
                    return _optionControl.open({
                        title: title,
                        items: options
                    }).always(function () {
                        uiBackdrop.hide();
                    });
                }

                function optionsClose() {
                    uiBackdrop.hide();
                    if (!_optionControl) {
                        var deferred = $.Deferred();
                        deferred.resolve();
                        return deferred;
                    }
                    return _optionControl.close().always(function () {
                        _optionControl = null;
                    });
                }

                this.confirm = confirm;
                this.alert = alert;
                this.options = {
                    isOpen: optionsIsOpen,
                    open: optionsOpen,
                    close: optionsClose
                };
            }

            return new Dialog();
        }
);