﻿define(["jquery", "underscore", "libHammer", 'helper', 'logger'],
        function ($, _, Hammer, helper, logger) {

            var VENDORS = [
                { prop: "Moz", propExpanded: "-moz-" },
                { prop: "Webkit", propExpanded: "-webkit-" },
                { prop: "Ms", propExpanded: "-ms-" },
                { prop: "O", propExpanded: "-o-" }];

            function capitaliseFirstLetter(string) {
                return string.charAt(0).toUpperCase() + string.slice(1);
            }

            function getVendorSpecificName(el, prop) {
                var style = el.style;
                var result = { prop: prop, propExpanded: prop };

                for (var i = 0, l = VENDORS.length; i < l; i++) {
                    var p = VENDORS[i];
                    var tryProp = p.prop + capitaliseFirstLetter(prop);
                    if (typeof style[tryProp] !== "undefined") {
                        return {
                            prop: tryProp,
                            propExpanded: p.propExpanded + prop
                        };
                    }
                }

                if (typeof style[prop] !== "undefined") {
                    return {
                        prop: prop,
                        propExpanded: prop
                    };
                }

                return result;
            }

            return {
                
               init: function () {
                   this._istouchmove = false;

                   this.html = document.body.parentNode;
                   this.document = document;
                   this.window = window;
                   this.$window = $(window);
                   this.$document = $(this.document);
                   this.$body = $("body");
                   this.$content = $("#content");
                   this.$header = $("#contentHeader");
                   this.$footer = $("#contentFooter");

                   this.$snapContent = $("#snapContent");
                   this.$snapDrawerLeft = $("#sideContainerLeft");
                   this.$snapDrawerRight = $("#sideContainerRight");

                   var desktopEvents = ['click','mousedown', 'mousemove', 'mouseup'];
                   if (this.browser.ie10) desktopEvents = ['click','MSPointerDown', 'MSPointerMove', 'MSPointerUp'];
                   if (this.browser.ie11) desktopEvents = ['click','pointerdown', 'pointermove', 'pointerup'];

                   this.event = {
                       tap: this.support.touch ? 'tap' : desktopEvents[0],
                       touchstart: this.support.touch ? 'touchstart' : desktopEvents[1],
                       touchmove: this.support.touch  ? 'touchmove' : desktopEvents[2],
                       touchend: this.support.touch ? 'touchend' : desktopEvents[3]
                   }

                   Hammer.defaults.stop_browser_behavior.touchAction = 'pan-y';

                   this._calculateDimensions();
                   this._initRequestAnimationFrame();
                   
               },
               _calculateDimensions:function(){
                   this.dimensions = {
                       body:{
                           width: this.$body.width(),
                           height: this.$body.height()
                       }
                   }
                   
               },
               _initRequestAnimationFrame:function() {
                    var lastTime = 0;
                    var vendors = ['webkit', 'moz'];
                    for (var x = 0; x < vendors.length && !window.requestAnimationFrame; ++x) {
                        window.requestAnimationFrame = window[vendors[x] + 'RequestAnimationFrame'];
                        window.cancelAnimationFrame =
                            window[vendors[x] + 'CancelAnimationFrame'] || window[vendors[x] + 'CancelRequestAnimationFrame'];
                    }

                    if (!window.requestAnimationFrame) {
                        logger.warn("requestAnimationFrame does not exist. Using polifill");
                        window.requestAnimationFrame = function (callback, element) {
                            var currTime = new Date().getTime();
                            var timeToCall = Math.max(0, 16 - (currTime - lastTime));
                            var id = window.setTimeout(function () { callback(currTime + timeToCall); },
                                timeToCall);
                            lastTime = currTime + timeToCall;
                            return id;
                        };
                    }
                    if (!window.cancelAnimationFrame) {
                        logger.warn("cancelAnimationFrame does not exist. Using polifill");
                        window.cancelAnimationFrame = function (id) {
                            clearTimeout(id);
                        };
                    }
               },
               transform: function (el, val) {
                   var deferred = $.Deferred();
                   if (el instanceof $) {
                       if (el.length == 0) {
                           return deferred.fail();
                       }
                       el = el[0];
                   }

                   if (!el || !el.style) {
                       return deferred.fail();
                   }

                   var es = el.style;

                   if (es.setProperty) {
                       es.setProperty("transform", val);
                       es.setProperty("-webkit-transform", val);
                   } else {
                       es.webkitTransform = es.MsTransform = es.msTransform = es.MozTransform = es.OTransform = es.transform = val;
                   }
                   
                   deferred.resolve();

                   return deferred;
               },
               setTransitionProperty:function(es,val) {
                   if (es.setProperty) {
                       es.setProperty("transition", val);
                       es.setProperty("-webkit-transition", val);
                   } else {
                       es.webkitTransition = es.MsTransition = es.msTransition = es.MozTransition = es.OTransition = es.transition = val;
                   }
               },
               transition: function (el, val,duration) {
                   var deferred = $.Deferred();
                   if (el instanceof $) {
                       if (el.length == 0) {
                           return deferred.fail();
                       }
                       el = el[0];
                   }

                   if (!el || !el.style) {
                       return deferred.fail();
                   }

                   var es = el.style;
                   var t = val;

                   if (val.indexOf("transform") > -1) {
                       var rt = getVendorSpecificName(el,"transform").propExpanded;
                       val = val.replace("transform", rt);
                   }
                   this.setTransitionProperty(es, val);

                   if (!val) {

                       deferred.resolve();

                       return deferred;
                   }

                   var timer = null;
                   if (duration) {
                       timer = window.setTimeout(function () {
                           timer = null;
                           logger.log("Transition timeout: " + val);
                           deferred.resolve();

                       }, (duration+200));
                   }

                   $(el).one('webkitTransitionEnd transitionend oTransitionend mozTransitionEnd', function (e) {
                       if (timer) {
                           window.clearTimeout(timer);
                           timer = null;
                       }
                       deferred.resolve();
                   });

                   return deferred.always(_.bind(function () {
                       if (es) {
                           this.setTransitionProperty(es, '');
                       }
                       if (el) {
                           el.offsetWidth;
                       }
                   },this));
               },

               support: {

                   touch: (window.Modernizr && Modernizr.touch === true) || (function () {
                       return !!(("ontouchstart" in window) || window.DocumentTouch && document instanceof DocumentTouch);
                   })(),

                   transforms3d: (window.Modernizr && Modernizr.csstransforms3d === true) || (function () {
                       var div = document.createElement('div').style;
                       return ("webkitPerspective" in div || "MozPerspective" in div || "OPerspective" in div || "MsPerspective" in div || "perspective" in div);
                   })(),

                   transforms: (window.Modernizr && Modernizr.csstransforms === true) || (function () {
                       var div = document.createElement('div').style;
                       return ('transform' in div || 'WebkitTransform' in div || 'MozTransform' in div || 'msTransform' in div || 'MsTransform' in div || 'OTransform' in div);
                   })(),

                   transitions: (window.Modernizr && Modernizr.csstransitions === true) || (function () {
                       var div = document.createElement('div').style;
                       return ('transition' in div || 'WebkitTransition' in div || 'MozTransition' in div || 'msTransition' in div || 'MsTransition' in div || 'OTransition' in div);
                   })(),

                   nativeScroll: function () {
                        //return false;
                       // Android < 4.0.0 has n suppport for overflow scroll
                       
                       if (typeof device !== 'undefined' &&
                           device.available &&
                           device.platform &&
                           device.platform.toLowerCase() === "android" &&
                           helper.fromVersionString(device.version).major < 4) return false;

                      return true;

                      //return false;
                   }()
               },

               browser: {

                   ie8: (function () {
                       var rv = -1; // Return value assumes failure.
                       if (navigator.appName == 'Microsoft Internet Explorer') {
                           var ua = navigator.userAgent;
                           var re = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
                           if (re.exec(ua) != null)
                               rv = parseFloat(RegExp.$1);
                       }
                       return rv != -1 && rv < 9;
                   })(),

                   ie10: window.navigator.msPointerEnabled,
                   ie11: window.navigator.pointerEnabled
               },
               

               destroy: function () {
                   this.html        = null;
                   this.document    = null;
                   this.window      = null;
                   this.$document   = null;
                   this.$body       = null;
                   this.$content    = null;
                   this.$header     = null;
                   this.$footer     = null;

                   this.$snapContent        = null;
                   this.$snapDrawerLeft     = null;
                   this.$snapDrawerRight    = null;
               },

               touch: function (e, funct) {
                    //if drag started, do nothing
                   if (this._istouchmove) {
                       logger.log("canceled touch because touch move.");
                       return true;
                   }
                   logger.log("touch detected");
                   // check if event was handled
                   if (e.isImmediatePropagationStopped()) return false;
                   e.preventDefault();
                   e.stopImmediatePropagation();

                   funct.apply(e.currentTarget, [e]);

                   return false;
               },
               toggleClass: function (el,val,condition) {
                   if (!el) return;
                   if (condition) {
                       if (!el.hasClass(val)) el.addClass(val);
                   } else {
                       if (el.hasClass(val)) el.removeClass(val);
                   }
               }
           };
        }
);