﻿define(["jquery",
    "backbone",
    "underscore",
    "uiPageTransition",
    "uiControlSelectList",
    "template",
    'uiGlobal',
    'globalize',
    'libHammer',
    'uiContainers',
    'views/controls/list',
    'logger'],
        function ($, Backbone, _, uiPageTransition, SelectListControl, template, uiGlobal, globalize, Hammer, uiContainers, ViewControlsList, logger) {

            var translateShared = globalize.localizeBase("views.shared");

            function Modal() {
                var el = null;
                var control = null;
                var _inTransition = false;
                var _open = false;
                var self = this;
                var _views = [];

                function open(options, View,callerView) {
                    if (View) {
                        return _openView(options, View, callerView);
                    }
                    return _openLegacy(options);
                }

                function _openView(options, View, callerView) {
                    var result = $.Deferred();

                    if (_inTransition) {
                        logger.log("opening in progress...");
                        result.resolve();
                        return result;
                    }

                    View.prototype._executeTransition = function () {
                        var self = this;
                        this.$el.toggleClass("modal content page", true);
                        uiPageTransition.slideUp(this.$el).always(_.bind(function (previousView, currentView) {
                            _inTransition = false;
                            //this.initializeSnapLeft(uiContainers.snap);
                            //this.initializeSnapRight(uiContainers.snap);
                            //this.initializeHeader(uiContainers.header);
                            //this.initializeFooter(uiContainers.footer);
                            //this.initializeHeaderSec(uiContainers.headerSec);
                            this.applyScroll();
                            if (callerView) {
                                callerView.hide();
                            }
                            result.resolve(this);
                        }, this));
                    }

                    _inTransition = true;

                    var inst = new View(options);

                    _views.push({
                        view: inst,
                        caller: callerView
                    });

                    return result;
                }

                function _openLegacy(options) {

                    logger.log("modal opening");
                    var result = $.Deferred();
                    if (_inTransition) {
                        logger.log("opening in progress...");

                        result.resolve();
                        return result;
                    }

                    options = $.extend({}, {
                    }, options);

                    data = options.data || {};

                    el = $("<div><div/>").attr("class", "modal page").html(options.template(data));

                    _inTransition = true;
                    _open = true;
                    uiPageTransition.slideUp(el).always(function (el1) {
                        _inTransition = false;
                        self.el = el;

                        result.resolve(self);
                    });

                    return result;
                }

                function closeLegacy() {
                    var result = $.Deferred();

                    if (!el) {
                        return result.resolve();
                    }

                    _inTransition = true;
                    // uiGlobal.$body.toggleClass("hasModal", false);
                    logger.log("modal close");
                    uiPageTransition.slideDown(el).always(function () {
                        _inTransition = false;
                        _open = false;

                        result.resolve(self);

                        if (el) {
                            el.remove();
                            el = null;
                        }

                        self.destroy();

                        logger.log("modal close end");

                    });

                    return result;
                }

                function close() {
                    logger.log("modal closing");

                    if (_views && _views.length > 0) {
                        logger.log("modal closing views");
                        var inst = _views.pop();

                        return closeView(inst);
                    }

                    return closeLegacy();
                }

                function closeView(view) {
                    var result = $.Deferred();

                    if (!view) {
                        return result.resolve();
                    }
                       
                    _inTransition = true;

                    if (view.caller) {
                        view.caller.initializeHeader(uiContainers.header);
                        view.caller.initializeFooter(uiContainers.footer);
                        view.caller.initializeHeaderSec(uiContainers.headerSec);
                        view.caller.show();
                    }

                    uiPageTransition.slideDown(view.view.$el).always(function () {

                        _inTransition = false;
                        _open = false;

                        result.resolve(view);

                        if (view.view && view.view.destroy) {
                            logger.log("modal calling destroy");
                            view.view.destroy();
                            view.view == null;
                        }

                        logger.log("modal close end");

                    });

                    return result;
                }

                function closeAll() {

                    if (_views && _views.length > 0) {
                        logger.log("modal closing views");

                        var results = [];
                        while ((o = _views.pop())) {
                            results.push(closeView(o));
                        }
                        
                        return $.when(results);
                    }
                    return closeLegacy();
                }

                function destroy() {
                    if (el) {
                        logger.log("modal removing");
                        el.off(".modal");
                        el.remove();
                        el = null;
                    }

                    if (control && control.destroy) {
                        control.destroy();
                    }
                    if (_views) {
                        _.each(_views, function (o) {
                            if (!o) return;
                            if (o.view) {
                                o.view.destroy();
                                o.view = null;
                            }
                            o.caller == null;
                        });
                    }
                }

                function isOpen() {
                    if (_views && _views.length > 0) return true;
                    if (_open) return true;
                    return false;
                }

                function openSelectList(options, callerView) {
                    
                    if (typeof callerView==='undefined') {
                        return openSelectListLegacy(options);
                    }

                    return open(options, ViewControlsList, callerView);
                }

                function openSelectListLegacy(options) {

                    var selectedHandle = function (data,source) {
                        if (options.onSelect) options.onSelect(data, source);
                        if (options.onClose) options.onClose();
                        self.close();
                    }
                    var cancelHandle = function () {
                        if (options.onCancel) options.onCancel();
                        if (options.onClose) options.onClose();
                        self.close();
                    }

                    control = new SelectListControl({
                        type:options.type,
                        onSelect: function (data, el, source) {
                            if (options.type == 'single') {
                                selectedHandle(data, source);
                            }
                        }
                    });

                    var isFunction = (typeof options.data == "function");

                    var modalData = {
                        hasHeader: true,
                        hasBody: true,
                        hasFooter: false,
                        renderHeader: function () {
                            
                            return template._headerOkCancel({
                                cancelEnabled: true,
                                cancelText: translateShared("close"),
                                cancelIcon: "icon-cancel",
                                cancelCssClass:"",
                                okEnabled: true,
                                okText: translateShared("accept"),
                                okIcon: "icon-accept",
                                okCssClass:"primary",
                                title: options.title||"",
                                subtitle: options.subtitle || "",
                                titleType:""
                            });
                        },
                        renderBody: function () {
                            if (!isFunction)
                            {
                                return control.render(options.data);
                            }
                            
                            return null;
                        }
                    };

                    var modalOptions = {
                        template: template.controls.modal,
                        data:modalData
                    };

                    var result = open(modalOptions).done(function (o) {
                        if (!control || !o.el) return;
                       
                        $(".action-cancel", o.el).hammer({
                        //    prevent_default: true
                        }).on(uiGlobal.event.tap, function (e) {
                            e.preventDefault();
                            e.stopPropagation();
                         //   uiGlobal.touch(e, function (e) {
                                cancelHandle();
                          //  });
                        });

                        $(".action-ok", o.el).hammer({
                         //   prevent_default: true
                        }).on(uiGlobal.event.tap, function (e) {
                            e.preventDefault();
                            e.stopPropagation();
                            //uiGlobal.touch(e, function (e) {
                                selectedHandle(control.selected(), control.data);
                           // });
                        });

                        if (isFunction) {
                            var header = $("[data-view='header']", o.el);
                            var container = $("[data-view='body']", o.el);
                            if (header) header.toggleClass("loading", true);
                            options.data().done(function (data) {
                                if (container) {
                                    var html = control.render(data);
                                    container.html(html);
                                    control.create(container);
                                }
                            }).always(function () {
                                if (header) header.toggleClass("loading", false);
                            });
                        } else {
                            control.create(o.el);
                        }
                    });

                    return result;
                }

                this.openSelectList = openSelectList;
                this.open = open;
                this.close = close;
                this.closeAll = closeAll;
                this.isOpen = isOpen;
                this.destroy = destroy;
                this.el = el;
            }

            return new Modal();
        }
);