﻿define(["jquery", "uiGlobal", 'logger'],
        function ($, uiGlobal, logger) {

            function PageSlider(container) {

                var container = container,
                    currentView,
                    stateHistory = [];

                this.slideView = function (view) {
                    var l = stateHistory.length,
                        state = window.location.hash;

                    if (l === 0) {
                        stateHistory.push(state);
                        return this.slideViewFrom(view);
                    }

                    var hasHistory = this.hasHistory();
                    if (hasHistory) {
                        stateHistory.pop();
                        return this.slideViewFrom(view, 'p-left');
                    }

                    stateHistory.push(state);
                    return this.slideViewFrom(view, 'p-right');
                }

                this.hasHistory = function () {
                    var l = stateHistory.length,
                        state = window.location.hash;

                    if (l === 0) return null;
                    if (state === stateHistory[l - 2]) return true;

                    return false;
                }

                // Use this function directly if you want to control the sliding direction outside PageSlider
                this.slideViewFrom = function (view, from) {
                    var result = $.Deferred();

                    if (!currentView || !from) {
                        view.$el.attr("class", "page p-center " + view.className);
                        currentView = view;
                        return result.resolve(null,currentView);
                    }

                    // Position the page at the starting position of the animation
                    var view1 = currentView;
                    var view2 = view;

                    view.$el.attr("class", "page " + from + " " + view.className);
                    currentView.$el.data("slidetransition", true);

                    // Force reflow. More information here: http://www.phpied.com/rendering-repaint-reflowrelayout-restyle/
                    container[0].offsetWidth;

                    // Position the new page and the current page at the ending position of their animation with a transition class indicating the duration of the animation
                    var transition = "transform 0.4s ease";
                    
                    uiGlobal.toggleClass(uiGlobal.$body,"has-transition", true);
                    uiGlobal.transition(view.$el, transition,400);
                    uiGlobal.transition(currentView.$el, transition, 400).always(function () {
                        uiGlobal.toggleClass(uiGlobal.$body,"has-transition", false);
                        result.resolve(view1, view2);
                    });

                    view.$el.attr("class", "page p-center " + view.className);
                    currentView.$el.attr("class", "page " + (from === "p-left" ? "p-right" : "p-left") + " " + currentView.className);
                    currentView = view;

                    return result.always(function (v1,v2) {
                        if (v2 && v2.$el) {
                            v2.$el.attr("class", "page p-center " + view.className);
                            uiGlobal.transition(v2.$el, '');
                            uiGlobal.transform(v2.$el, '');                            
                        }
                    });
                }


                this.slideUp = function (el, holder) {
                    holder = holder || container;
                    var result = $.Deferred();

                    if (!el || !holder) {
                        return result.reject();
                    }

                    // Position the page at the starting position of the animation
                    var currentClass = el.attr("class") ||"";
                  
                    el.attr("class", "p-bottom " + currentClass);
                    holder.append(el);
                    el.data("currentclass", currentClass);
                   
                    // Force reflow. More information here: http://www.phpied.com/rendering-repaint-reflowrelayout-restyle/
                    holder[0].offsetWidth;

                    var transition = "transform 0.4s ease";
                    uiGlobal.transition(el, transition, 400).always(function () {
                        result.resolve(el, holder);
                        uiGlobal.transition(el, '');
                        uiGlobal.transform(el, '');
                    });

                    // Position the new page and the current page at the ending position of their animation with a transition class indicating the duration of the animation
                    el.attr("class", "p-center " + currentClass);

                    return result;
                }

                this.slideDown = function (el, holder) {
                    holder = holder || container;
                    var result = $.Deferred();

                    if (!el || !holder) {
                        return result.reject();
                    }
                    
                    var currentClass = el.data("currentclass") || "";
                    el.attr("class", "p-center " + currentClass);

                    // Force reflow. More information here: http://www.phpied.com/rendering-repaint-reflowrelayout-restyle/
                    holder[0].offsetWidth;

                    var transition = "transform 0.4s ease";
                    uiGlobal.transition(el, transition,400).always(function () {
                        result.resolve(el, holder);
                        uiGlobal.transition(el, '');
                        uiGlobal.transform(el, '');
                    });

                    // Position the new page and the current page at the ending position of their animation with a transition class indicating the duration of the animation
                    el.attr("class", "p-bottom " + currentClass);

                    return result;
                }

                this.fadeIn = function (el, holder) {
                    holder = holder || container;
                    var result = $.Deferred();

                    if (!el || !holder) {
                        result.reject(el, holder);
                        return result;
                    }

                    // Position the page at the starting position of the animation
                    var currentClass = el.attr("class");
                    el.data("currentclass", currentClass);
                    el.attr("class", "p-fade p-fade-out " + currentClass);
                    logger.log("fade in class: p-fade  p-fade-out " + currentClass);
                    holder.append(el);

                    // Force reflow. More information here: http://www.phpied.com/rendering-repaint-reflowrelayout-restyle/
                    holder[0].offsetWidth;

                    var transition = "opacity 0.4s ease";
                    uiGlobal.transition(el, transition, 400).done(function () {
                        result.resolve(el, holder);
                    }).fail(function () {
                        result.reject(el, holder);
                    }).always(function () {
                        uiGlobal.transition(el, '');
                        uiGlobal.transform(el, '');
                    });

                    // Position the new page and the current page at the ending position of their animation with a transition class indicating the duration of the animation
                    el.attr("class", "p-fade " + currentClass);

                    return result;
                }

                this.fadeOut = function (el, holder) {
                    holder = holder || container;
                    var result = $.Deferred();
                    if (!el || !holder) {
                        result.reject(el, holder);
                        return result;
                    }

                    var currentClass = el.data("currentclass") || "";
                    logger.log("fade out class: p-fade " + currentClass);
                    el.attr("class", "p-fade " + currentClass);
                   
                    // Force reflow. More information here: http://www.phpied.com/rendering-repaint-reflowrelayout-restyle/
                    holder[0].offsetWidth;

                    var transition = "opacity 0.4s ease";
                    uiGlobal.transition(el, transition,400).done(function () {
                        result.resolve(el, holder);
                        if (el) {
                            el.remove();
                        }
                    }).fail(function () {
                        result.reject(el, holder);
                    }).always(function () {
                        uiGlobal.transition(el, '');
                        uiGlobal.transform(el, '');
                    });

                    // Position the new page and the current page at the ending position of their animation with a transition class indicating the duration of the animation
                    el.attr("class", "p-fade p-fade-out " + currentClass);

                    return result;
                }

            }

            return new PageSlider($("#snapContent"));
        }
);