define(["jquery", "underscore", "uiPageTransition", 'uiGlobal', 'logger'],
        function ($, _, uiPageTransition, uiGlobal, logger) {
                        
            function Toast() {
                var el = null;
                var _isOpen = false;
                var _hasTransition = false;

                function info(options) {

                    if (typeof options === 'string') {
                        options = {
                            content: options
                        };
                    }

                    options.cssClass = options.cssCLass || "toast-info";

                    return open(options);
                }

                function danger(options) {

                    if (typeof options === 'string') {
                        options = {
                            content: options
                        };
                    }

                    options.cssClass = options.cssCLass || "toast-danger";

                    return open(options);
                }

                function success(options) {

                    if (typeof options === 'string') {
                        options = {
                            content: options
                        };
                    }

                    options.cssClass = options.cssCLass || "toast-success";

                    return open(options);
                }

                function warning(options) {

                    if (typeof options === 'string') {
                        options = {
                            content: options
                        };
                    }

                    options.cssClass = options.cssCLass || "toast-warning";

                    return open(options);
                }

                function open(options) {
                    var result = $.Deferred();
                    if (_isOpen || _hasTransition) {
                        result.reject();
                        return result;
                    }

                    if (typeof options === 'string') {
                        options = {
                            content: options
                        };
                    }

                    _hasTransition = true;
                    options = $.extend({},{
                        content: '',
                        duration: 3000,
                        cssClass:"toast-info",
                        position: 'bottom' // top, bottom, center
                    }, options);

                    el = $("<div />").addClass("toast").addClass(options.cssClass);

                    var innerEl = $("<p />").addClass("toast-message").html(options.content);
                    innerEl.appendTo(el);

                    // Set the Toast's position
                    switch (options.position) {
                        case 'top': el.css({top : '72px'}); break;
                        case 'center': break;
                        default: el.css({ bottom: '72px' });;
                    }

                    uiGlobal.$body.append(el);
                    
                  //  var toastWidth = innerEl.outerWidth();
                   // toastWidth = toastWidth.slice(0, toastWidth.length - 2);

                //    var left = (window.innerWidth / 2) - (toastWidth / 2);

                  //  el[0].style.marginLeft = left + 'px';

                    var self = this;
                    uiPageTransition.fadeIn(el, uiGlobal.$body).always(function () {
                        logger.log("toast open transition end: " + options.duration);
                        if (options.duration){
                            setTimeout(function () {
                                logger.log("end");
                                close();
                            }, options.duration);
                        }
                        _hasTransition = false;
                        _isOpen = true;
                        result.resolve(self);
                    });

                    return result;
                }

                function close() {
                    var result = $.Deferred();
                    logger.log("close: " + _isOpen + " " + _hasTransition);
                    if (!_isOpen || _hasTransition) {
                        result.reject();
                        return result;
                    }

                    var self = this;
                    _hasTransition = true;
                    uiPageTransition.fadeOut(el, uiGlobal.$body).done(function () {
                        _hasTransition = false;
                        _isOpen = false;
                        result.resolve(self);
                        destroy();
                    });

                    return result;
                }

                function destroy() {
                    logger.log("destroy");
                    if (el) {
                        el.remove();
                        el = null;
                    }
                  
                    _isOpen = false;
                    _hasTransition = false;
                }

                this.open = open;
                this.info = info;
                this.danger = danger;
                this.success = success;
                this.warning = warning;
                this.close = close;
                this.destroy = destroy;
                this.el = el;
            }

            return new Toast();
        }
);
