﻿define(["jquery","underscore", "backbone", "validation"],
        function ($,_, Backbone, validation, settings) {

            var UIValidation = function () {

                function init() {
                    _.extend(Backbone.Model.prototype, Backbone.Validation.mixin);
                    _.extend(Backbone.Validation.callbacks, {
                        valid: function (view, attr, selector) {
                           
                            selector = parseSelector(attr, selector, view,null);
                            if (!selector) return;

                            var element;
                            if (selector instanceof $) {
                                 element = selector;
                            } else {
                                if (selector.selector) {
                                    selector = selector.selector;
                                }
                                 element = view.$(selector);
                            }
                            var group = resolveElementGroup(element);

                            group.removeClass('has-error');
                            
                            group.find(".error").remove();
                        },
                        invalid: function (view, attr, error, selector) {
                           
                            selector = parseSelector(attr, selector, view,error);
                            if (!selector) return;

                            var element;
                            var errorText = error;
                            if (selector instanceof $) {
                                element = selector;
                            } else {
                                if (selector.error && selector.selector) {
                                    errorText = selector.error;
                                    selector = selector.selector;
                                }
                                element = view.$(selector);
                            }
                            var group = resolveElementGroup(element);

                            group.addClass('has-error');

                            var errorEl = group.find(".error");
                            if (errorEl.length === 0) {
                                errorEl = $("<span class=\"error\"></span>");

                                var errorContainer = group.find(".form-control");
                                if (errorContainer.length === 0) errorContainer = group;

                                errorContainer.append(errorEl);
                            }
                           // var target = group.find(".error");
                            errorEl.text(errorText);
                        }
                    });
                }

                function resolveElementGroup(element) {
                    if (element.hasClass("form-group")) return element;
                    var result = element.parents(".form-group");
                    if (result && result.length > 0) return result;
                    return element;
                }

                function parseSelector(attr, selector,view,error) {
                    if (_.isFunction(selector)) {
                        selector = selector.apply(view, [attr, view, error]);
                    } else {
                        selector = (selector == 'id' ? '#' + attr : '[' + selector + '=' + attr + ']');
                    }

                    return selector;
                }

                function parse(model, serverErrors) {
                    // fill in validation error
                    var baseName = null;
                    //var attrs = flatten(result.ModelState);
                    var modelAttributes = model.attributes;
                    var errors = {};

                    applyServerValidationProp(modelAttributes, "model", serverErrors, errors);

                    return errors;

                }

                function applyServerValidationProp(items, baseKey, serverErrors, errors) {
                    if (items == null) return;
                    $.each(items, function (key, value) {
                        if (value != null & typeof value == "object" && (!(value instanceof Date))) {
                            applyServerValidationProp(value, baseKey + "." + key, serverErrors, errors);
                        } else {
                            var fullKey = baseKey + "." + key;
                            if (serverErrors.hasOwnProperty(fullKey)) {
                                var errorList = serverErrors[fullKey];

                                if (errorList.length > 0) {
                                    errors[key] = errorList[0];
                                }
                            }

                        }
                    });
                }

                // Flattens an object
                // eg:
                //
                //     var o = {
                //       address: {
                //         street: 'Street',
                //         zip: 1234
                //       }
                //     };
                //
                // becomes:
                //
                //     var o = {
                //       'address.street': 'Street',
                //       'address.zip': 1234
                //     };
                var flatten = function (obj, into, prefix) {
                    into = into || {};
                    prefix = prefix || '';

                    _.each(obj, function (val, key) {
                        if (obj.hasOwnProperty(key)) {
                            if (val && typeof val === 'object' && !(
                              val instanceof Date ||
                              val instanceof RegExp ||
                              val instanceof Backbone.Model ||
                              val instanceof Backbone.Collection)
                            ) {
                                flatten(val, into, prefix + key + '.');
                            }
                            else {
                                into[prefix + key] = val;
                            }
                        }
                    });

                    return into;
                };

                this.init = init;
                this.parse = parse;
                this.bind = Backbone.Validation.bind;
                this.callbacks = Backbone.Validation.callbacks;
                this.flatten = flatten;
            }



            return new UIValidation();
        }
);