﻿define(["jquery",
    "underscore",
    "backbone",
    "api",
    "helper",
    'globalize',
    'uiControlBase',
    "uiPageTransition",
    "uiContainers",
    "uiValidation",
    "uiToast",
    'uiGlobal',
    'preferences',
    'uiModal',
    'uiToast',
    'uiScroll',
    'libHammer', 'tracking', 'logger'],
        function ($, _, Backbone, api,helper, globalize,uiControlBase, uiPageTransition, uiContainers,
            uiValidation, uiToast, uiGlobal, preferences, uiModal, uiToast, uiScroll, Hammer, tracking, logger) {

            var translate = globalize.localizeBase("views.base");
            var translateLocation = globalize.localizeBase("views.location");
            var translateShared = globalize.localizeBase("views.shared");

            var ErrorView = uiControlBase.extend({
                template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-sad icon-6x block\"></i> <%=text %><p><button class=\"btn btn-small btn-primary\"><%=buttonText%></button></p></div></div>"),
                _hammer:null,
                getRenderData: function () {
                    return {
                        text: translate("error.info"),
                        buttonText: translate("error.refresh.button"),
                    }
                },
                onRender: function (el) {
                    if (!this.el) return this;
                    var self = this;

                    this._hammer = Hammer($("button",this.el)[0], {
                     //   prevent_default: true
                    }).on(uiGlobal.event.tap, function (e) {
                        if (self && self.el) {
                            self.el.toggleClass("highlite", true);
                        }
                        self.trigger("select", self);
                    });
                },
                onBeforeDestory: function () {
                    if (this._hammer) {
                        this._hammer.off("tap");
                        this._hammer = null;
                    }
                }
            });

            var View = Backbone.View.extend({
                _isRendered: false,
                _isTransitionExecuting: false,
                _controlContainer:[],
                className: "",
                validatorSelectorMask:"#&",
                title: null,
                subtitle:null,
                access: null,
                model: null,
                _scroller:null,
                
                initialize: function (options) {
                    this._controlContainer = [];

                    this.trackPage(options);
                },
                
                setModel: function (m) {
                    Backbone.Validation.unbind(this, { model: this.model });
                    this.model = m;

                    uiValidation.bind(this, {
                        selector: this.getValidatorSelector
                    });

                    return m;
                },
                saveModel: function (data,options) {
                    var loader = null;

                    if (options) {
                        if (options instanceof $) {
                            loader = options;
                        } else {
                            loader = options.loader;
                        }
                    } 

                    if (loader) loader.toggleClass("loading", true);

                    var res = this.model.save(data);
                    var self = this;
                    if (!res) {
                        res = $.Deferred();
                        res.reject(this.model.validationError);
                        this.onSaveModelValidationError();
                    } else {
                        res.fail(function (xhr) {
                            if (xhr.status == 400) {
                                self.parseServerValidationError(xhr.responseJSON);
                                self.onSaveModelValidationError();
                            }
                            
                        });
                    }

                    return res.always(function () {
                        if (loader) loader.toggleClass("loading", false);
                    });
                },
                onSaveModelValidationError:function(){
                    uiToast.danger(translateShared("validation.message"));
                },
                trackPage:function(){
                    tracking.trackPage();
                },
                fetchModel: function (data, options) {
                    var loader = null;

                    if (options) {
                        if (options instanceof $) {
                            loader = options;
                        } else {
                            loader = options.loader;
                        }
                    } 

                    if (loader) loader.toggleClass("loading", true);

                    var res = this.model.fetch(data);
                    var self = this;
                    if (!res) {
                        res = $.Deferred();
                        res.reject();
                    } 
                    return res.always(function () {
                        if (loader) loader.toggleClass("loading", false);
                    });;
                },

                fetch: function (data, model, options) {
                    var loader = null;
                    
                    if (options) {
                        if (options instanceof $) {
                            loader = options;
                        } else {
                            loader = options.loader;
                        }
                    } 

                    if (loader) loader.toggleClass("loading", true);

                    if (typeof model === "undefined") model = this.model;

                    var res = null;
                    if (typeof model === 'function') {
                        res = model();
                    } else {
                        res = model.fetch(data);
                    }

                    var self = this;
                    if (!res) {
                        res = $.Deferred();
                        res.reject();
                    }
                     
                    //if (options && options.onBegin) {
                    //    options.onBegin();
                    //} else if (uiContainers) {
                    //    uiContainers.header.get().toggleClass("loading", true);
                    //}

                    return res.fail(function (data) {
                        self.onError(data);
                    }).always(function () {
                        if (loader) loader.toggleClass("loading", false);
                    });
                },

                parseServerValidationError: function (errors) {
                    if (errors && errors.error) {
                        // global model errors
                        $.each(errors.error, _.bind(function (key, value) {
                            uiValidation.callbacks.invalid(this, "", value, this.getValidatorSelector);
                        }, this));
                    }
                    
                    if (errors && errors.errors) {
                        // property errors
                        $.each(errors.errors, _.bind(function (key, value) {
                            uiValidation.callbacks.invalid(this, key, value, this.getValidatorSelector);
                        }, this));
                    }
                },

                getValidatorSelector: function (attr) {
                    return this.validatorSelectorMask.replace("&", attr);
                },

                renderTemplate: function () {
                    return "";
                },
                onNotAuthenticated: function(){
                    uiToast.danger(translate("not-authenticated-message"));
                },
                onError:function(error,View,viewData){
                    if (!this.$el) return;

                    this.$el.empty();

                    if (typeof View === 'undefined') View = ErrorView;

                    var error = new View(viewData || {});
                    this.registerControl(error);
                    this.render(error);

                    if (error instanceof ErrorView) {
                        error.on("select", _.bind(function () {
                            this.reload();
                        }, this));
                    }
                },
                render: function (control,container) {
                    if (!this.$el) return this;
                    if (this.access == "auth" && !api.isAuthenticated()) {
                        if (this.onNotAuthenticated) {
                            this.onNotAuthenticated();
                        }
                        this.navigate("login");
                        return this;
                    }

                    if (control && control.createIn) {
                        control.createIn(null, container||this.$el);
                        this.registerControl(control);
                    } else {
                        var html = this.renderTemplate();
                        if (html === null) return this;
                        this.$el.html(html);
                    }

                    if (!this._isRendered) {

                        this.appendToContainer();
                        this.initializeTapEvents();
                       
                        this._executeTransition();

                        this.initializeSnapLeft(uiContainers.snap);
                        this.initializeSnapRight(uiContainers.snap);
                        this.initializeHeader(uiContainers.header);
                        this.initializeFooter(uiContainers.footer);
                        this.initializeHeaderSec(uiContainers.headerSec);

                        if (this.onpageinitdone) this.onpageinitdone();

                    }

                  //  this.triggerPageContentChanged();

                    if (this.onpagerenderdone) this.onpagerenderdone();

                    // scroll to top
                    // uiGlobal.$content.scrollTop(0);
                  //  this.applyScroll();
                    this._isRendered = true;

                    return this;
                },
                initializeTapEvents: function () {
                    if (!this.$el) return this;
                    this.$el.hammer({
                        prevent_default: false
                    });

                    return this;
                },
                appendToContainer:function(){
                    uiContainers.main.get().append(this.$el);
                },
                reload:function(){
                    this.render();
                },
                initializeSnapLeft: function (o) {
                    o.viewLeftDefault();
                },
                applyScroll: function (el) {
                    if (!el)
                    {
                        el = this.$el;
                    }
                    if (this._scroller) {
                        logger.log("scroll refresh");
                        this._scroller.refresh();
                    } else {
                        logger.log("scroll refresh");
                        this._scroller = new uiScroll(el[0], {});
                    }
                },
                initializeSnapRight: function (o) {
                    var self = this;
                    o.viewRightDefault();
                },

                initializeHeader: function (o) {
                    var s = uiContainers.snap;
                    var self = this;

                 //   o.show();
                    o.viewDefault({
                        refresh:{
                            visible:(!!this.onRefresh)
                        },
                        back:{
                            visible: (!!this.onBack && (typeof this.isBackVisible === 'undefined' || this.isBackVisible())),
                            title: translateShared("back")
                        },
                        title: (typeof this.title === 'function') ? this.title() : this.title,
                        subtitle: (typeof this.subtitle === 'function') ? this.subtitle() : this.subtitle,
                        onSettings: function () {
                            s.toggleLeft().always(function () {
                                logger.log("Toggle left done");
                            });
                        },
                        onRefresh: function () {
                            self.onRefresh();
                        },
                        onBack: function () {
                            self.onBack();
                        },
                        onTitle: function () {
                            if (self.onTitleSelect) self.onTitleSelect();
                        },
                        onFilter: function () {
                            self.navigate("offer/search-filter");
                        }
                    });
                },
                initializeHeaderSec: function (o) {
                    o.viewNone();
                },
                initializeFooter: function (o) {
                  //  o.show();
                    o.viewDefault();
                },

                openLocationPicker: function () {
                    Backbone.Notifications.trigger("locationselect",this);
                },

                triggerPageContentChanged: function () {
                    this.applyScroll();
                   // uiGlobal.$document.trigger("pagecontentchanged", this);
                },

                _executeTransition: function () {
                  //  if (this._isTransitionExecuting) return;
                    //   this._isTransitionExecuting = true;
                    return uiPageTransition.slideView(this).always(_.bind(function (previousView, currentView) {
                        //  this._isTransitionExecuting = false;
                        if (previousView) {
                            previousView.destroy();
                        }
                        if (currentView) {
                            
                            currentView.applyScroll();
                        }
                       
                    }, this));
                },
                destroy: function () {
                    if (this.onremoving) this.onremoving();

                    this.hideKeyboard();

                    Backbone.Validation.unbind(this, { model: this.model });
                   
                    this.remove();
                    this.unbind();
                    this.unregisterControls();
                    return this;
                },
                navigateBack: function () {
                    Backbone.Notifications.trigger("navigateback");
                },
                navigate: function (route,data) {
                    Backbone.Notifications.trigger("navigate", route,data);
                },
                nl2br: function (str, is_xhtml) {
                    return helper.nl2br(str, is_xhtml);
                },
                registerControl:function(control){
                    if (!control || !this._controlContainer) return this;
                    this._controlContainer.push(control);

                    return this;
                },
                unregisterControls: function () {
                    
                    if (this._controlContainer && this._controlContainer.length > 0) {
                        logger.log("unreg: " + this.cid + " " + this._controlContainer.length + " " + this.title);
                        this._controlContainer = $.map(this._controlContainer, function (o) {
                            if (o || o.destroy) {
                                o.destroy();
                                o = null;
                                return null;
                            }
                            return o;
                        });
                    };
                    return this;
                },
                show: function () {
                    if (this.$el) {
                        uiGlobal.toggleClass(this.$el, "invisible", false);
                    }
                },
                hide: function () {
                    if (this.$el) {
                        uiGlobal.toggleClass(this.$el, "invisible", true);
                    }
                },
                hideKeyboard: function () {
                    // blur current focused element (hides phone keyboard if opened)
                    if (uiGlobal.document.activeElement || uiGlobal.document.activeElement.blur) {
                        uiGlobal.document.activeElement.blur();
                    }
                    if (this.$el) {
                        $("input", this.$el).blur();
                        this.$el.focus();
                    }
                }
            });

            return View;
        }
);

