﻿define([
  'jquery',
  'underscore',
  'views/base',
  'template',
  'api',
  'helper',
  'uiToast',
  'uiContainers',
  'uiControlSelectList',
  'globalize',
  'uiControlViewList',
  'uiControlButton',
  'uiControlTile',
  'settings'
], function ($, _, ViewBase, template, api,helper,uiToast, uiContainers, SelectListControl, globalize, uiControlViewList, uiControlButton,uiControlTile, settings) {

    var translateShared = globalize.localizeBase("views.shared");
    var translate = globalize.localizeBase("views.category");
    var listTemplate = _.template("<div class=\"list\"></div>");

    var ACTION_SELECT_PARENT = 1;
    var ACTION_GOTO_PARENT = 2;

    var ButtonControl = uiControlButton.extend({
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                data: null,
                title: null,
                icon: null,
                iconSec: null,
                markerIcon: null,
                selected: false,
                cssClass: null
            }, options);
        },
        template: template.controls.selectListItem,
        getRenderData: function () {

            return {
                title: this.options.title,
                icon: this.options.icon,
                iconSec: this.options.iconSec,
                markerIcon: this.options.markerIcon,
                selected: this.options.selected,
                cssClass: this.options.cssClass
            };
        }
    });

    var View = ViewBase.extend({
        location: null,
        listControl: null,
        currentItemControl:null,
        options: null,
        currentLevel: null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                maxLevel: 99999,
                minLevel: 0,
                selectLastLevel: false,
                category: null,
                rootLevelName: ""
            }, options);

            this.category = this.options.category || this.creteDefaultCategory();

            if (this.category && this.category.level) {
                this.currentLevel = this.ensureValidLevel(this.category.level);
            }

            var self = this;

            this.listControl = new uiControlViewList({
                template: listTemplate,
                scrollElement: this.$el,
                enableMore: false,
                createItem: function (item) {
                    if (item.action == ACTION_GOTO_PARENT) {
                        return new ButtonControl({
                            title: item.name,
                            icon: "icon icon-chevron-up",
                            data: item
                        });
                    }
                    
                    return new ButtonControl({
                        title: item.name,
                        iconSec: (item.childrenCount > 0 && !item.action) ? "icon icon-chevron" : null,
                        markerIcon: "icon-radiobox",
                        selected: item.selected,
                        icon: helper.toCategoryCssClass(item),
                        data: item
                    });
                }
            });

            var loader = null;

            this.listControl.on("loadstart", function (data, opt) {

            }).on("loadend", function (data, opt,parent) {
                
                if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $("<div />").addClass("content-fixed");

                    if (parent && !self.currentItemControl && !self.options.selectLastLevel) {
                        self.currentItemControl = self.createCategoryTile(parent, container);
                    }
                    self.render(self.listControl, container);

                    self.$el.empty().append(container);

                }

                if (parent && self.currentItemControl) {
                    self.currentItemControl.value(parent);
                }

                self.triggerPageContentChanged();
            }).on("select", function (data) {
                if (data && data.options && data.options.data) {
                    self.onSelect(data.options.data);
                }
            }).on("loaderror", function () {
                self.onError();
            });

            this.load(this.category, this.currentLevel);
        },
        createCategoryTile:function(category,container){
            var button = new uiControlTile({
                content: function (val) {
                    return val.name;
                    //return translate("offers-in-category.button").replace("[NAME]", category.name);
                },
                cssClass: function (val) {
                    return "primary card space-vertical no-border";
                },
                mediaIcon: function (val) {
                    if (!val || !val.id) return "icon icon-category";
                    return helper.toCategoryCssClass(val);
                }
            });

            button.value(category);
            this.registerControl(button);
            button.create(container);

            button.off("select").on("select", _.bind(function (el, val) {
                if (val) this.onSelect({
                        id: val.id,
                        level: val.level,
                        name: val.name,
                        childrenCount: val.childrenCount,
                        effectiveMetaData: val.effectiveMetaData,
                        action: ACTION_SELECT_PARENT,
                        category: val,
                        selected: (this.category.id == val.id)
                    });
            }, this));

            return button;
        },
        reload: function () {
            this.load(this.category, this.currentLevel);
        },
        ensureValidLevel: function (level) {
            if (level < this.options.minLevel) return this.options.minLevel;
            if (level > this.options.maxLevel) return this.options.maxLevel;

            return level;
        },
        renderTemplate: function () {
            return "";
        },
        onSelect: function (data) {
            if (!data) return;

            // was "previous level" selected?
            if (data.action == ACTION_GOTO_PARENT) {
                // load next level
                //this.currentLevel = this.ensureValidLevel(this.currentLevel - 1);
                //if (this.category.level == this.currentLevel + 1) this.category = this.category.parent || this.creteDefaultCategory();
                this.category = data.category;
                this.load(data.category, data.level);
                return;
            }

            if (data.action == ACTION_SELECT_PARENT) {
                //if (this.category.level == this.currentLevel - 1) {
                    this.selectCategory(data.category);
               // } else {
                    //this.selectCategory(this.category.parent || this.creteDefaultCategory());
               // }

                return;
            }

            if (data.level == this.category.level) {
                this.category = $.extend({}, this.category, {
                    id: data.id,
                    name: data.name,
                    level: data.level,
                    effectiveMetaData: data.effectiveMetaData,
                    childrenCount:data.childrenCount
                });
            } else {
                this.category = $.extend({}, {
                    parent: this.category
                }, {
                    id: data.id,
                    name: data.name,
                    level: data.level,
                    effectiveMetaData: data.effectiveMetaData,
                    childrenCount: data.childrenCount
                });
            }

            if (this.currentLevel >= this.options.maxLevel || data.level >= this.options.maxLevel || data.childrenCount == 0) {
                this.selectCategory(this.category);
                return;
            }
            if (this.options.selectLastLevel && data.childrenCount == 0) {
                this.selectCategory(this.category);
                return;
            }

            // load next level
            this.currentLevel = this.ensureValidLevel(data.level + 1);

            this.load(this.category, this.currentLevel);
        },
        selectCategory: function (data) {
            if (this.options.selectLastLevel) {
                if (!data || data.childrenCount != 0) {
                    if (!data) {
                        uiToast.danger(translate("select.info"));
                    } else {
                        uiToast.danger(translate("select-in.info").replace("[CATEGORY]",data.name));
                    }
                    
                    return;
                }
            }

            if (this.options && this.options.onSelect) {
                this.options.onSelect(data);
            }
            this.navigateBack();
        },
        mapCategoryToApiModel: function (val, requestedLevel) {
            if (val && !val.childrenCount) {
                val = val.parent;
            } else if (val && val.level == requestedLevel +1) {
                val = val.parent;
            }

            return {
                withChildren: true,
                withParent:false,
                id: (val) ? val.id : null
            }
        },
       
        load: function (category,requestedLevel) {
            var self = this;
            uiContainers.header.loading(true);
            return this.listControl.load(function () {

                var apiModel = self.mapCategoryToApiModel(category, requestedLevel);
                
                var deferred = $.Deferred();
                api.collection.category(apiModel).done(function (data) {
                    var source = [];
                    var result = [];
                    var level = 0;
                    var parent = null;
                    
                    if (data.length > 0) {
                        source = data[0].children;
                        
                        parent = data[0].value;
                        if (parent) {
                            parent.level = helper.getNodeLevel(parent);
                        } else {
                            parent = self.creteDefaultCategory();
                        }
                        
                        level = parent.level + 1;
                    }
                    parent = parent || self.creteDefaultCategory();

                    result = $.map(source, function (o, i) {
                        return {
                            id: o.id,
                            name: o.name,
                            childrenCount: o.childrenCount,
                            effectiveMetaData: o.effectiveMetaData,
                            level: level,
                            selected: (self.category && self.category.id == o.id)
                        }
                    });

              //      var parent = ((self.category.id == rootId) ? self.category : self.category.parent) || self.creteDefaultCategory();
                    if (!self.options.selectLastLevel) {
                        //result.splice(0, 0, {
                        //    id: parent.id,
                        //    level: parent.level,
                        //    name: parent.name,
                        //    childrenCount: parent.childrenCount,
                        //    effectiveMetaData:parent.effectiveMetaData,
                        //    action: ACTION_SELECT_PARENT,
                        //    category: parent,
                        //    selected: (self.category.id == parent.id)
                        //});
                    }

                    if (parent.level > self.options.minLevel && parent) {
                        var name = "";
                        var parentCatName = "";
                        if (parent.level > 1) {
                            name = translate("previous.button");
                        } else {
                            name = translate("previous.root.button");
                        }

                        if (parent.parent) {
                            parentCatName = parent.parent.name;
                        } else {
                            parentCatName = parent.name;
                        }

                        result.splice(0, 0, {
                            level: self.ensureValidLevel(parent.level - 1),
                            category: parent,
                            action: ACTION_GOTO_PARENT,
                            name: name.replace("[NAME]", parentCatName)
                        });
                    }

                    deferred.resolve(result,parent);
                }).fail(function (err) {
                    deferred.reject(err);
                });

                return deferred;

            }, {
                empty: true
            }).always(function () {
                uiContainers.header.loading(false);
            });
        },
        mapApiResultToModel: function (result) {
            if (!result) return null;

            var items = [];

            if (result.country) items.push(result.country);
            if (result.region) items.push(result.region);
            if (result.city) items.push(result.city);
            if (result.neighbourhood) items.push(result.neighbourhood);

            var value = null;
            _.each(result, function (o) {
                value = $.extend({}, { parent: value }, o.value);
            });

            return value;

        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        creteDefaultCategory: function () {
            return {
                id: null,
                name: this.options.rootLevelName,
                level: 0
            };
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                onOk: _.bind(function (e) {
                    this.selectCategory(this.category);
                }, this),
                onCancel: _.bind(function (e) {
                    this.navigateBack();
                }, this)
            });
        }

    });
    return View;
});