﻿define([
  'jquery',
  'underscore',
  'views/base',
  'template',
  'globalize',
  'uiControlViewList',
  'uiControlButton'
], function ($, _, ViewBase, template, globalize, uiControlViewList, uiControlButton) {

    var translateShared = globalize.localizeBase("views.shared");
    var translate = globalize.localizeBase("views.category");
    var listTemplate = _.template("<div class=\"list\"></div>");

    var ButtonControl = uiControlButton.extend({
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                data: null,
                type:"single"
            }, options);
        },
        template: template.controls.selectListItem,
        getRenderData: function () {

            return {
                title: this.options.title,
                icon: null,
                iconSec: (this.options.hasMore?"icon icon-chevron":null),
                markerIcon: (this.options.type=='multiple'?"icon-checkbox":"icon-radiobox"),
                selected: this.options.selected,
                cssClass:""
            };
        },
        selected: function (val) {
            if (typeof val === 'undefined') return this.options.selected;
            this.options.selected = val;
            if (this.options.data) this.options.data.selected = val;
            if (this.created()) {
                this.el.toggleClass("selected", val);
            }
        }
    });
    
    var View = ViewBase.extend({
        listControl: null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            
            // set option default
            this.options = $.extend({}, {
                type: "single",
                title: "",
                subTitle: "",
                enableMore: false,
                list: [],
                value:null
            }, options);

            var self = this;

            this.title = this.options.title;
            this.subTitle = this.options.subTitle;

            this.listControl = new uiControlViewList({
                template: listTemplate,
                scrollElement: this.$el,
                enableMore:this.options.enableMore,
                createItem: function (item) {
          
                    return new ButtonControl({
                        id:item.id,
                        title: item.title,
                        selected: item.selected,
                        data: item,
                        type: self.options.type
                    });
                }
            });

            this.listControl.on("loadstart", function (data, opt) {
                
            }).on("loadend", function (data, opt) {
                if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $("<div />").addClass("content-fixed");
                    self.$el.empty().append(container);
                    self.render(self.listControl, container);
                }
            }).on("select", function (data) {
                if (self.isMultiple() && data) {
                    data.selected(!data.selected());
                } else if (data && data.options && data.options.data) {
                    self.onSelect(data.options.data);
                }
            }).on("loaderror", function (data) {
                self.onError();
            });

            this.load();
        },
        renderTemplate: function () {
            return "";
        },
        onpagerenderdone:function(){

        },        
        onSelect: function (data) {
            if (!this.isMultiple()) {
                this.save(data);
            }
        },
        load: function () {
            var self = this;
            
            return this.listControl.load(self.options.list, {
                empty: true
            }).done(function (data) {
                
                self.applyScroll();
            });
        },
        initializeFooter: function (o) {
            o.hide();
        },
        save: function (data) {
            var values = null;
            if (this.options && this.options.onSelect) {
                if (this.isMultiple()) {
                    values = _.filter(this.listControl.list, function (o) {
                        return o.selected;
                    });
                } else {
                    values = data;
                }
            }
            this.options.onSelect(values);
            this.navigateBack();
        },
        initializeHeader: function (o) {
           
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                onOk: _.bind(function (e) {
                    this.save();
                }, this),
                onCancel: _.bind(function (e) {
                    this.navigateBack();
                }, this)
            });
        },
        isMultiple: function () {
            return (this.options.type == "multiple");
        }
       
    });
    return View;
});