﻿define([
  'jquery',
  'underscore',
  'globalize',
  'views/base',
  'api',
  'uiTimeAgo',
  'template',
  'uiGlobal',
  'helper',
  'preferences',
  'uiControlViewSlider',
  'uiSwipe',
  'uiControlButton',
  'uiControlLoader'
], function ($, _, globalize, ViewBase, api, uiTimeago, template, uiGlobal, helper, preferences, uiControlViewSlider, uiSwipe, uiControlButton, uiControlLoader) {

    var View = ViewBase.extend({
        location: null,
        sliderControl:null,
        initialize: function () {
            this.constructor.__super__.initialize.call(this);

            this.location = preferences.location();

            this.sliderControl = new uiControlViewSlider();

            this.title = _.bind(function () {
                var title = globalize.localize("views.location.iran");
                if (this.location && this.location.name && this.location.id) title = this.location.name;
                return "<i class=\"icon icon-location\"></i> " + title;
            }, this);

            this.load();

        },
        reload:function(){
            this.load();
        },
        load: function () {
            var loader = new uiControlLoader();
            this.$el.html('');
            this.render(loader);

            var self = this;

            api.offer.category({
                regionId: helper.getLocationRegionId(this.location),
                categoryId: null
            }).done(_.bind(function (data) {
                if (loader) loader.destroy();
                this.loadCategoryList(data.category, data.children);
            }, this));
        },
        loadCategoryList: function (category, list) {
            if (!this.$el) return;
            var self = this;
            var container = $("<div />");
                       
            container.addClass("category-list-big");
           
            _.each(list, _.bind(function (o) {
                var button = new uiControlButton();
                self.registerControl(button);
               
                button.createIn({
                    title: o.name,
                    id: o.id,
                    icon: helper.toCategoryCssClass(o),
                    childrenCount: o.childrenCount
                }, container);
                

                button.on("select", function () {
                    self.navigate("offer/category/" + this.options.id);
                });
            }, this));

            this.$el.empty().append($("<div />").addClass("content-fixed").append(container));
        },
        onpagerenderdone: function () {
            //var categoryContainer = this.$(".category-list-big");

            //var categoryItemTemplate = _.template("<a data-href=\"<%=link%>\"><i class=\"<%=icon%>\"></i><label><%=name%></label></a>");
            
            //api.offer.category({
            //    regionId: (this.location) ? this.location.id : null,
            //    categoryId:null
            //}).done(function (data) {
                
            //    //if (data.length > 0) {
            //    //    data = data[0].children || [];
            //    //}
            //    var html = "";
            //    if (!data || !data.children) return;
            //    $.each(data.children, function (i,o) {
            //        html += categoryItemTemplate({
            //            icon: helper.toCategoryCssClass(o),
            //            name: o.name,
            //            link: "#offer/category/" + o.id
            //        });
            //    });
                
            //    categoryContainer.html(html);
            //});

        //    this.loadDiscover();
        },
        loadDiscover: function () {
            if (!this.sliderControl) return;

            // get hot offers
            api.offer.get({
                    regionId: (this.location) ? this.location.id : null,
                    pageSize:5
            }).done(_.bind(function (data) {
                this.renderDiscover(data.items);
            },this));

            
        },
        renderDiscover: function (data) {
            
            var discoverContainer = this.$(".offer-discover-list");
            var width = $(this.el).children().width();
            var height = Math.floor(uiGlobal.$body.height()*0.3);
            this.sliderControl.createIn({
                height: height,
                width: width,
                items: $.map(data, function (o) {

                    var result =  {
                        image: null
                    }

                    if (!o || !o.images || o.images.length == 0) return result;

                    result.image = api.offer.imageUrl(o.images[0].key, {
                        height: height,
                        width: width,
                        mode: 3
                    });

                    return result;
                })
            }, discoverContainer);
        },
        onremoving: function () {
            if (this.sliderControl) {
                this.sliderControl.destroy();
                this.sliderControl = null;
            }
        },
        onTitleSelect: function () {
            this.openLocationPicker();
        }
        
    });
    return View;
});