﻿define([
  'jquery',
  'underscore',
  'views/base',
  'template',
  'api',
  'uiToast',
  'uiContainers',
  'uiControlSelectList',
  'globalize',
  'uiControlViewList',
  'uiControlButton',
  'uiControlTile',
  'settings'
], function ($, _, ViewBase, template, api,uiToast, uiContainers, SelectListControl, globalize, uiControlViewList, uiControlButton,uiControlTile, settings) {

    var translateShared = globalize.localizeBase("views.shared");
    var translate = globalize.localizeBase("views.location");
    var listTemplate = _.template("<div class=\"list\"></div>");

    var LEVEL_COUNTRY = 1;
    var LEVEL_REGION = 2;
    var LEVEL_CITY = 3;
    var LEVEL_NEIGHBOURHOOD = 4;

    var ACTION_SELECT_PARENT = 1;
    var ACTION_GOTO_PARENT = 2;

    var ButtonControl = uiControlButton.extend({
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = $.extend({}, {
                data: null,
                title: null,
                icon: null,
                iconSec: null,
                markerIcon: null,
                selected: false,
                cssClass: null
            }, options);
        },
        template: template.controls.selectListItem,
        getRenderData: function () {

            return {
                title: this.options.title,
                icon: this.options.icon,
                iconSec: this.options.iconSec,
                markerIcon: this.options.markerIcon,
                selected: this.options.selected,
                cssClass: this.options.cssClass
            };
        }
    });

    var View = ViewBase.extend({
        location: null,
        listControl: null,
        currentItemControl: null,
        options: null,
        currentLevel: LEVEL_REGION,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            
            this.options = $.extend({}, {
                maxLevel: LEVEL_NEIGHBOURHOOD,
                minLevel: LEVEL_REGION,
                selectLastLevel: false,
                location: null,
                rootLevelName: translate("iran")
            },options);

            this.location = this.options.location || this.creteDefaultLocation();

            if (this.location && this.location.level) {
                this.currentLevel = this.ensureValidLevel(this.location.level);

                // if region has no parent info
                if (this.location.level == LEVEL_REGION && !this.location.parent) {
                    this.location.parent = this.creteDefaultLocation();
                }
            }

            var self = this;

            this.listControl = new uiControlViewList({
                template: listTemplate,
                scrollElement: this.$el,
                enableMore:false,
                createItem: function (item) {
                    if (item.action == ACTION_GOTO_PARENT) {
                        return new ButtonControl({
                            title: item.name,
                            icon: "icon icon-chevron-up",
                            data: item
                        });
                    }

                    return new ButtonControl({
                        title: item.name,
                        iconSec: (item.childrenCount > 0 && item.level < self.options.maxLevel) ? "icon icon-chevron" : null,
                        markerIcon: "icon-radiobox",
                        selected: item.selected,
                        data: item
                    });
                }
            });

            var loader = null;

            this.listControl.on("loadstart", function (data, opt) {
                
            }).on("loadend", function (data, opt,parent) {
                if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $("<div />").addClass("content-fixed");

                    if (parent && !self.currentItemControl && !self.options.selectLastLevel && self.options.minLevel < self.options.maxLevel) {
                        self.currentItemControl = self.createLocationTile(parent, container);
                    }   
                    
                    self.render(self.listControl, container);
                    self.$el.empty().append(container);
                }

                if (parent && self.currentItemControl) {
                    self.currentItemControl.value(parent);
                }

                self.triggerPageContentChanged();

            }).on("select", function (data) {
                if (data && data.options && data.options.data) {
                    self.onSelect(data.options.data);
                }
            }).on("loaderror", function (data) {
                self.onError();
            });
            
            this.load(this.mapLocationToQuery(this.location,this.currentLevel));
        },
        createLocationTile: function (location, container) {
            var button = new uiControlTile({
                content: function (val) {
                    return val.name;
                    //return translate("offers-in-category.button").replace("[NAME]", category.name);
                },
                cssClass: function (val) {
                    return "primary card space-vertical no-border";
                },
                mediaIcon: function (val) {
                    return"icon icon-location";
                }
            });

            button.value(location);
            this.registerControl(button);
            button.create(container);

            button.off("select").on("select", _.bind(function (el, val) {
                if (val) this.onSelect({
                    id: val.id,
                    level: val.level,
                    name: val.name,
                    childrenCount: val.childrenCount,
                    action: ACTION_SELECT_PARENT,
                    selected: (val.level == self.location.level && self.location.id == val.id)
                });
            }, this));

            return button;
        },
        reload: function () {
            this.load(this.mapLocationToQuery(this.location, this.currentLevel));
        },
        mapLocationToQuery: function (location,requestedLevel) {
            if (!location || requestedLevel == LEVEL_COUNTRY || requestedLevel == LEVEL_REGION || !location.parent) {
                return {
                    level: LEVEL_COUNTRY
                }
            }

            if (requestedLevel == location.level && location.parent) {
                return {
                    level: location.parent.level,
                    id: location.parent.id
                }
            }

            return {
                level: requestedLevel - 1,
                id:location.id
            }
        },
        ensureValidLevel:function(level){
            if (level < this.options.minLevel) return this.options.minLevel;
            if (level > this.options.maxLevel) return this.options.maxLevel;

            return level;
        },
        renderTemplate: function () {
            return "";
        },
        onpagerenderdone:function(){

        },
        onSelect: function (data) {
            if (!data) return;

            // was "previous level" selected?
            if (data.action == ACTION_GOTO_PARENT) {
                // load next level
                this.currentLevel = this.ensureValidLevel(this.currentLevel - 1);
                if (this.location.level == this.currentLevel + 1) this.location = this.location.parent || this.creteDefaultLocation();

                this.load(this.mapLocationToQuery(this.location, this.currentLevel));
                return;
            }

            if (data.action == ACTION_SELECT_PARENT) {
                if (this.location.level == this.currentLevel - 1) {
                    this.selectLocation(this.location);
                } else {
                    this.selectLocation(this.location.parent||this.creteDefaultLocation());
                }
                
                return;
            }
            
            if (data.level == this.location.level) {
                this.location = $.extend({}, this.location, {
                    id: data.id,
                    name: data.name,
                    level: data.level,
                    childrenCount: data.childrenCount
                });
            } else {
                this.location = $.extend({}, {
                    parent: this.location
                }, {
                    id: data.id,
                    name: data.name,
                    level: data.level,
                    childrenCount: data.childrenCount
                });
            }
            
            if (this.currentLevel >= this.options.maxLevel || data.level >= this.options.maxLevel || data.childrenCount ==0) {
                this.selectLocation(this.location);
                return;
            }
            if (this.options.selectLastLevel && data.childrenCount ==0) {
                this.selectLocation(this.location);
                return;
            }

            // load next level
            this.currentLevel = this.ensureValidLevel(data.level + 1);
            
            this.load(this.mapLocationToQuery(this.location, this.currentLevel));
        },
        selectLocation: function (data) {
            if (this.options.selectLastLevel) {
                if (!data || (data.childrenCount != 0 && data.level < this.options.maxLevel)) {
                    if (!data) {
                        uiToast.danger(translate("select.info"));
                    } else {
                        if (data.level == LEVEL_REGION) {
                            uiToast.danger(translate("select-in-region.info").replace("[LOCATION]",data.name));
                        } else if (data.level == LEVEL_CITY) {
                            uiToast.danger(translate("select-in-city.info").replace("[LOCATION]",data.name));
                        } else {
                            uiToast.danger(translate("select.info"));
                        }
                    }

                    return;
                }
            }

            if (this.options && this.options.onSelect) {
                this.options.onSelect(data);
            }
            this.navigateBack();
        },
        load: function (val) {
            var self = this;
            uiContainers.header.loading(true);
            return this.listControl.load(function () {

                var parameters = {
                    withChildren: true,
                    region: (val && val.level == LEVEL_REGION)?val.id:null,
                    city: (val && val.level == LEVEL_CITY) ? val.id : null,
                    neightbourhood: (val && val.level == LEVEL_NEIGHBOURHOOD) ? val.id : null
                }

                var deferred = $.Deferred();
                api.collection.location(parameters).done(function (data) {
                    var source = [];
                    var result = [];
                    var level = LEVEL_COUNTRY;
                    
                    if (val.level == LEVEL_COUNTRY && data.country) {
                        source = data.country.children;
                        level = LEVEL_REGION;
                    }
                    if (val.level == LEVEL_REGION && data.region) {
                        source = data.region.children;
                        level = LEVEL_CITY;
                    }
                    if (val.level == LEVEL_CITY && data.city) {
                        source = data.city.children;
                        level = LEVEL_NEIGHBOURHOOD;
                    }

                    result = $.map(source, function (o, i) {
                        return {
                            id: o.id,
                            name: o.name,
                            childrenCount: o.childrenCount||0,
                            level: level,
                            selected: (self.location && self.location.level == level && self.location.id == o.id)
                        }
                    });
                   
                    var parent = ((self.location.level == level - 1) ? self.location : self.location.parent) || {
                        id: settings.countryIdIran,
                        name: self.options.rootLevelName,
                        level:LEVEL_COUNTRY
                    }
                    
                    if (!self.options.selectLastLevel && self.options.minLevel == self.options.maxLevel) {
                        result.splice(0, 0, {
                            id: parent.id,
                            level: parent.level,
                            name: parent.name,
                            childrenCount: parent.childrenCount,
                            action: ACTION_SELECT_PARENT,
                            selected: (parent.level == self.location.level && self.location.id == parent.id)
                        });
                    }

                    if (self.currentLevel > self.options.minLevel && parent) {
                        var name = "";
                        
                        if (parent.level == LEVEL_REGION) {
                            name = translate("previous.region.button");
                        }
                        if (parent.level == LEVEL_CITY) {
                            name = translate("previous.city.button");
                        }
                        
                        result.splice(0, 0, {
                            action: ACTION_GOTO_PARENT,
                            name: name.replace("[NAME]",parent.name)
                        });
                    }

                    deferred.resolve(result,parent);
                }).fail(function (err) {
                    deferred.reject(err);
                });

                return deferred;

            }, {
                empty: true
            }).always(function () {
                uiContainers.header.loading(false);
                
                if (val && val.level == LEVEL_COUNTRY) uiContainers.header.title(translate("title.region"));
                if (val && val.level == LEVEL_REGION) uiContainers.header.title(translate("title.city"));
                if (val && val.level == LEVEL_CITY) uiContainers.header.title(translate("title.neighbourhood"));
            });
        },
        mapApiResultToModel:function(result){
            if (!result) return null;

            var items = [];
            
            if (result.country) items.push(result.country);
            if (result.region) items.push(result.region);
            if (result.city) items.push(result.city);
            if (result.neighbourhood) items.push(result.neighbourhood);
            
            var value = null;
            _.each(result, function (o) {
                value = $.extend({}, {parent:value},o.value);
            });

            return value;

        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        creteDefaultLocation:function(){
            return {
                id: settings.countryIdIran,
                name: this.options.rootLevelName,
                level: LEVEL_COUNTRY
            };
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                onOk: _.bind(function (e) {
                    this.selectLocation(this.location);
                }, this),
                onCancel: _.bind(function (e) {
                    this.navigateBack();
                }, this)
            });
        }
       
    });
    return View;
});