﻿define([
  'jquery',
  'underscore',
  'globalize',
  'api',
  'models/login',
  'views/base',
  'uiToast',
  'template',
  'uiGlobal',
  'helper'
], function ($, _, globalize, api, Model, ViewBase, uiToast, template, uiGlobal, helper) {

    var translate = globalize.localizeBase("views.login");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        events: {
            "submit form": "save"
        },
        options:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            this.options = options;
            this.title = translate("title");
            this.setModel(new Model());
            this.render();
        },
        renderTemplate: function () {
            return template.login({
                translate:translate
            });
        },
        save: function (e) {
            var self = this;
            if (e) e.preventDefault();
            var res = this.saveModel({
                email: this.$("#email").val(),
                password: this.$("#password").val()
            }, this.$("form")).done(function (data) {
                if (!data.isConfirmed) {
                    uiToast.warning(translate("confirm-account.info"));
                    return;
                }

                api.authTicket(data.ticket);
                //uiToast.open(translate("welcome-message").replace("[USER]", data.user.nickname));
                uiToast.success(translate("welcome-message"));
                if (self.options && self.options.onSuccess) {
                    self.options.onSuccess();
                } else {
                    self.navigateBack();
                }
            });

        },
        onpagerenderdone: function () {
            // this.$("#email").focus();
            var elements = $("[data-href]", this.$el);
            elements.off(".login").hammer({
               // prevent_default: true
            }).on(uiGlobal.event.tap + ".login", function (e) {
                e.preventDefault();
                helper.handleNavigation($(this));
            });
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                okEnabled: false,
                onCancel: $.proxy(function () {
                    this.navigate('');
                },this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        onremoving: function () {
            // destroy current controls
            if (this.$el) {
                $("[data-href]", this.$el).off(".login");
            }
        },
        getValidatorSelector: function (attr) {
            if (attr == '') {
                return "#email";
            }
            return "#" + attr;
        }
        
    });
    return View;
});