﻿define([
  'jquery',
  'backbone',
  'views/base',
  'uiToast',
  'template',
  'settings',
  'globalize',
  'api',
  'helper',
  'device',
  'uiGlobal'
], function ($, Backbone, ViewBase, uiToast, template, settings, globalize, api, helper, deviceAdapter, uiGlobal) {

    var translate = globalize.localizeBase("views.me.about-device");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        initialize: function () {
            this.constructor.__super__.initialize.call(this);
            this.title = translate("title");
            this.load();
        },
        load:function(){
            this.render();
            api.application.info().always(_.bind(function () {
                this.render();
                this.checkUpdate();
            },this));
        },
        renderTemplate: function () {

            return template.me.aboutDevice({
                items: this.getInfo()
            });
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("back"),
                okEnabled: false
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        checkUpdate:function(){

            var hasUpdate = helper.isVersionGreater(settings.version, settings.appVersion);
            if (hasUpdate && settings.source.location) {
                Backbone.Notifications.trigger("updateready", settings);
            }

        },
        getInfo: function () {
            var result = [];

            var installedVersionString = this.toVersionString(settings.version);
           
            if (helper.isVersionGreater(settings.version , settings.appVersion) && settings.source.location) {
                installedVersionString = installedVersionString + " <a target=\"_system\" href=\"" + settings.source.location + "\">install new version</a>";
            }

            result.push({
                name: "Installed version",
                values: installedVersionString,
            });

            result.push({
                name: "Latest version",
                values: this.toVersionString(settings.appVersion),
            });

            result.push({
                name: "Api version",
                values: this.toVersionString(settings.apiVersion),
            });

            result.push({
                name: "Platform",
                values: this.toPlatformString(deviceAdapter.platform()),
            });

            result.push({
                name: "Source",
                values: this.toSourceString(settings.source),
            });

            result.push({
                name: "Href",
                values: window.location.href,
            });

            result.push({
                name: "Language",
                values: globalize.culture().name,
            });

            result.push({
                name: "Client time",
                values: (new Date()),
            });

            var viewportWidth = uiGlobal.$window.width();
            var viewportHeight = uiGlobal.$window.height();

            result.push({
                name: "Dimensions",
                values: viewportWidth + "x" + viewportHeight
            });

            result.push({
                name: "Touch",
                values: uiGlobal.support.touch,
            });

            result.push({
                name: "3d transform",
                values: uiGlobal.support.transforms3d,
            });

            result.push({
                name: "Transform",
                values: uiGlobal.support.transforms,
            });

            result.push({
                name: "Transitions",
                values: uiGlobal.support.transitions,
            });

            result.push({
                name: "Agent",
                values: navigator.userAgent,
            });

            result.push({
                name: "Device",
                values: deviceAdapter.asString(),
            });

            result.push({
                name: "Native scroll",
                values: uiGlobal.support.nativeScroll,
            });

            result.push({
                name: "Support app exit",
                values: !!(navigator && navigator.app && navigator.app.exitApp),
            });
            

            result.push({
                name: "Pixel ratio",
                values: window.devicePixelRatio||"/",
            });


            return result;
        },
        toVersionString: function (o) {
            return helper.toVersionString(o) || "/";
        },

        toPlatformString: function (o) {
            var PLATFORM_WEB = 1;
            var PLATFORM_ANDROID = 2;
            var PLATFORM_IOS = 3;

            if (o == PLATFORM_WEB) return "web";
            if (o == PLATFORM_ANDROID) return "android";
            if (o == PLATFORM_IOS) return "ios";
            return "/";
        },
        toSourceString: function (o) {
            var SOURCE_APP = 1;
            var SOURCE_MYKET = 2;
            var SOURCE_SIBCHE = 3;
            var SOURCE_BAAZAR = 4;
            var SOURCE_GOOGLE_PLAY = 5;
            var SOURCE_APPLE_STORE = 6;

            if (o && o.id == SOURCE_APP) return "SOURCE_APP";
            if (o && o.id == SOURCE_MYKET) return "SOURCE_MYKET";
            if (o && o.id == SOURCE_SIBCHE) return "SOURCE_SIBCHE";
            if (o && o.id == SOURCE_BAAZAR) return "SOURCE_BAAZAR";
            if (o && o.id == SOURCE_GOOGLE_PLAY) return "SOURCE_GOOGLE_PLAY";
            if (o && o.id == SOURCE_APPLE_STORE) return "SOURCE_APPLE_STORE";
            return "/";
        }

        
    });
    return View;
});