﻿define([
  'jquery',
  'backbone',
  'uiValidation',
  'underscore',
  'globalize',
  'views/base',
  'template',
  'uiContainers',
  'uiControlEditorText',
  'uiControlEditorSelectSingle',
  'uiControlEditorSelectMultiple',
  'uiControlEditorTextArea',
  'uiControlEditorCheckBox',
  'logger'
], function ($,Backbone,uiValidation, _, globalize, ViewBase,
    template,
    uiContainers,
    uiEditorText,
    uiEditorSelectSingle,
    uiEditorSelectMultiple,
    uiEditorTextArea,
    uiEditorCheckBox,
    logger
    ) {

    var EDITOR_TEXTBOX = 1;
    var EDITOR_TEXTAREA = 2;
    var EDITOR_DROPDOWNLIST = 3;
    var EDITOR_LISTBOX = 4;
    var EDITOR_RANGE = 5;
    var EDITOR_CHECKBOX = 6;
    var EDITOR_RADIOBUTTONLIST = 7;
    var EDITOR_CHECKBOXLIST = 8;

    var TYPE_NUMBER = 1;
    var TYPE_BOOL = 2;
    var TYPE_DATETIME = 3;
    var TYPE_STRING = 4;
    var TYPE_COLLECTION = 5;

    var translate = globalize.localizeBase("views.me.offer-edit");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        attributeControls: [],
        offerAttributesContainer: null,
        access: "auth",
        events: {
            "submit form": "save"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            this.options = options;
            
            this.model = options.model;

            var self = this;

            this.model.on("validated", function (isValid, model, error) {
                if (!error.attributesDummy || !model.attributes || !model.attributes.attributes) return;

                _.each(model.attributes.attributes, function (o) {

                    var errorEl = _.find(error.attributesDummy, function (x) {
                        return x.value.id == o.id;
                    });

                    if (errorEl) {
                        Backbone.Validation.callbacks.invalid(self, o, errorEl, function () {
                            return self.getValidatorSelector(o, self, errorEl);
                        });
                    } else {
                        Backbone.Validation.callbacks.valid(self, o, function () {
                            return self.getValidatorSelector(o, self, null);
                        });
                    }

                });
                
            });

            uiValidation.bind(this, {
                selector: this.getValidatorSelector
            });
            
            var self = this;

            this.title = translate("attributes.title");
            this.render();
        },
        renderTemplate: function () {
            var viewModel = $.extend(this.model.toJSON(), {
                translate: translate
            });

            return template.me.offerEditAttributes(viewModel);
        },
        onpagerenderdone: function () {
            this.offerAttributesContainer = this.$("[data-role='offer-attributes']");
            this.renderAttributes();
        },
        renderAttributes: function () {
            var data = this.model.get("attributes");
            var self = this;

            // destroy current controls
            this.destroyAttributeControls();
            this.attributeControls = [];

            this.offerAttributesContainer.off(".offerattributes");
            if (!data || data.length == 0) {
                this.offerAttributesContainer.html('');

                return;
            }

            var viewData = $.map(data, function (o, i) {
                return {
                    id: o.id,
                    title: o.name,
                    editor: o.editor,
                    required: o.required,
                    values: o.values,
                    options: o.options,
                    type:o.type
                }
            });

            this.offerAttributesContainer.empty();

            $.each(viewData, $.proxy(function (i, o) {
                var control = null;
                var renderData = {};
                var controlValue = null;

                if (o.editor == EDITOR_TEXTBOX) {
                    control = new uiEditorText({});
                   
                    var editorType = "text";
                    if (o.type == TYPE_NUMBER) {
                        editorType = "number";
                    }

                    renderData = {
                        label: o.title,
                        required: o.required,
                        type:editorType
                    };
                    controlValue = o.values;
                }
                else if (o.editor == EDITOR_TEXTAREA) {
                    control = new uiEditorTextArea({});
                    renderData = {
                        label: o.title,
                        required: o.required,
                    };
                    controlValue = o.values;
                } else if (o.editor == EDITOR_LISTBOX || o.editor == EDITOR_CHECKBOXLIST) {
                    control = new uiEditorSelectMultiple({}, self);
                    var list = $.map(o.options||[], function (x) {
                        return self.mapListItem(x, false);
                    });
                    renderData = {
                        label: o.title,
                        required: o.required,
                        list: list,
                        listTitle: o.title
                    };

                    if (o.values) {
                        // find in list
                        controlValue = $.map(o.options, function (xo) {
                            if (!_.contains(o.values, xo.id)) return null;
                            return self.mapListItem(xo, true);
                        });
                    }

                } else if (o.editor == EDITOR_RADIOBUTTONLIST || o.editor == EDITOR_DROPDOWNLIST) {
                    control = new uiEditorSelectSingle({}, self);
                    
                    var list = $.map(o.options || [], function (x) {
                        return self.mapListItem(x,false);
                    });

                    renderData = {
                        label: o.title,
                        required: o.required,
                        list: list,
                        listTitle: o.title
                    };

                    if (o.values) {
                        // find in list
                        var controlValues = $.map(o.options, function (xo) {
                            if (xo.id != o.values) return null;
                            return self.mapListItem(xo, true);
                        });
                        controlValue = (controlValues && controlValues.length > 0) ? controlValues[0] : null;
                    }
                } else if (o.editor == EDITOR_RANGE) {
                    control = new uiEditorRange({},self);
                    renderData = {
                        label: o.title,
                        required: o.required,
                    };
                } else if (o.editor == EDITOR_CHECKBOX) {
                    control = new uiEditorCheckBox({});
                    renderData = {
                        label: o.title,
                        required: o.required,
                    };

                    controlValue = o.values;
                } else {
                    logger.log("Editor not supported: " + o.editor);
                }

                if (!control) {
                    return;
                }

                control.id = o.id;
                control.required = o.required;
                renderData.id = "attribute_" + o.id;
                this.attributeControls.push(control);

                var c = $(control.render(renderData));
                this.offerAttributesContainer.append(c);
                control.create(c);

                if (control.val) control.val(controlValue);
            }, this));

        },
        mapListItem:function(a,selected){
            if (!a) return null;
            return {
                id:a.id,
                title:a.name,
                selected:selected
            }
        },
        save: function (e) {
            if (e) e.preventDefault();
            var self = this;

            this.hideKeyboard();

            // collect values
            var currentAttributes = this.model.toJSON().attributes;
            
            var controlIndex = {};

            _.each(this.attributeControls, function (o) {
                controlIndex[o.id] = o;
            });

            if (currentAttributes) {
                _.each(currentAttributes, function (o) {
                    // find control
                    var control = controlIndex[o.id];
                    if (!control) {
                        logger.log("Control not found: " + o.id);
                        return;
                    }

                    // get value
                    var value = control.val();

                    // extract values
                    if (value && _.isArray(value)) {

                        value = $.map(value, function (o, i) {
                            if (typeof o.id === "undefined") return o;
                            return o.id;
                        });
                    }
                    if (value && typeof value.id !== 'undefined') {
                        value = value.id;
                    }

                    o.values = value;
                });
            }

            this.model.set({
                attributes: currentAttributes
            });

            if (!this.model.isValid(true)) {
                this.onSaveModelValidationError();
                return;
            }
            if (this.options && this.options.onSelect) {
                this.options.onSelect(this.model.toJSON());
            }
            this.navigateBack();
        },
        initializeHeader: function (o) {
           
                o.viewSaveCancel({
                    title: this.title,
                    cancelText: translateShared("cancel"),
                    cancelIcon: "icon-cancel",
                    okText: translateShared("accept"),
                    onCancel: $.proxy(function () {
                        this.navigateBack();
                    }, this),
                    onOk: $.proxy(function () {
                        this.save();
                    }, this)
                });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        getValidatorSelector: function (attr,view,error) {
            if (attr == "attributesDummy") return null;
            
            if (!_.isString()) {
                return {
                    error: (error && error.errors && error.errors.length > 0) ? error.errors[0] : null,
                    selector: "#attribute_" + attr.id
                }
            }

            return null;
        },
        destroyAttributeControls: function () {
            if (this.attributeControls) {
                $.each(this.attributeControls, function (i, o) {
                    if (o && o.destroy) {
                        o.destroy();
                    }
                });
            }
        },
        onremoving: function () {
            // destroy current controls
            this.destroyAttributeControls();
            this.attributeControls = null;

            if (this.model) {
                this.model.off("validated");
                this.model = null;
            }
        }
    });

    return View;
});