﻿define([
  'jquery',
  'underscore',
  'globalize',
  'views/base',
  'uiToast',
  'template',
  'uiContainers'
], function ($, _, globalize, ViewBase,uiToast,
    template,
    uiContainers
    ) {


    var translate = globalize.localizeBase("views.me.offer-edit");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        validatorSelectorMask: "[name='&']",
        access: "auth",
        events: {
            "submit form": "save",
            "tap [data-action='reset']": "onReset"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            this.options = options;
            this.setModel(options.model);

            var self = this;

            this.title = translate("contact.title");
            this.render();
        },
        renderTemplate: function () {
            var viewModel = $.extend(this.model.toJSON(), {
                translate: translate
            });

            return template.me.offerEditContact(viewModel);
        },
        save: function (e) {
            if (e) e.preventDefault();
            var self = this;

            this.model.set({
                mobile: this.$("[name='mobile']").val(),
                landline1: this.$("[name='landline1']").val(),
                landline2: this.$("[name='landline2']").val(),
            });
            
            if (!this.model.isValid(true)) {
                this.onSaveModelValidationError();
                return;
            }
            if (this.options && this.options.onSelect) {
                this.options.onSelect(this.model.toJSON());
            }
            this.navigateBack();
        },
        onReset:function(e){
            if (e) e.preventDefault();

            this.$("[name='mobile']").val("");
            this.$("[name='landline1']").val("");
            this.$("[name='landline2']").val("");

            uiToast.open(translate("contact.reset.info"));

        },
        initializeHeader: function (o) {
           
                o.viewSaveCancel({
                    title: this.title,
                    cancelText: translateShared("cancel"),
                    cancelIcon: "icon-cancel",
                    okText: translateShared("accept"),
                    onCancel: $.proxy(function () {
                        this.navigateBack();
                    }, this),
                    onOk: $.proxy(function () {
                        this.save();
                    }, this)
                });
        },
        initializeFooter: function (o) {
            o.viewNone();
        }
    });

    return View;
});