﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'uiModal',
  'uiToast',
  'uiDialog',
  'uiGlobal',
  'helper',
  'template',
  'uiContainers'
], function ($, _, api, globalize, ViewBase,
    uiModal,
    uiToast,
    uiDialog,
    uiGlobal,
    helper,
    template,
    uiContainers
    ) {

    var translate = globalize.localizeBase("views.me.offer-edit-doping-bank");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        access: "auth",
        model: null,
        doping: null,
        duration:null,
        offerId: null,
        dopingId: null,
        durationId:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            var self = this;
            this.title = translate("title");

            this.offerId = options.id;
            this.dopingId = options.doping;
            this.durationId = options.duration;
            if (this.durationId == 'end') this.durationId = null;
            var data = api.me.offer.doping.list(options.id).done(_.bind(function (data) {
                if (data.hasDoping) {
                    this.onHasDoping();
                    return;
                }
                var doping = _.find(data.items, function (o) {
                    return o.type = options.doping;
                });
                if (doping) {

                    this.duration = _.find(doping.durations, function (o) {
                        return (o.days == self.durationId);
                    });
                }
                this.model = data.banks;
            }, this));

            var dopings = api.collection.doping(this.dopingId).done(_.bind(function (data) {
                this.doping = data;
            }, this));

            uiContainers.header.loading(true);
            $.when(data, dopings).done(_.bind(function () {
                this.load();
            }, this)).fail(_.bind(function (data) {
                // show error
                this.onError(data);
            }, this)).always(function () {
                uiContainers.header.loading(false);
            });

        },
        onHasDoping: function () {
            this.navigate("me/offer/" + this.offerId);
        },
        load: function () {
            if (this.model && this.doping && this.duration) {
                this.render();
            } else {
                this.onError();
            }
        },
        reload: function () {
            this.load();
        },
        renderTemplate: function () {
            var items = [];
            var self = this;
                
            _.each(this.model, function (o) {
                items.push({
                    id: o.id,
                    name: o.name,
                    logoUrl: o.logoUrl
                });

            });

            var durationName = "";
            var discountName = "";
            var durationPriceName = "";
            if (!this.duration.days) {
                durationName = globalize.localize("views.me.offer-edit-doping-duration.duration.untilEnd");
            } else {
                durationName = globalize.localize("views.me.offer-edit-doping-duration.duration.days").replace("[DAYS]", this.duration.days);
            }
            if (this.duration.discount) {
                discountName = globalize.localize("views.me.offer-edit-doping-duration.savings.label").replace("[PERCENT]", this.duration.discount);
            }
            if (this.duration.price) {
                durationPriceName = globalize.localize("views.me.offer-edit-doping-duration.price.label").replace("[PRICE]", globalize.format(this.duration.price / 10, "C0"));
            }

            var viewModel = {
                items: items,
                duration:{
                    id: this.duration.days,
                    duration: durationName,
                    discount: discountName,
                    price: durationPriceName
                },
                doping: {
                    type: this.doping.id,
                    icon: helper.toDopingIcon(this.doping.id),
                    name: this.doping.name,
                    description: this.doping.description,
                    price: ""
                },
                translate: translate
            }

            return template.me.offerEditDopingBank(viewModel);
        },
        onpagerenderdone: function () {
            var elements = $("[data-val]", this.$el);
            var self = this;
            elements.off(".doping").hammer({
                //prevent_default: true
            }).on(uiGlobal.event.tap + ".doping", function (e) {
                e.preventDefault();
                var val = $(this).data("val");
                var duration = self.durationId;
                if (!duration) duration = "end";
                self.navigate("me/offer/edit/" + self.offerId + "/doping/" + self.dopingId + "/duration/" + duration + "/bank/" + val + "/confirm");
            });
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("back"),
                okIcon: "icon-chevron",
                okCssClass: "",
                onCancel: $.proxy(function () {
                    this.navigate("me/offer/" + this.offerId);
                }, this),
                onOk: $.proxy(function () {
                    this.navigateBack();
                }, this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        onremoving: function () {
            // destroy current controls
            if (this.$el) {
                $("[data-val]", this.$el).off(".doping");
            }
        }
    });

    return View;
});