﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'uiModal',
  'uiToast',
  'uiDialog',
  'uiGlobal',
  'uiContainers',
  'helper',
  'template',
  'uiControlBase'
], function ($, _, api, globalize, ViewBase,
    uiModal,
    uiToast,
    uiDialog,
    uiGlobal,
    uiContainers,
    helper,
    template,
    uiControlBase
    ) {

    var translate = globalize.localizeBase("views.me.offer-edit-doping-confirm");
    var translateShared = globalize.localizeBase("views.shared");

    var SuccessResult = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-logo icon-4x block\"></i> <%=text %><p><a class=\"btn btn-block btn-primary nowrap\"><%=buttonText%></a></p></div></div>"),
        getRenderData: function () {
            return {
                text: translate("bank.redirect.info"),
                buttonText: translate("bank.redirect.button"),
            }
        },
        onBeforeDestory: function () {
            if (this.el) {
                this.el.off(uiGlobal.event.tap);
            }
        },
        onRender: function () {
            if (!this.el) return this;
            var self = this;

            this.el.hammer({
               //  prevent_default: false
            }).on(uiGlobal.event.tap, function (e) {
                e.preventDefault();
                e.stopPropagation();
                self.trigger("select", self);
            });
        }
    });

    var View = ViewBase.extend({
        access: "auth",
        doping: null,
        duration: null,
        bank:null,
        offerId: null,
        dopingId: null,
        durationId: null,
        bankId: null,
        events: {
            "submit form.confirm": "save"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.title = translate("title");
            var self = this;
            this.offerId = options.id;
            this.dopingId = options.doping;
            this.durationId = options.duration;
            this.bankId = options.bank;
            if (this.durationId == 'end') this.durationId = null;
            var data = api.me.offer.doping.list(options.id).done(_.bind(function (data) {
                if (data.hasDoping) {
                    this.onHasDoping();
                    return;
                }
                var doping = _.find(data.items, function (o) {
                    return o.type = options.doping;
                });
                if (doping) {

                    this.duration = _.find(doping.durations, function (o) {
                        return (o.days == self.durationId);
                    });
                }
                this.bank = _.find(data.banks, function (o) {
                    return o.id == options.bank;
                });
            }, this));

            var dopings = api.collection.doping(this.dopingId).done(_.bind(function (data) {
                this.doping = data;
            }, this));

            uiContainers.header.loading(true);
            $.when(data, dopings).done(_.bind(function () {
                this.load();
            }, this)).fail(_.bind(function (data) {
                // show error
                
                this.onError(data);
            }, this)).always(function () {
                uiContainers.header.loading(false);
            });

        },
        onHasDoping:function(){
            this.navigate("me/offer/" + this.offerId);
        },
        load: function () {
            if (this.doping && this.duration && this.bank) {
                this.render();
            } else {
                this.onError();
            }
        },
        reload: function () {
            this.load();
        },
        renderTemplate: function () {
            var items = [];
            var self = this;

            var durationName = "";
            var discountName = "";
            var durationPriceName = "";
            if (!this.duration.days) {
                durationName = globalize.localize("views.me.offer-edit-doping-duration.duration.untilEnd");
            } else {
                durationName = globalize.localize("views.me.offer-edit-doping-duration.duration.days").replace("[DAYS]", this.duration.days);
            }
            if (this.duration.discount) {
                discountName = globalize.localize("views.me.offer-edit-doping-duration.savings.label").replace("[PERCENT]", this.duration.discount);
            }
            if (this.duration.price) {
                durationPriceName = globalize.localize("views.me.offer-edit-doping-duration.price.label").replace("[PRICE]", globalize.format(this.duration.price / 10, "C0"));
            }

            var viewModel = {
                items: items,
                duration: {
                    id: this.duration.days,
                    duration: durationName,
                    discount: discountName,
                    price: durationPriceName
                },
                doping: {
                    type: this.doping.id,
                    icon: helper.toDopingIcon(this.doping.id),
                    name: this.doping.name,
                    description: this.doping.description,
                    price: ""
                },
                bank: {
                    id: this.bank.id,
                    name: this.bank.name,
                    logoUrl: this.bank.logoUrl
                },
                translate: translate
            }

            return template.me.offerEditDopingConfirm(viewModel);
        },
        onpagerenderdone: function () {
            var elements = $("button", this.$el);
            var self = this;
            elements.off(".doping").hammer({
              //  prevent_default: true
            }).on(uiGlobal.event.tap + ".doping", function (e) {
                e.preventDefault();
                
                self.save();
            });
        },
        save: function (e) {
            if (e) e.preventDefault();
            
            var self = this;
            uiContainers.header.loading(true);
            api.me.offer.doping.prepare(this.offerId, {
                doping: this.dopingId,
                bank: this.bank.id,
                duration:this.durationId
            }).done(function (data) {
                // do bank stuff
                self.siteLogin(data);
            }).fail(function () {
                uiToast.danger(translate("error.info"));
            }).always(function () {
                uiContainers.header.loading(false);
            });
        },
        siteLogin: function (doping) {
            uiContainers.header.loading(true);
            var self = this;
            return api.me.siteLoginRequest(doping.url).done(function (data) {
                window.open(data.url, "_system");
                self.renderSuccess();
            }).fail(function () {
                uiToast.danger(translate("error.info"));
            }).always(function () {
                uiContainers.header.loading(false);
            });
        },
        renderSuccess: function () {
            var self = this;
            this.$el.empty();

            var res = new SuccessResult();
            this.render(res);
            res.on("select", function () {
                self.navigate("#me/offer/" + self.offerId);
            });
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("back"),
                okIcon: "icon-chevron",
                okCssClass: "",
                onCancel: $.proxy(function () {
                    this.navigate("me/offer/" + this.offerId);
                }, this),
                onOk: $.proxy(function () {
                    this.navigateBack();
                }, this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        onremoving: function () {
            // destroy current controls
            if (this.$el) {
                $("[data-val]", this.$el).off(".doping");
            }
        }
    });

    return View;
});