﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'uiModal',
  'uiToast',
  'uiDialog',
  'uiGlobal',
  'helper',
  'template',
  'uiContainers'
], function ($, _, api,globalize, ViewBase,
    uiModal,
    uiToast,
    uiDialog,
    uiGlobal,
    helper,
    template,
    uiContainers
    ) {

    var translate = globalize.localizeBase("views.me.offer-edit-doping-duration");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        access: "auth",
        model: null,
        doping:null,
        offerId: null,
        dopingId:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.title = translate("title");

            this.offerId = options.id;
            this.dopingId = options.doping;
            var data = api.me.offer.doping.list(options.id).done(_.bind(function (data) {
                if (data.hasDoping) {
                    this.onHasDoping();
                    return;
                }

                this.model = _.find(data.items, function (o) {
                    return o.type = options.doping;
                });
            }, this));

            var dopings = api.collection.doping(this.dopingId).done(_.bind(function (data) {
                this.doping = data;
            }, this));

            uiContainers.header.loading(true);
            $.when(data, dopings).done(_.bind(function () {
                this.load();
            }, this)).fail(_.bind(function (data) {
                // show error
                
                this.onError(data);
            }, this)).always(function () {
                uiContainers.header.loading(false);
            });

        },
        onHasDoping: function () {
            this.navigate("me/offer/" + this.offerId);
        },
        load: function () {
            
            if (this.model && this.doping) {
                this.render();
            } else {
                this.onError();
            }
        },
        reload: function () {
            this.load();
        },
        renderTemplate: function () {
            var items = [];
            var self = this;
                
            _.each(this.model.durations, function (o) {

                var durationName = "";
                var discountName = "";
                var priceName = "";
                if (!o.days) {
                    durationName = globalize.localize("views.me.offer-edit-doping-duration.duration.untilEnd");
                } else {
                    durationName = globalize.localize("views.me.offer-edit-doping-duration.duration.days").replace("[DAYS]", o.days);
                }
                if (o.discount) {
                    discountName = globalize.localize("views.me.offer-edit-doping-duration.savings.label").replace("[PERCENT]", o.discount);
                }
                if (o.price) {
                    priceName = globalize.localize("views.me.offer-edit-doping-duration.price.label").replace("[PRICE]", globalize.format(o.price / 10, "C0"));
                }

                items.push({
                    id:o.days,
                    duration: durationName,
                    discount: discountName,
                    price: priceName
                });

            });
            
            var viewModel = {
                items: items,
                doping: {
                    type: this.model.type,
                    icon: helper.toDopingIcon(this.model.type),
                    name: this.doping.name,
                    description: this.doping.description,
                    price: ""
                },
                translate: translate
            }

            return template.me.offerEditDopingDuration(viewModel);
        },
        onpagerenderdone: function () {
            var elements = $("[data-val]", this.$el);
            var self = this;
            elements.off(".doping").hammer({
               // prevent_default: true
            }).on(uiGlobal.event.tap + ".doping", function (e) {
                e.preventDefault();
                var val = $(this).data("val");
                if (!val) val = "end";
                self.navigate("me/offer/edit/" + self.offerId + "/doping/" + self.dopingId + "/duration/" + val+"/bank");
            });
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,

                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("back"),
                okIcon: "icon-chevron",
                okCssClass:"",
                onCancel: $.proxy(function () {
                    this.navigate("me/offer/" + this.offerId);
                }, this),
                onOk: $.proxy(function () {
                    this.navigateBack();
                }, this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        onremoving: function () {
            // destroy current controls
            if (this.$el) {
                $("[data-val]", this.$el).off(".doping");
            }
        }
    });

    return View;
});