﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'uiModal',
  'uiToast',
  'uiDialog',
  'uiGlobal',
  'helper',
  'template',
  'uiContainers',
  'settings'
], function ($, _, api,globalize, ViewBase,
    uiModal,
    uiToast,
    uiDialog,
    uiGlobal,
    helper,
    template,
    uiContainers,
    settings
    ) {

    var translate = globalize.localizeBase("views.me.offer-edit-doping");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        access: "auth",
        model: null,
        dopings: [],
        offerId:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.title = translate("title");
            this.offerId = options.id;

            var data = null;

            if (settings.source.canBuyVitamins) {
                data = api.me.offer.doping.list(options.id);
            } else {
                data = $.Deferred().resolve({
                    hasDoping: false,
                    items:[]
                });
            }

            data.done(_.bind(function (data) {
                if (data.hasDoping) {
                    this.onHasDoping();
                    return;
                }

                this.model = data;
            }, this)).fail(_.bind(function () {
                // show error
                this.model = null;
            }, this));

            var dopings = api.collection.doping().done(_.bind(function (data) {
                this.dopings = data;
            }, this));

            uiContainers.header.loading(true);
            $.when(data, dopings).done(_.bind(function () {
                this.render();
            }, this)).fail(_.bind(function (data) {
                // show error
                this.onError(data);
            }, this)).always(function () {
                uiContainers.header.loading(false);
            });
            
        },
        onHasDoping: function () {
            this.navigate("me/offer/" + this.offerId);
        },
        renderTemplate: function () {
            var items = [];
            var self = this;
            if (this.model && this.dopings) {
                var doping = _.find(this.dopings, function (o) { return o.id == 0 })
                if (doping) {
                    items.push({
                        type: 0,
                        icon: helper.toDopingIcon(0),
                        name: doping.name,
                        description: doping.description,
                        price: "",
                        selected: false
                    });
                }

                _.each(this.model.items, function (o) {
                    var doping = _.find(self.dopings, function (x) { return x.id == o.type });
                    
                    if (doping) {
                        items.push({
                            type: o.type,
                            icon: helper.toDopingIcon(o.type),
                            name: doping.name,
                            description: doping.description,
                            price: "",
                            selected: false
                        });
                    }

                });
            }
            var viewModel = {
                items:items,
                translate: translate
            }
          
            return template.me.offerEditDoping(viewModel);
        },
        onpagerenderdone: function () {
            var elements = $("[data-val]", this.$el);
            var self = this;
            elements.off(".doping").hammer({
               // prevent_default: true
            }).on(uiGlobal.event.tap + ".doping", function (e) {
                e.preventDefault();
                var val = $(this).data("val");
                // free offer
                if (val == 0) {
                    uiContainers.header.loading(true);
                    api.me.offer.complete(self.offerId).done(function () {
                        uiToast.success(translate("free.success.info"));
                        self.navigate("me/offer/" + self.offerId);
                    }).fail(function () {
                        uiToast.danger(translate("free.error.info"));
                    }).always(function () {
                        uiContainers.header.loading(false);
                    });
                    //self.navigate("me/offer/edit/" + self.offerId + "/doping/confirm");
                } else {
                    self.navigate("me/offer/edit/" + self.offerId + "/doping/" + val + "/duration");
                }
            });
        },
        initializeHeader: function (o) {
            o.viewOkCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("back"),
                okIcon: "icon-chevron",
                okCssClass:"",
                onCancel: $.proxy(function () {
                    this.navigate("me/offer/" + this.offerId);
                }, this),
                onOk: $.proxy(function () {
                    this.navigate("me/offer/edit/" + this.offerId);
                }, this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        onremoving: function () {
            // destroy current controls
            if (this.$el) {
                $("[data-val]", this.$el).off(".doping");
            }
        }
    });

    return View;
});