﻿define([
  'jquery',
  'underscore',
  'globalize',
  'views/base',
  'template',
  'uiContainers',
  'uiDialog',
  'uiToast',
  'uiGlobal',
  'uiControlBase',
  'tracking',
  'uiControlEditorOfferImageGallery',
  'logger'
], function ($, _, globalize, ViewBase,
    template,
    uiContainers,
    uiDialog,
    uiToast,
    uiGlobal,
    uiControlBase,
    tracking,
    uiEditorOfferImageGallery, logger) {


    var translate = globalize.localizeBase("views.me.offer-edit");
    var translateShared = globalize.localizeBase("views.shared");

    var EmptyView = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-image icon-6x block\"></i><p><%=text %></p>" +
            
            "<a class=\"control-file btn btn-primary btn-block\" data-action=\"image-upload\">" +
            "<i class=\"icon icon-image-add\"></i> <%=uploadButtonText %>" +
            "<%if (isWebUpload) { %>" +
            "<input type=\"file\" accept=\"image/*\" multiple=\"true\"> " +
            "<%} %>" +
            "</a>" +
            "</div></div>"),

        getRenderData: function () {
            return {
                text: translate("noimage"),// "Offer is no longer available."
                uploadButtonText: translate("gallery.add.button"),
                isWebUpload:this.options.isWebUpload
            }
        }
    });

    var GalleryView = uiControlBase.extend({
        template: template.me.offerEditGallery
    });


    var View = ViewBase.extend({
        validatorSelectorMask: "[name='&']",
        access: "auth",
        galleryControl: null,
        isGalleryFooterRendered: false,
        imageDimensions:null,
        events: {
            "tap [data-action='image-upload']": "onFileUpload",
            "tap #offer-gallery-set-primary": "onGallerySetPrimary",
            "tap #offer-gallery-delete": "onGalleryRemove",
            "change input[type='file']": "onWebUpload"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            this.options = options;
            this.setModel(options.model);

            var self = this;
            this.galleryControl = new uiEditorOfferImageGallery();

            this.title = translate("gallery.title");
            this.calculateImageDimensions();
            this.renderGallery();
        },
        calculateImageDimensions: function () {
            var imageWidth = Math.floor(Math.min((uiGlobal.$body.width()/3) - 30, 190));
            var imageHeight = imageWidth;
            var dim = {
                width: imageWidth,
                height: imageHeight
            }

            this.imageDimensions = dim;
        },
        renderTemplate:function(){
            return "";
        },
        //_renderTemplate: function () {
        //    var viewModel = $.extend(this.model.toJSON(), {
        //        isWebUpload: this.isWebUpload(),
        //        translate: translate,
        //        width: this.imageDimensions.width,
        //        height: this.imageDimensions.height
        //    });

        //    return template.me.offerEditGallery(viewModel);
        //},
        //onpagerenderdone: function () {
        //    this.renderGallery();
        //},
        renderGallery: function () {
            var images = this.model.get("images");

         //   if (images && images.length > 0) {

                this.$el.empty();

                var viewModel = $.extend(this.model.toJSON(), {
                    isWebUpload: this.isWebUpload(),
                    translate: translate,
                    width: this.imageDimensions.width,
                    height: this.imageDimensions.height
                });

                var galleryView = new GalleryView(viewModel);
                this.registerControl(galleryView);
                this.render(galleryView);

                // load gallery
                var container = $("#offer-file-upload");
                var list = $("[data-role='list']", container);

                var self = this;
                //if (this.isWebUpload()) {
                //    var webUploadEl = $("input[type='file']", container);

                //    webUploadEl.on("change", function (e) {
                //        var el = this;
                //        if (!el.files || el.files.length == 0) return;

                //        for (var i = 0; i < el.files.length; i++) {
                //            self.galleryControl.upload(el.files[i]);
                //        }
                //    });
                //}

                this.galleryControl.createIn(list, {
                    width: this.imageDimensions.width,
                    height: this.imageDimensions.height,
                    onSelectionChanged: _.bind(self.onGallerySelectionChanged, self)
                });
                this.galleryControl.val(images);

            //} else {
            //    this.$el.empty();

            //    var emptyView = new EmptyView({
            //        isWebUpload: this.isWebUpload()
            //    });
            //    this.registerControl(emptyView);
            //    this.render(emptyView);

            //}
        },
        onWebUpload: function (e) {

            var el = e.target;
            if (!el.files || el.files.length == 0) return;
            tracking.trackEvent("offerEditGallery", "fileUpload", "webUpload", (this.model ? this.model.id : null));

            for (var i = 0; i < el.files.length; i++) {
                this.galleryControl.upload(el.files[i]);
            }
        },
        onGallerySelectionChanged: function (items) {
            
        },
        onFileUpload: function (e) {

            if (e) e.preventDefault();
            if (e) e.stopPropagation();

            //if we have web upload, dialog opens automaticaly
            if (this.isWebUpload()) {

                return;
            }

            var self = this;
            tracking.trackEvent("offerEditGallery", "fileUpload", "options", (this.model ? this.model.id : null));
            uiDialog.options.open(translate("upload.label"), [
                { name: translate("upload.from-gallery"), icon: "icon-images", action: "gallery" },
                { name: translate("upload.from-camera"), icon: "icon-camera", action: "camera" },
                { name: translate("upload.cancel"), cssClass:"btn btn-danger" }
            ]).done(function (data) {
                if (data && data.action == "gallery") {
                    tracking.trackEvent("offerEditGallery", "fileUpload", "gallery", (this.model ? this.model.id : null));
                    self.startGallery();
                }
                if (data && data.action == "camera") {
                    tracking.trackEvent("offerEditGallery", "fileUpload", "camera", (this.model ? this.model.id : null));
                    self.startCamera();
                }
            });
        },
        startCamera: function () {

            navigator.camera.getPicture(_.bind(this.uploadFromCameraOrGallery, this), function () { }, {
                quality: 50,
                correctOrientation: true,
                destinationType: Camera.DestinationType.FILE_URI,
                sourceType: Camera.PictureSourceType.CAMERA,
               // allowEdit: true,
                targetWidth: 400,
                targetHeight: 400
            });

        },
        startGallery: function () {

            navigator.camera.getPicture(_.bind(this.uploadFromCameraOrGallery, this), function () { }, {
                quality: 50,
                correctOrientation: true,
                destinationType: Camera.DestinationType.FILE_URI,
                sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
               // allowEdit: true,
                targetWidth: 400,
                targetHeight: 400
            });
        },
        uploadFromCameraOrGallery: function (imageURI) {
            var self = this;
            
            window.resolveLocalFileSystemURI(imageURI,
                function (fileEntry) {
                    fileEntry.file(function (file) {
                        
                            self.galleryControl.upload({
                                name: file.name,
                                size: file.size,
                                type: file.type || "image/jpeg",
                                fullPath: fileEntry.toURL()
                            }).fail(function (err) {
                                logger.error("Upload failed: " + err);
                            });
                       
                    }, function (err) {
                        logger.error("ERROR file: " + err);
                    });

                },
                function (err) {
                    logger.error("ERROR resolveLocalFileSystemURI: " + err);
                }
            );
        },
        isWebUpload: function () {
            return !(navigator && navigator.camera);
        },
      
        onGallerySet
        
        : function (e) {
            if (e) {
                e.preventDefault();
                e.stopPropagation();
            }
            var selected = this.galleryControl.selected();
            if (!selected || selected.length != 1) {
                uiToast.danger(translate("gallery.selection-empty.primary.info"));
                return;
            }
            this.galleryControl.primary(this.galleryControl.selected()).selected(null);
        },
        onGalleryRemove: function (e) {
            if (e) {
                e.preventDefault();
                e.stopPropagation();
            }
            var self = this;
            var selected = self.galleryControl.selected();
            if (!selected || selected.length == 0) {
                uiToast.danger(translate("gallery.selection-empty.remove.info"));
                return;
            }
            uiDialog.confirm(translate("gallery.remove.notification")).done(function () {
                self.galleryControl.remove(selected);
            });
            
        },
        //hideGalleryFooter: function () {
        //    uiContainers.footer.hide();
        //    //this.galleryControl.selected(null);
        //},
        save: function (e) {
            if (e) e.preventDefault();
            var self = this;

            this.model.set({
                images: this.galleryControl.val()
            });

            if (!this.model.isValid(true)) {
                this.onSaveModelValidationError();
                return;
            }
            if (this.options && this.options.onSelect) {
                this.options.onSelect(this.model.toJSON());
            }
            this.navigateBack();
        },
        initializeHeader: function (o) {
           
                o.viewSaveCancel({
                    title: this.title,
                    cancelText: translateShared("cancel"),
                    cancelIcon: "icon-cancel",
                    okText: translateShared("accept"),
                    onCancel: $.proxy(function () {
                        this.navigateBack();
                    }, this),
                    onOk: $.proxy(function () {
                        this.save();
                    }, this)
                });
        },
        initializeFooter: function (o) {
            data = {
                items: [
                { href: "javascript:void(0);", id: "offer-gallery-upload", icon: "icon-plus", label: translate("gallery.add.button"), cssClass: null },
                { href: "javascript:void(0);", id: "offer-gallery-set-primary", icon: "icon-eye", label: translate("gallery.primary.button"), cssClass: null },
                { href: "javascript:void(0);", id: "offer-gallery-delete", icon: "icon-remove", label: translate("gallery.remove.button"), cssClass: "primary" }
                ]
            };

            o.viewDefault(data);

            var el = o.get();
            var self = this;

            $("#offer-gallery-upload", el).hammer().off("tap").on("tap", function (e) {
                self.onFileUpload(e);
            });

            $("#offer-gallery-set-primary", el).hammer().off("tap").on("tap", function (e) {
                self.onGallerySetPrimary(e);
            });
            $("#offer-gallery-delete", el).hammer().off("tap").on("tap", function (e) {
                 self.onGalleryRemove(e);
            });
        },
        onremoving: function () {
            // destroy current controls
            uiDialog.options.close();
            if (this.galleryControl && this.galleryControl.destroy) {
                this.galleryControl.destroy();
            }
            this.galleryControl = null;
        }
    });

    return View;
});