﻿define([
  'jquery',
  'underscore',
  'globalize',
  'views/base',
  'template',
  'uiContainers'
], function ($, _,globalize,ViewBase,
    template,
    uiContainers
    ) {

    var translate = globalize.localizeBase("views.me.offer-edit");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        validatorSelectorMask: "[name='&']",
        access: "auth",
        events: {
            "submit form": "save"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            this.options = options;
            this.setModel(options.model);

            var self = this;

            this.title = translate("text.title");
            this.render();
        },
        renderTemplate: function () {
            var viewModel = $.extend(this.model.toJSON(), {
                translate: translate
            });

            return template.me.offerEditText(viewModel);
        },
        save: function (e) {
            if (e) e.preventDefault();
            var self = this;
           
            this.hideKeyboard();

            this.model.set({
                name: this.$("[name='name']").val(),
                description: this.$("[name='description']").val(),
            });

            if (!this.model.isValid(true)) {
                this.onSaveModelValidationError();
                return;
            }
            if (this.options && this.options.onSelect) {
                this.options.onSelect(this.model.toJSON());
            }
            this.navigateBack();
        },
        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                onCancel: $.proxy(function () {
                    this.navigateBack();
                }, this),
                onOk: $.proxy(function () {
                    this.save();
                }, this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        }
    });

    return View;
});