﻿define([
  'jquery',
  'underscore',
  'api',
  'helper',
  'globalize',
  'preferences',
  'views/base',
  'helper',
  'uiModal',
  'uiToast',
  'uiDialog',
  'uiGlobal',
  'settings',
  'models/me/offer-text',
  'models/me/offer-contact',
  'models/me/offer-gallery',
  'models/me/offer-attributes',
  'models/me/offer-category',
  'models/me/offer-location',
  'template',
  'uiContainers',
  'views/tos',
  'views/location',
  'views/category',
  'views/me/offer-edit-text',
  'views/me/offer-edit-contact',
  'views/me/offer-edit-attributes',
  'views/me/offer-edit-gallery',
  'uiControlTile',
  'uiControlEditorText',
  'uiControlEditorSelectSingle',
  'uiControlEditorSelectMultiple',
  'uiControlEditorTextArea',
  'uiControlEditorCheckBox',
  'uiControlEditorOfferCategory',
  'uiControlEditorOfferImageGallery',
  'uiControlEditorLocation',
  'tracking'
], function ($, _, api,helper,
    globalize, preferences, ViewBase, helper,
    uiModal,
    uiToast,
    uiDialog,
    uiGlobal,
    settings,
    ModelText,
    ModelContact,
    ModelGallery,
    ModelAttributes,
    ModelCategory,
    ModelLocation,
    template,
    uiContainers,
    ViewTOS,
    ViewLocation,
    ViewCategory,
    ViewOfferEditText,
    ViewOfferEditContact,
    ViewOfferEditAttributes,
    ViewOfferEditGallery,
    uiControlTile,
    uiEditorText,
    uiEditorSelectSingle,
    uiEditorSelectMultiple,
    uiEditorTextArea,
    uiEditorCheckBox,
    uiEditorOfferCategory,
    uiEditorOfferImageGallery,
    uiEditorLocation,
    tracking
    ) {

    var LEVEL_COUNTRY = 1;
    var LEVEL_REGION = 2;
    var LEVEL_CITY = 3;
    var LEVEL_NEIGHBOURHOOD = 4;

    var translate = globalize.localizeBase("views.me.offer-edit");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        attributesControl: null,
        categoryControl: null,
        locationControl: null,
        textControl: null,
        contactControl: null,
        galleryControl: null,
        modelText: null,
        modelContact: null,
        modelGallery: null,
        modelAttributes: null,
        modelCategory: null,
        modelLocation: null,
        _isSaving: false,
        options:null,
        access: "auth",
        events: {
            "submit form": "save",
            "tap #tos": "termsOfService"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            var self = this;

            this.options = options;

            this.title = translate("title");

            this.modelText = new ModelText().on("change", function () {
                self.textControl.value(this.toJSON(),true);
            }).on("validated", function (isValid, model, error) {
                if (!isValid) {
                    self.textControl.setValidationErrors(translate("validation.text.invalid"));
                } else {
                    self.textControl.clearValidationErrors();
                }
            });

            this.modelContact = new ModelContact().on("change", function () {
                self.contactControl.value(this.toJSON(), true);
            }).on("validated", function (isValid, model, error) {
                if (!isValid) {
                    self.contactControl.setValidationErrors(translate("validation.contact.invalid"));
                } else {
                    self.contactControl.clearValidationErrors();
                }
            });

            this.modelGallery = new ModelGallery().on("change", function () {
                self.galleryControl.value(this.toJSON(), true);
            }).on("validated", function (isValid, model, error) {
                if (!isValid) {
                    self.galleryControl.setValidationErrors("");
                } else {
                    self.galleryControl.clearValidationErrors();
                }
            });

            this.modelAttributes = new ModelAttributes().on("change", function () {
                self.attributesControl.value(this.toJSON(), true);
            }).on("validated", function (isValid, model, error) {
                if (!isValid) {
                    self.attributesControl.setValidationErrors(translate("validation.attributes.invalid"));
                } else {
                    self.attributesControl.clearValidationErrors();
                }
            });

            this.modelLocation = new ModelLocation().on("change", function () {
                self.locationControl.value(this.toJSON().location, true);
            }).on("validated", function (isValid, model, error) {
                if (!isValid) {
                    self.locationControl.setValidationErrors(translate("validation.location.invalid"));
                } else {
                    self.locationControl.clearValidationErrors();
                }
            });

            this.modelCategory = new ModelCategory().on("change", function () {
                self.categoryControl.value(this.toJSON().category, true);
            }).on("validated", function (isValid, model, error) {
                if (!isValid) {
                    self.categoryControl.setValidationErrors(translate("validation.category.invalid"));
                } else {
                    self.categoryControl.clearValidationErrors();
                }
                self.modelAttributes.category = (isValid?model.attributes.category:null);

                if (typeof self.modelAttributes.isValid() !== 'undefined') {
                    self.modelAttributes.validate();
                }

                if (typeof self.modelLocation.isValid() !== 'undefined') {
                    self.modelLocation.validate();
                }
                
            });

            this.galleryControl = new uiEditorOfferImageGallery();
            
            // create category control
            this.categoryControl = new uiControlTile({
                content: function (val) {
                    if (!val) {
                        return "<p>" + translate("select-category.info") + "</p>";
                    }

                    var html = "<p><i class=\"icon " + helper.toCategoryCssClass(val) + "\"></i>" + val.name + "</p>";
                    var items = helper.toCategoryParentList(val.parent);
                    if (items.length > 2) {
                        items = [
                            items[0], null, items[items.length - 1]
                        ]
                    }
                    var htmlContent = "<ul class=\"small list-path\">" + $.map(items, function (o, i) {
                        if (!o) return "<li>...</li>";
                        if (i == 0) {
                            return "<li>" + o.name + "</li>";
                        }
                        return "<li>" + o.name + "</li>";
                    }).join('') + "</ul>";

                    if (htmlContent) {
                        html += "<ul class=\"small list-path\">" + htmlContent + "</ul>";
                    }

                    return html;
                },
                mediaIcon: "icon icon-category"
            });
            this.categoryControl.on("select", _.bind(function () {
                this.openCategorySelect();
            }, this));
            this.categoryControl.on("changed", function () {
                var value = this.value();
                
                if (value && value.effectiveMetaData && value.effectiveMetaData.isRealestate) {
                    self.modelLocation.maxLevel = self.locationControl.maxLevel = LEVEL_NEIGHBOURHOOD;
                    var loc = self.locationControl.value();
                    self.locationControl.value(loc, true);
                } else {
                    // if neighbourhood is selected, then select parent
                    var loc = self.locationControl.value();
                    if (loc && loc.level == LEVEL_NEIGHBOURHOOD) {
                        self.locationControl.value(loc.parent,true);
                    }
                    self.modelLocation.maxLevel = self.locationControl.maxLevel = LEVEL_CITY;
                }

               // self.loadAttributes();
            });
            this.registerControl(this.categoryControl);

            // create location control
            this.locationControl = new uiControlTile({
                content: function (val) {
                    
                    if (!val || !val.name) {
                        return "<p>" + translate("select-location.info") + "</p>";
                    }
                    var html = "<p>" + val.name + "</p>";

                    var htmlContent = "";
                    var current = val.parent;
                    while (current && current.level > LEVEL_COUNTRY) {

                        htmlContent += "<li>" + current.name + "</li>";
                        current = current.parent;
                    }
                    if (htmlContent) {
                        html += "<ul class=\"small list-path\">" + htmlContent + "</ul>";
                    }

                    return html;
                },
                mediaIcon: "icon icon-location"
            });
            this.locationControl.on("select", _.bind(function () {
                this.openLocationSelect();
            }, this));
            this.modelLocation.maxLevel = this.locationControl.maxLevel = LEVEL_CITY;
            this.registerControl(this.locationControl);

            /* create text control */
            this.textControl = new uiControlTile({
                content: function (val) {
                    if (!val || (!val.name && !val.description)) {
                        return "<p>" + translate("select-text.info") + "</p>"
                    }
                    var html = "";

                    if (val.name && $.trim(val.name).length > 0) {
                        html += "<h1>" + val.name + "</h1>";
                    }
                    if (val.description && $.trim(val.description).length > 0) {
                        html += "<p class=\"tile-description text-small\">" + self.nl2br(val.description) + "</p>";
                    }

                    return html;
                },
                mediaIcon: "icon icon-pen"
            });
            this.textControl.on("select", _.bind(function () {
                this.openTextSelect();
            }, this));
            this.registerControl(this.textControl);

            /* create contact control */
            this.contactControl = new uiControlTile({
                content: function (val) {
                    if (!val || (!val.mobile && !val.landline1 && !val.landline2)) {
                        if (this.isValid() == true) {
                            return "<p>" + translate("no-contact.info") + "</p>";
                        }
                        return "<p>" + translate("select-contact.info") + "</p>";
                    }
                    var html = "<ul class=\"list-plain\">";

                    if (val.mobile && $.trim(val.mobile).length > 0) {
                        html += "<li><i class=\"icon icon-mobile\"></i>" + val.mobile + "</li>";
                    }
                   
                    if (val.landline1 && $.trim(val.landline1).length > 0) {
                        html += "<li><i class=\"icon icon-phone\"></i>" + val.landline1 + "</li>";
                    }

                    if (val.landline2 && $.trim(val.landline2).length > 0) {
                        html += "<li><i class=\"icon icon-phone\"></i>" + val.landline2 + "</li>";
                    }

                    html += "</ul>";

                    return html;
                },
                mediaIcon: "icon icon-phone"
            });
            this.contactControl.on("select", _.bind(function () {
                this.openContactSelect();
            }, this));
            this.registerControl(this.contactControl);

            /* create gallery control */
            this.galleryControl = new uiControlTile({
                content: function (val) {
                    if (!val || !val.images || !val.images.length) {
                        if (this.isValid() == true) {
                            return "<p>" + translate("no-images.info") + "</p>";
                        }
                        return "<p>" + translate("select-images.info") + "</p>"
                    }
                   
                    var html = "<p>" + translate("uploaded-images.info").replace("[NUMBER]",val.images.length) + "</p>";

                    return html;
                },
                validate: function (val) {
                    return null;
                },
                mediaIcon: "icon icon-image"
            });
            this.galleryControl.on("select", _.bind(function () {
                this.openGallerySelect();
            }, this));
            this.registerControl(this.galleryControl);

            /* create attributes control */
            this.attributesControl = new uiControlTile({
                content: function (val) {
                    if (!val || !val.attributes || !val.attributes.length) {
                        return "<p>" + translate("select-attributes.info") + "</p>"
                    }

                    if (!self.modelAttributes.category || !self.modelAttributes.isValid()) {
                        return "<p>" + translate("select-attributes.info") + "</p>"
                    }
                   
                    var cat = self.modelAttributes.category;
                    var html = "";
                    
                    var priceName = (cat && cat.effectiveMetaData && cat.isRental ? translateShared("rent.label"):translateShared("price.label"));

                    var list = helper.prepareOfferAttributeList(self.modelAttributes.attributes.attributes);
                    html = template.offer._attributes({
                        cssClass: "dl-horizontal text-small",
                        attributes: list
                    });
                    return html;
                },
                mediaIcon: "icon icon-cog"
            });
            this.attributesControl.on("select", _.bind(function () {
                this.openAttributesSelect();
            }, this));
            this.registerControl(this.attributesControl);

            if (options.id) {
                uiContainers.header.loading(true);
                api.me.offer.get(parseInt(options.id)).done(_.bind(function (data) {
                    this.load(data);
                }, this)).always(function () {
                    uiContainers.header.loading(false);
                });
            } else {
                this.load(null);
            }
        },
        load: function (data) {
            var self = this;
            var category = null;
            var location = null;

            if (data == null) {
                // try to get category from offer filter
                var offerFilter = preferences.offerFilter();
                if (offerFilter && offerFilter.category) {
                    category = offerFilter.category;
                }

                // try to get location from preferences
                location = preferences.location();

                if (category) {
                    var apiCategoryModel = this.mapCategoryToApiModel(category);
                    this.categoryControl.loading(true);
                    api.collection.category(apiCategoryModel).done(function (data) {
                        if (!data) return;

                        var value = self.mapCategoryApiResultToModel(data);

                        self.modelCategory.set({
                            category:value
                        });

                        if (self.modelCategory.isValid(true)) {
                            self.loadAttributes();
                        };

                    }).always(function () {
                        self.categoryControl.loading(false);
                    });
                } else {
                    self.modelCategory.set({
                        category: this.createDefaultCategory()
                    });
                }
                if (location) {
                    var apiLocationModel = this.mapLocationToApiModel(location);
                    this.locationControl.loading(true);
                    api.collection.location(apiLocationModel).done(function (data) {
                        if (!data) return;

                        var value = self.mapLocationApiResultToModel(data);

                        self.modelLocation.set({
                            location:value
                        });

                    }).always(function () {
                        self.locationControl.loading(false);
                    });
                } else {
                    self.modelLocation.set({
                        location: this.createDefaultLocation()
                    });
                }

                var lastContactInfo = preferences.lastContactInfo();
                if (lastContactInfo) {
                    this.modelContact.set(lastContactInfo);
                }

            } else {
                category = data.category;
                location = data.location;

                var items = [];

                if (location.region) items.push($.extend(location.region, { level: LEVEL_REGION }));
                if (location.city) items.push($.extend(location.city, { level: LEVEL_CITY }));
                if (location.neighbourhood) items.push($.extend(location.neighbourhood, { level: LEVEL_NEIGHBOURHOOD }));

                var value = null;
                _.each(items, function (o) {
                    value = $.extend({
                        level: o.level
                    }, { parent: value }, o);
                });
                location = value;

                this.modelText.set({
                    name: data.name,
                    description: data.description
                });

                this.modelContact.set({
                    mobile: data.mobile,
                    landline1: data.landline1,
                    landline2: data.landline2
                });

                this.modelGallery.set({
                    images: data.images
                });
                
                this.modelCategory.set({
                    category: category
                });

                this.modelLocation.set({
                    location: location
                });

                this.modelAttributes.set({
                    attributes: data.attributes
                });

                this.modelCategory.validate();
                this.modelLocation.validate();
                this.modelText.validate();
                this.modelContact.validate();
                this.modelGallery.validate();
                this.modelAttributes.validate();
                
            }

            this.render();
        },
        renderTemplate: function () {
            var viewModel = {
                translate: translate
            }

            return template.me.offerEdit(viewModel);
        },
        onpagerenderdone: function () {
            this.attributesControl.create(this.$("[data-role='offer-attributes']"));
            this.galleryControl.create(this.$("[data-role='offer-gallery']"));
            this.categoryControl.create(this.$("[data-role='offer-category']"));
            this.locationControl.create(this.$("[data-role='offer-location']"));
            this.textControl.create(this.$("[data-role='offer-text']"));
            this.contactControl.create(this.$("[data-role='offer-contact']"));
        },
        save: function (e) {
            if (e) e.preventDefault();
            if (this._isSaving) return;
            var self = this;

            if (!this.isValid()) {
                this.onSaveModelValidationError();
                return;
            }

            var location = this.mapLocationToApiModel(this.modelLocation.get("location"));
            var category = this.modelCategory.get("category");
            var images = this.modelGallery.get("images");
            var attributes = $.map(this.modelAttributes.get("attributes"), function (o) {
                values = o.values;
                if (!_.isArray(values)) values = [values];
                return {
                    id: o.id,
                    values:values
                }
            });

            var apiData = $.extend({},this.modelText.toJSON(), this.modelContact.toJSON());

            apiData.category = (category == null) ? null : category.id;
            apiData.location = location;
            if (images && images.length > 0) apiData.images = images;
            if (attributes && attributes.length > 0) apiData.attributes = attributes;
            
            var deferred = $.Deferred();
            var isNew = (!this.options.id);
            if (isNew) {
                deferred = api.me.offer.add(apiData);
            } else {
                deferred = api.me.offer.update(this.options.id,apiData);
            }

            uiContainers.header.loading(true);
            this._isSaving = true;

            deferred.done(function (o) {
                preferences.lastContactInfo(self.modelContact.toJSON());
                if (isNew) {
                  //  uiToast.success(translate("success.create.info"));
                    self.navigate("me/offer/edit/" + o.id + "/doping");
                } else {
                    uiToast.success(translate("success.edit.info"));
                    self.navigateBack();
                }

            }).fail(function () {
                uiToast.danger(translate("save.error"));
            }).always(function () {
                uiContainers.header.loading(false);
                this._isSaving = false;
            });
           
        },
        isValid:function(){
            var _isValid = true;
            
            if (!this.modelText.isValid(true)) _isValid = false;
            if (!this.modelContact.isValid(true)) _isValid = false;
            if (!this.modelGallery.isValid(true)) _isValid = false;
            if (!this.modelAttributes.isValid(true)) _isValid = false;
            if (!this.modelCategory.isValid(true)) _isValid = false;
            if (!this.modelLocation.isValid(true)) _isValid = false;

            return _isValid;
        },
        initializeHeader: function (o) {
           
                o.viewSaveCancel({
                    title: this.title,
                    cancelText: translateShared("cancel"),
                    cancelIcon: "icon-cancel",
                    okText: translateShared("accept"),
                    onCancel: _.bind(function () {
                        tracking.trackEvent("offerEdit", "form", "cancel");
                        this.navigateBack();
                    }, this),
                    onOk: _.bind(function () {
                        this.save();
                    }, this)
                });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        termsOfService: function (e) {
            e.preventDefault();
            uiModal.open({},ViewTOS,this);
        },
        onremoving: function () {
            // destroy current controls
            uiDialog.options.close();
            if (this.modelText) {
                this.modelText.off();
                this.modelText = null;
            }
            if (this.modelContact) {
                this.modelContact.off();
                this.modelContact = null;
            }
            if (this.modelGallery) {
                this.modelGallery.off();
                this.modelGallery = null;
            }
            if (this.modelAttributes) {
                this.modelAttributes.off();
                this.modelAttributes = null;
            }
            if (this.modelCategory) {
                this.modelCategory.off();
                this.modelCategory = null;
            }
            if (this.modelLocation) {
                this.modelLocation.off();
                this.modelLocation = null;
            }
        },
        /* category */
        mapCategoryToApiModel: function (category) {
            var result = {
                withChildren: false,
                withParent: false,
                id: category
            }

            return result;
        },
        mapCategoryApiResultToModel: function (result) {
            if (!result || !result.length) return null;

            var value = result[0].value;
            if (value) {
                helper.applyNodeLevel(value);
            }
            return value;
        },
        openCategorySelect: function () {
            var self = this;
            uiContainers.header.loading(true);

            var oldCategory = this.modelCategory.toJSON().category;
            var oldCategoryId = (oldCategory) ? oldCategory.id : null;

            uiModal.open({
                category: oldCategory,
                rootLevelName: translate("category.any"),
                selectLastLevel: true,
                onSelect: function (val) {

                    // if same category was selected - do nothing
                    if (oldCategoryId == val.id) {
                        return;
                    }

                    self.modelCategory.set({
                        category:val
                    });

                    self.modelCategory.validate();
                    
                    self.loadAttributes();

                }
            }, ViewCategory, this).always(function () {
                uiContainers.header.loading(false);
            });
        },
        createDefaultCategory: function () {
            return {
                id: null,
                name: translate("select-category.info"),
                level: 0
            };
        },
        /* location */
        createDefaultLocation: function () {
            return {
                id: settings.countryIdIran,
                name: translate("select-location.info"),
                level: LEVEL_COUNTRY
            };
        },
        mapLocationApiResultToModel: function (result) {
            if (!result) return null;

            var items = [];

            if (result.country) {
                result.country.value.name = translate("location.any");
                items.push($.extend(result.country, { level: LEVEL_COUNTRY }));
            }
            if (result.region) items.push($.extend(result.region, { level: LEVEL_REGION }));
            if (result.city) items.push($.extend(result.city, { level: LEVEL_CITY }));
            if (result.neighbourhood) items.push($.extend(result.neighbourhood, { level: LEVEL_NEIGHBOURHOOD }));

            var value = null;
            _.each(items, function (o) {
                value = $.extend({
                    level: o.level
                }, { parent: value }, o.value);
            });

            return value;
        },
        mapLocationToApiModel: function (location) {
            var result = {
                withChildren: false
            }
            if (!location) return result;

            var current = location;
            while (current) {
                if (current.level == LEVEL_COUNTRY) {

                }
                if (current.level == LEVEL_REGION) {
                    result.region = current.id;
                }
                if (current.level == LEVEL_CITY) {
                    result.city = current.id;
                }
                if (current.level == LEVEL_NEIGHBOURHOOD) {
                    result.neighbourhood = current.id;
                }
                current = current.parent;
            }

            return result;
        },
        openLocationSelect: function () {
            var self = this;
            uiContainers.header.loading(true);
            uiModal.open({
                location: this.modelLocation.toJSON().location,
                rootLevelName: translate("location.any"),
                maxLevel:this.locationControl.maxLevel||LEVEL_CITY,
                selectLastLevel:true,
                onSelect: function (val) {
                    self.modelLocation.set({
                        location:val
                    });
                    self.modelLocation.validate();
                }
            }, ViewLocation, this).always(function () {
                uiContainers.header.loading(false);
            });
        },

        /* text */
        openTextSelect: function () {
            var self = this;
           
            uiModal.open({
                model: this.modelText.clone(),
                onSelect: function (value) {
                    self.modelText.set(value);
                    self.modelText.validate();
                }
            }, ViewOfferEditText, this);
        },

        /* contact */
        openContactSelect: function () {
            var self = this;
            uiModal.open({
                model: this.modelContact.clone(),
                onSelect: function (value) {
                    self.modelContact.set(value);
                    self.modelContact.validate();
                }
            }, ViewOfferEditContact, this);
        },

        /* gallery */
        openGallerySelect: function () {
            var self = this;
            uiModal.open({
                model: this.modelGallery.clone(),
                onSelect: function (value) {
                    self.modelGallery.set(value);
                    self.modelGallery.validate();
                }
            }, ViewOfferEditGallery, this);
        },

        /* attributes */
        openAttributesSelect: function () {
            var self = this;

            // check if we have category
            if (!this.modelCategory.isValid()) {
                uiToast.danger(translate("no-category.warning"));
                return;
            }

            // if attributes control is not loaded, exit
            if (this.attributesControl.loading()) return;

            var clonedModel = this.modelAttributes.clone();
            clonedModel.category = this.modelAttributes.category;
            uiModal.open({
                model: clonedModel,
                onSelect: function (value) {
                    self.modelAttributes.set(value);
                    self.modelAttributes.validate();
                }
            }, ViewOfferEditAttributes, this);
        },
        loadAttributes: function () {
            if (!this.modelCategory.isValid()) return $.Deferred().resolve(null);
            
            var categoryId = this.modelCategory.get("category").id;

            this.attributesControl.loading(true);
            return api.collection.getCategoryAttributes(categoryId).done(_.bind(function (data) {
                this.modelAttributes.set("attributes", data);
            }, this)).always(_.bind(function () {
                this.attributesControl.loading(false);
            },this));
        }
        
    });

    View.access = "auth";

    return View;
});