﻿define([
  'jquery',
  'underscore',
  'views/base',
  'api',
  'uiSwipe',
  'uiModal',
  'uiToast',
  'uiDialog',
  'uiGlobal',
  'uiContainers',
  'uiTimeAgo',
  'globalize',
  'libHammer',
  'models/me/offer',
  'template',
  'settings',
  'helper'
], function ($, _, ViewBase, api, uiSwipe, uiModal, uiToast, uiDialog, uiGlobal, uiContainers, uiTimeAgo, globalize, Hammer, Model, template, settings, helper) {
    var translate = globalize.localizeBase("views.me.offer-view");
    var translateShared = globalize.localizeBase("views.shared");
    var translateFooter = globalize.localizeBase("views.footer");

    var View = ViewBase.extend({
        elDelete: null,
        elEdit: null,
        elUpgrade:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.title = translate("title");

            if (options.offer) {
                this.setModel(new Model(options.offer));
                this.render();
            } else {
                this.setModel(new Model({
                    id: options.id
                }));
            }

            this.load();
        },
        load: function () {
            // fetch record
            var self = this;

            // get from cache

            // load view from cache
            //self.render();

            // load again from service
            this.fetchModel({
                data: null
            }, uiContainers.header.get()).done(function () {
                self.render();
                self.loadDopings();
            });
        },
        loadDopings: function () {
            if (!settings.source.canBuyVitamins) {
                this.renderDopingEdit(null);
                return;
            }
            api.me.offer.doping.list(this.model.id).done(_.bind(function (data) {
                this.renderDopingEdit(data);
            }, this)).fail(_.bind(function () {
                this.renderDopingEdit(null);
            },this));
        },
        renderDopingEdit: function (data) {
            if (!this.elUpgrade) return;
            
            var self = this;

            if (!data) {
                this.elUpgrade.toggleClass("disabled", true);
                return;
            }

            if (data.isExpired) {
                this.elUpgrade.hammer({
                  //  prevent_default: true
                }).off(uiGlobal.event.tap).on(uiGlobal.event.tap + ".view", _.bind(function (e) {
                    e.preventDefault();
                    uiToast.danger(translate("upgrade.offer-expired.notification"));
                }, this));
                this.elUpgrade.toggleClass("disabled", false);
                return;
            } else if (data.hasDoping) {
                // no vitamins available
                this.elUpgrade.toggleClass("disabled", true);
                return;
            }
            
            this.elUpgrade.toggleClass("disabled", false);
            this.elUpgrade.hammer({
              //  prevent_default: true
            }).off(uiGlobal.event.tap).on(uiGlobal.event.tap + ".view", function (e) {
                e.preventDefault();
                if (self.model) {
                    self.navigate("me/offer/edit/" + self.model.id + "/doping");
                }
            });
        },
        renderTemplate: function () {
            var model = this.model.toJSON();
            var imageWidth = Math.min(uiGlobal.$body.width(), 992);
            var imageHeight = Math.min(Math.floor(imageWidth * 3 / 4), 400);

            model.image = function (key) {
                return api.offer.imageUrl(key, {
                    height: imageHeight,
                    width: imageWidth,
                    mode: 1
                });
            }
            model.imageDimensions = {
                width: imageWidth,
                height: imageHeight
            };
            model.description = this.nl2br(model.description);
            model.translate = translate;
            model.priceName = (model.category && model.category.effectiveMetaData && model.category.effectiveMetaData.isRental ? translate("rent.label") : translate("price.label"));

            // remove price and price sec from attributes
            model.attributes = helper.prepareOfferAttributeList(model.attributes, [-1, -2]);

            model.format = function (val, f) {
                return globalize.format(val, f);
            };
            model.renderDate = function (val) {
                return uiTimeAgo.inWords(new Date().getTime() - helper.parseDate(val).getTime());
            };

            model.translate = translate;

            return template.me.offerView(model);
        },
        onpagerenderdone: function () {
            var model = this.model.toJSON();

            var isMultipleImages = false;
            var initialSlide = 0;
            if (model.images && model.images.length > 1) {
                isMultipleImages = true;
                _.each(model.images, function (o, i) {
                    if (o.primary) initialSlide = i;
                });
            }
            
            var swipe = new uiSwipe($('#offer-gallery')[0], {
                pagination: isMultipleImages ? "#offer-gallery-pager" : null,
                loop: isMultipleImages,
                initialSlide: initialSlide
                //cssWidthAndHeight:true
            });

        },
        onBack: function () {
            this.navigate('/me/offer');
        },
        initializeFooter: function (o) {
            if (this.elDelete) {
                this.elDelete.off(".view");
            }
            if (this.elEdit) {
                this.elEdit.off(".view");
            }
            if (this.elUpgrade) {
                this.elUpgrade.off(".view");
            }

            data = {
                items: [
                { href: "javascript:void(0);", id: "offer-edit", icon: "icon-pen", label: translate("footer.edit"), cssClass: "" },
                { href: "javascript:void(0);", id: "offer-upgrade", icon: "icon-offer-upgrade", label: translate("footer.upgrade"), cssClass: " disabled" },
                { href: "javascript:void(0);", id: "offer-delete", icon: "icon-remove", label: translate("footer.delete"), cssClass: "primary" }
                ]
            };

            o.viewDefault(data);

            var el = o.get();

            this.elDelete = $("#offer-delete", el);
            this.elEdit = $("#offer-edit", el);
            this.elUpgrade = $("#offer-upgrade", el);

            var self = this;
            this.elDelete.hammer({
               // prevent_default: true
            }).on(uiGlobal.event.tap + ".view", function (e) {
                // uiGlobal.touch(e, function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    uiDialog.confirm(translate("remove.notification")).done(function () {
                        self.onDelete();
                    });
               // });
            });
            this.elEdit.hammer({
               // prevent_default: true
            }).on(uiGlobal.event.tap + ".view", function (e) {
                e.preventDefault();
               // uiGlobal.touch(e, function (e) {
                    self.navigate("/me/offer/edit/" + self.model.id);
               // });
            });
            
        },
        onDelete: function () {
            api.me.offer.remove(this.model.id).done(_.bind(function () {
                uiToast.success(translate("remove.success.info"));
                this.navigate('/me/offer');
            }, this)).fail(function () {
                uiToast.danger(translate("remove.error.info"));
            }).always(function () {
            });
        },
      
        onremoving: function () {
            if (this.elUpgrade) {
                this.elUpgrade.off(".view");
            }
            if (this.elEdit) {
                this.elEdit.off(".view");
            }
            if (this.elDelete) {
                this.elDelete.off(".view");
            }
        }
    });
    return View;
});