﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'template',
  'uiGlobal',
  'cache',
  'uiContainers',
  'preferences',
  'uiControlBase',
  'uiControlViewList',
  'uiControlLoader',
  'uiModal',
  'views/offer/view',
  'uiControlViewOfferListItem'
], function ($,
    _,
    api,
    globalize,
    ViewBase,
    template,
    uiGlobal,
    Cache,
    uiContainers,
    preferences,
    uiControlBase,
    uiControlViewList,
    uiControlLoader,
    uiModal,
    ViewOffer,
    uiControlViewOfferListItem) {

    var translate = globalize.localizeBase("views.me.offer");
    var translateShared = globalize.localizeBase("views.shared");

    var EmptyResult = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-logo icon-6x block\"></i> <%=text %><p><a class=\"btn btn-primary\" href=\"<%=buttonLink%>\"><%=buttonText%></a></p></div></div>"),
        getRenderData: function () {
            return {
                text:translate("empty.message"),
                buttonText: translate("empty.add-offer.button"),
                buttonLink: "#me/offer/create"
            }
        }
    });

    var offerListTemplate = _.template("<div class=\"offer-list offer-list-me\" />");
    var STATUS_ONMODERATION = 3;
    var View = ViewBase.extend({
        access: "auth",
        apiData: null,
        cache: null,
        pageSize: 10,
        currentPage: 1,
        listControl: null,
        shouldEmpty: true,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            var self = this;
            this.title = translate("title");

            this.listControl = new uiControlViewList({
                template: offerListTemplate,
                scrollElement: this.$el,
                enableMore: true,
                createItem: function (item) {
                    var data = item;
                    data.offerStatusLabel = (item.active ? translate("active.label") : translate("inactive.label"));

                    if (item.moderation == STATUS_ONMODERATION || item.moderation == 4) {
                        data.offerStatusLabel = translate("onmoderation.label");
                    }

                    if (typeof item.numberOfVisits !== 'undefined') {
                        data.offerViewedLabel = translate("view-number.label").replace("[NUMBER]", (item.numberOfVisits || 0));
                    } else {
                        data.offerViewedLabel = "";
                    }
                    return new uiControlViewOfferListItem({
                        data: data,
                        template: template.me._offerListItem
                    });
                },
                more: function () {
                    return self.loadNext();
                },
                hasMore: function (data, list) {
                    var hasMore = false;

                    if (data && list && data.items && data.items) hasMore = (data.items.length > list.length);

                    return hasMore;
                },
                getList: function (data) {
                    if (data && data.items) {
                        return $.map(data.items, function (o, i) {
                            if (i >= self.pageSize) return null;
                            return o;
                        });
                    }
                    return [];
                }
            });
            this.registerControl(this.listControl);

            var loader = null;

            this.listControl.on("loadstart", function (data, opt) {
                if (opt && opt.isListEmpty) {
                    loader = new uiControlLoader();
                    self.registerControl(loader);
                    
                    self.listControl.destroy();
                    self.$el.empty();
                    self.render(loader);
                }
            }).on("loadend", function (data, opt) {
                
                if (loader) {
                    loader.destroy();
                    loader == null;
                }
                
                if (self && self.listControl && self.listControl.list.length == 0 && self.$el) {
                    self.listControl.destroy();
                    self.$el.empty();

                    var empty = new EmptyResult({});
                    self.registerControl(empty);
                    self.render(empty);

                } else if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $(template.me.offer({}));
                    self.$el.empty().append(container);
                    self.render(self.listControl, $("[data-role='list']",container));
                }

            }).on("select", function (data) {
                if (data && data.options && data.options.data) {
                    self.navigate("#me/offer/" + data.options.data.id, data.options.data);
                    //uiModal.open({
                    //    offer: data.options.data
                    //}, ViewOffer, self);
                }
            }).on("more", function () {
                self.loadNext();
            });

            this.cache = new Cache();

            this.load(this.currentPage);
        },
        load: function (page) {
            var self = this;

            this.apiData = {
                p: page,
                pageSize:this.pageSize + 1
            };

            // get from cache
            //if (page == 1) {
            //    var cacheKey = api.offer.getCacheKey(this.apiData);
            //    var items = this.cache.get(cacheKey);
            //    if (items && items.items && items.items.length > 0) {
            //        // load view from cache
            //        this.listControl.load(items, {
            //            empty: true
            //        });
            //    }
            //}

            return this.listControl.load(function () {
                return api.me.offer.get(self.apiData);
            }, {
                empty: self.shouldEmpty
            }).done(function (data) {
                self.shouldEmpty = false;

                // store to cache
              //  if (page == 1) self.cache.set(cacheKey, data);
                self.applyScroll();
            });
        },
        reload: function () {
            this.currentPage = 1;
            this.shouldEmpty = true;
            this.load(this.currentPage);
        },
       
        loadNext: function () {
            this.currentPage++;

            return this.load(this.currentPage);
        },
        //onremoving: function () {
        //    uiContainers.header.show();
        //    uiContainers.footer.show();
        //},
        onBack: function () {
            this.navigateBack();
        }
    });
    return View;
});