﻿define([
  'jquery',
  'underscore',
  'globalize',
  'views/base',
  'api',
  'template',
  'uiGlobal',
  'cache',
  'helper',
  'preferences',
  'uiControlButton',
  'uiControlLoader',
  'uiControlTile'
], function ($, _, globalize, ViewBase, api, template, uiGlobal, Cache, helper, preferences, uiControlButton, uiControlLoader, uiControlTile) {

    var translate = globalize.localizeBase("views.offer.category");
    var translateShared = globalize.localizeBase("views.shared");

    var View = ViewBase.extend({
        location:null,
        cache: null,
        currentCategoryId: null,
        currentCategory:null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.location = preferences.location();
            
            this.title = _.bind(function () {
                var title = globalize.localize("views.location.iran");
                if (this.location && this.location.name && this.location.id) title = this.location.name;
                return "<i class=\"icon icon-location\"></i> " + title;
            }, this);

            this.cache = new Cache();
            
            this.currentCategoryId = options.id || null;
            this.load(this.currentCategoryId);

        },
        reload: function () {
            this.location = preferences.location();
            this.load(this.currentCategoryId);
        },
        load: function (id) {
            var loader = new uiControlLoader();
            this.$el.html('');
            this.render(loader);

            var self = this;

            api.offer.category({
                regionId: helper.getLocationRegionId(this.location),
                categoryId: id || null,
                depth: 2
            }).done(_.bind(function (data) {
                if (loader) loader.destroy();
                
                // if only one child, then auto select category
                if (data && data.children && data.children.length == 0) {
                    self.viewCategoryOffers(self.currentCategoryId);
                    return;
                }
                if (data && data.children && data.children.length == 1) {
                    self.viewCategoryOffers(data.children[0].id);
                    return;
                }

                
                this.currentCategory = data.category;
                this.loadCategoryList(data.category,data.children);
            }, this)).fail(_.bind(function () {
                this.onError();
            },this));
        },
        loadCategoryList: function (category,list) {
            if (!this.$el) return;
            var self = this;
            var wrapper = $("<div />").addClass("content-fixed");
            var container = $("<div class=\"\" />");
                        
            if (category) {
                container.addClass("list");

                button = new uiControlTile({
                    content: function (val) {
                        return translate("offers-in-category.button").replace("[NAME]",category.name);
                    },
                    cssClass: function (val) {
                        return "primary card space-vertical no-border";
                    },
                    mediaIcon: function () {
                        return helper.toCategoryCssClass(category);
                    }
                });
                self.registerControl(button);
                button.create(wrapper);

                button.on("select", function () {
                    self.viewCategoryOffers(category.id);
                });

                var backButton = new uiControlButton();
                self.registerControl(backButton);

                var backButtonTitle = (category.parent ? translate("previous.button").replace("[NAME]", category.parent.name) : translate("previous.root.button"));
                var backButtonId = (category.parent ? category.parent.id : "");

                backButton.createIn({
                    title: backButtonTitle,
                    icon: "icon icon-chevron-up",
                    cssClass:"",
                    data: backButtonId
                }, container);

                backButton.on("select", function (data) {
                    if (data && data.options && data.options.data) {
                        self.navigate("offer/category/" + data.options.data);
                    } else {
                        self.navigate("offer/category");
                    }
                });
            } else {
                container.addClass("category-list-big");

                // if root category then add "view all offers button"
                list.unshift({
                    name: translate("all-offers.button"),
                    id: null
                });
            }

            _.each(list, _.bind(function (o) {
                var button = null;
                
                if (category) {
                    button = new uiControlButton();
                    self.registerControl(button);
                    button.createIn({
                        title: o.name,
                        id: o.id,
                        iconSec: "icon icon-chevron",
                        //badge: o.count,
                        childrenCount: o.childrenCount||0
                    }, container);
                } else {
                    //button = new uiControlTile({
                    //    content: function (val) {
                    //        return "";
                    //    },
                    //    cssClass: function (val) {
                    //        return "";
                    //    },
                    //    mediaIcon: "icon icon-category"
                    //});
                    //self.registerControl(button);
                    //button.value(o);
                    //button.create(container);
                    button = new uiControlButton();
                    self.registerControl(button);
                    button.createIn({
                        title: o.name,
                        id: o.id,
                      //  icon: (o.id == null ? "icon icon-offer-list" : helper.toCategoryCssClass(o)),
                        childrenCount: o.childrenCount||0
                    }, container);
                }

                button.on("select", function (data) {
                    
                    // if we only have 1 subcategory, go directly to offers
                    if (this.options.childrenCount <= 1) {
                        self.viewCategoryOffers(this.options.id);
                        return;
                    }
                    if (this.options.id) {
                        self.navigate("offer/category/" +this.options.id);
                    } else {
                        self.navigate("offer/category");
                    }
                    
                });
            },this));

            wrapper.append(container);

            this.$el.empty().append(wrapper);
        },
        viewCategoryOffers:function(id){
            // save category
            var filterSettings = api.offer.filterSettings() || {};
            filterSettings.category = id;
            api.offer.filterSettings(filterSettings);

            // go to offer listing
            this.navigate("offer/search");
        },
        onBack: function () {
            this.navigateBack();
        },
        isBackVisible: function () {
            return !!this.currentCategoryId;
        },
        onTitleSelect: function () {
            this.openLocationPicker();
        }
        
    });
    return View;
});