﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'template',
  'uiGlobal',
  'cache',
  'uiContainers',
  'preferences',
  'uiControlBase',
  'uiControlViewList',
  'uiControlLoader',
  'uiModal',
  'views/offer/view',
  'uiControlViewOfferListItem'
], function ($,
    _,
    api,
    globalize,
    ViewBase,
    template,
    uiGlobal,
    Cache,
    uiContainers,
    preferences,
    uiControlBase,
    uiControlViewList,
    uiControlLoader,
    uiModal,
    ViewOffer,
    uiControlViewOfferListItem) {

    var translate = globalize.localizeBase("views.offer.favorites");
    var translateShared = globalize.localizeBase("views.shared");

    var EmptyResult = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-offer-favorite-empty icon-6x block\"></i> <%=text %><p><a class=\"btn btn-primary btn-block\" href=\"<%=buttonLink%>\"><%=buttonText%></a></p></div></div>"),
        getRenderData: function () {
            return {
                text: translate("empty.message"),
                buttonText: translate("empty.search.button"),
                buttonLink: "#offer/search"
            }
        }
    });

    var offerListTemplate = _.template("<div class=\"offer-list offer-list-favorites\" />");

    var View = ViewBase.extend({
        shouldEmpty: true,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);
            
            this.title = translate("title");

            var self = this;

            this.listControl = new uiControlViewList({
                template: offerListTemplate,
                scrollElement: this.$el,
                enableMore: true,
                createItem: function (item) {
                    return new uiControlViewOfferListItem({
                        data: item
                    });
                },
                more: function () {
                    return self.loadNext();
                },
                hasMore: function (data, list) {
                    var hasMore = false;

                    if (data && list && data.items && data.items) hasMore = (data.items.length > list.length);

                    return hasMore;
                },
                getList: function (data) {
                    if (data && data.items) {
                        return $.map(data.items, function (o, i) {
                            if (i >= self.pageSize) return null;
                            return o;
                        });
                    }
                    return [];
                }
            });
            this.registerControl(this.listControl);

            var loader = null;

            this.listControl.on("loadstart", function (data, opt) {
                if (opt && opt.isListEmpty) {
                    loader = new uiControlLoader();
                    self.registerControl(loader);
                    self.listControl.destroy();
                    self.$el.empty();
                    self.render(loader);
                }
            }).on("loadend", function (data, opt) {
                
                if (loader) {
                    loader.destroy();
                    loader == null;
                }

                if (self && self.listControl && self.listControl.list.length == 0 && self.$el) {
                    self.listControl.destroy();
                    self.$el.empty();

                    var empty = new EmptyResult({});
                    self.registerControl(empty);
                    self.render(empty);

                } else if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $("<div />").addClass("content-fixed");
                    self.$el.empty().append(container);
                    self.render(self.listControl, container);

                }

            }).on("select", function (data) {
                if (data && data.options && data.options.data) {
                   // self.navigate("#offer/" + data.options.data.id, data.options.data);
                    uiModal.open({
                        offer: data.options.data
                    }, ViewOffer, self);
                }
            });

            //this.title = _.bind(function () {
            //    var location = preferences.location();
            //    var title = globalize.localize("views.location.iran");
            //    if (location && location.name) title = location.name;
            //    return "<i class=\"icon icon-location\"></i> " + title;
            //}, this);
            this.cache = new Cache();

            this.load(this.currentPage);

        },
        load: function () {
            var self = this;
           
            return this.listControl.load(function () {
                var favorites = api.offer.getFavorites();

                if (!favorites || favorites.length == 0) {
                    return null;
                }

                return api.offer.get({
                    p: 1,
                    ids: favorites
                });
            }, {
                empty: self.shouldEmpty
            }).done(function (data) {
                self.shouldEmpty = false;
                self.applyScroll();
            });
        },
        reload: function () {
            this.shouldEmpty = true;
            this.load();
        },
        onremoving: function () {
           // uiContainers.header.show();
          //  uiContainers.footer.show();
        },
        onTitleSelect: function () {
            this.openLocationPicker();
        }
        
    });
    return View;
});