﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'template',
  'uiGlobal',
  'cache',
  'helper',
  'uiContainers',
  'preferences',
  'uiControlBase',
  'uiControlViewList',
  'uiControlLoader',
  'uiModal',
  'views/offer/view',
  'uiControlViewOfferListItem'
], function ($,
    _,
    api,
    globalize,
    ViewBase,
    template,
    uiGlobal,
    Cache,
    helper,
    uiContainers,
    preferences,
    uiControlBase,
    uiControlViewList,
    uiControlLoader,
    uiModal,
    ViewOffer,
    uiControlViewOfferListItem) {

    var translate = globalize.localizeBase("views.offer.hot");
    var translateShared = globalize.localizeBase("views.shared");

    var EmptyResult = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-offer-hot icon-6x block\"></i> <%=text %><p><a class=\"btn btn-primary\" href=\"<%=buttonLink%>\"><%=buttonText%></a></p></div></div>"),
        getRenderData: function () {
            return {
                text: translate("empty.message"),
                buttonText: translate("empty.search.button"),
                buttonLink: "#offer/search"
            }
        }
    });

    var offerListTemplate = _.template("<div class=\"offer-list offer-list-hot\" />");

    var View = ViewBase.extend({
        location: null,
        pageSize: 10,
        currentPage: 1,
        apiData: null,
        shouldEmpty: true,
        cache: null,
        initialize: function () {
            this.constructor.__super__.initialize.call(this);

            this.location = preferences.location();

            this.title = translate("title");

            var self = this;
            
            this.listControl = new uiControlViewList({
                template: offerListTemplate,
                scrollElement: this.$el,
                enableMore: true,
                createItem: function (item) {
                    return new uiControlViewOfferListItem({
                        data: item
                    });
                },
                more: function () {
                    return self.loadNext();
                },
                hasMore: function (data, list) {
                    var hasMore = false;

                    if (data && list && data.items && data.items) hasMore = (data.items.length > list.length);

                    return hasMore;
                },
                getList: function (data) {
                    if (data && data.items) {
                        return $.map(data.items, function (o, i) {
                            if (i >= self.pageSize) return null;
                            return o;
                        });
                    }
                    return [];
                }
            });

            var loader = null;

            this.listControl.on("loadstart", function (data, opt) {
                if (opt && opt.isListEmpty) {
                    loader = new uiControlLoader();
                    self.listControl.destroy();
                    self.$el.empty();
                    self.render(loader);
                }
            }).on("loadend", function (data, opt) {
                if (loader) {
                    loader.destroy();
                    loader == null;
                }

                if (self && self.listControl && self.listControl.list.length == 0 && self.$el) {
                    self.listControl.destroy();
                    self.$el.empty();

                    var empty = new EmptyResult({});
                    self.render(empty);

                } else if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $("<div />").addClass("content-fixed");
                    self.$el.empty().append(container);
                    self.render(self.listControl, container);
                }

            }).on("select", function (data) {
                if (data && data.options && data.options.data) {
                   
                   uiModal.open({
                       offer: data.options.data
                   }, ViewOffer, self);
                }
            }).on("more", function () {
                self.loadNext();
            }).on("reload", function () {
                self.reload();
            });

            //this.title = _.bind(function () {
            //    this.location = preferences.location();
            //    var title = globalize.localize("views.location.iran");
            //    if (this.location && this.location.name) title = this.location.name;
            //    return "<i class=\"icon icon-location\"></i> " + title;
            //}, this);
            this.cache = new Cache();

            this.load(this.currentPage);

        },
        load: function (page) {
            var self = this;

            var locationId = helper.getLocationRegionId(this.location);

            this.apiData = {
                p: page,
                pageSize:this.pageSize + 1,
                regionId: locationId,
                filterPreset: 2
            };

            // get from cache
            if (page == 1) {
                var cacheKey = api.offer.getCacheKey(this.apiData);
                var items = this.cache.get(cacheKey);
                if (items && items.items && items.items.length > 0) {
                    // load view from cache
                    this.listControl.load(items, {
                        empty: true
                    });
                }
            }

            return this.listControl.load(function () {
                return api.offer.get(self.apiData);
            }, {
                empty: self.shouldEmpty
            }).done(function (data) {
                self.shouldEmpty = false;

                // store to cache
                if (page == 1) self.cache.set(cacheKey, data);
                self.applyScroll();
            });
        },
        reload: function () {
            this.currentPage = 1;
            this.shouldEmpty = true;
            this.load(this.currentPage);
        },
       
        loadNext: function () {
            this.currentPage++;

            return this.load(this.currentPage);
        },
        onremoving: function () {
        //    uiContainers.header.show();
         //   uiContainers.footer.show();
        },
        onTitleSelect: function () {
            this.openLocationPicker();
        }

    });
    return View;
});