﻿define([
  'jquery',
  'underscore',
  'views/base',
  'template',
  'globalize',
  'uiToast'
], function ($, _, ViewBase, template, globalize, uiToast) {

    var translateShared = globalize.localizeBase("views.shared");
    var translate = globalize.localizeBase("views.offer.search-filter.price.edit");
    
    var View = ViewBase.extend({
        value: null,
        options: null,
        events: {
            "tap [data-action='reset']": "onReset",
            "submit form": "onSave"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.options = options;
            this.value = options.value || {};

            this.title = translate("title");

            this.load();
        },
        renderTemplate: function () {
            return template.offer.searchFilterPriceRange({
                translate: translate,
                to: (this.value ? this.value.to : ""),
                from: (this.value ? this.value.from : ""),
            });
        },
        onpagerenderdone:function(){

        },
        onSelect: function (data) {
            this.value = data;
            this.save();
        },
        load: function (value) {
            this.render();
        },
        initializeFooter: function (o) {
            o.hide();
        },
        onSave:function(e){
            if (e) e.preventDefault();
            this.save();
        },
        save: function () {
            this.hideKeyboard();

            var elFrom = $("input[name='from']", this.$el);
            var elTo = $("input[name='to']", this.$el);

            // validate stuff
            this.value = this.value || {};
            this.value.from = elFrom.val();
            this.value.to = elTo.val();

            if (this.options && this.options.onSelect) {
                this.options.onSelect(this.value);
            }
            this.navigateBack();
        },
        onReset: function () {
            this.value = null;

            uiToast.open(translate("any.selected"));

            if (this.options && this.options.onSelect) {
                this.options.onSelect(this.value);
            }
            this.navigateBack();
        },
        initializeHeader: function (o) {
           
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                onOk: _.bind(function (e) {
                    this.onSave(e);
                }, this),
                onCancel: _.bind(function (e) {
                    this.navigateBack();
                }, this)
            });
        }
       
    });
    return View;
});