﻿define([
  'jquery',
  'underscore',
  'api',
  'uiModal',
  'uiToast',
  'views/base',
  'template',
  'globalize',
  'preferences',
  'uiControlBase',
  'uiControlTile',
  'uiGlobal',
  'uiContainers',
  'settings',
  'helper',
  'views/location',
  'views/category',
  'views/offer/search-filter-price-range',
], function ($, _, api, uiModal, uiToast, ViewBase, template, globalize, preferences, uiControlBase,uiControlTile, uiGlobal, uiContainers, settings, helper,ViewLocation, ViewCategory, ViewOfferPriceRange) {

    var translate = globalize.localizeBase("views.offer.search-filter");
    var translateShared = globalize.localizeBase("views.shared");

    var LEVEL_COUNTRY = 1;
    var LEVEL_REGION = 2;
    var LEVEL_CITY = 3;
    var LEVEL_NEIGHBOURHOOD = 4;

    var DEFAULT_SORT = 1;

    var View = ViewBase.extend({
        itemTemplate: _.template("<a data-action=\"<%=action%>\" data-val=\"<%=((typeof val!='undefined')?val:null)%>\"><i class=\"icon icon-chevron\"></i><%=name%></a>"),
        filterSettings: null,
        preferencesLocation:null,
        selectedCategoryPath: null,
        locationControl: null,
        categoryControl: null,
        priceControl: null,
        sortControl: null,
        imageControl: null,
        events: {
            "tap [data-action='reset-filter']": "onReset",
            "submit form": "onSave"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            this.title = translate("title");

            this.filterSettings = api.offer.filterSettings() || {};
            this.preferencesLocation = preferences.location() || this.creteDefaultLocation();

            var self = this;

            this.locationControl = new uiControlTile({
                content: function (val) {
                    if (!val) {
                        return "<p>" + translate("location.any") + "</p>"
                    }
                    var html = "<p>" + val.name + "</p>";

                    var htmlContent = "";
                    var current = val.parent;
                    while (current && current.level > LEVEL_COUNTRY) {
                        
                        htmlContent += "<li>" + current.name + "</li>";
                        current = current.parent;
                    }
                    if (htmlContent) {
                        html += "<ul class=\"small list-path\">" + htmlContent + "</ul>";
                    }

                    return html;
                },
                cssClass: function (val) {
                    if (preferences.hasOfferFilterLocation(val)) return "success";

                    return "";
                },
                mediaIcon:"icon icon-location"
            });
            this.categoryControl = new uiControlTile({
                content: function (val) {
                    if (!val) {
                        return "<p>" + translate("category.any") + "</p>"
                    }
                    
                    var html = "<p><i class=\"icon " + helper.toCategoryCssClass(val) + "\"></i>" + val.name + "</p>";
                    var items = helper.toCategoryParentList(val.parent);
                 //   if (items.length > 0) items.splice(0,1);
                    if (items.length > 2) {
                        items = [
                            items[0], null, items[items.length - 1]
                        ]
                    }
                    var htmlContent = "<ul class=\"small list-path\">" + $.map(items, function (o, i) {
                        if (!o) return "<li>...</li>";
                        if (i == 0) {
                            return "<li>" + o.name + "</li>";
                        }
                        return "<li>" + o.name + "</li>";
                    }).join('') + "</ul>";
                    
                    if (htmlContent) {
                        html += "<ul class=\"small list-path\">" + htmlContent + "</ul>";
                    }

                    return html;
                },
                cssClass: function (val) {
                    if (preferences.hasOfferFilterCategory(val)) return "success";

                    return "";
                },
                mediaIcon: function (val) {
                    return "icon icon-category";
                }
            });
            this.priceControl = new uiControlTile({
                content: function (val) {
                    if (val && val.from && val.to) {
                        return "<p>" + translate("price.from-to").replace("[FROM]", val.from).replace("[TO]", val.to) + "</p>";
                    }

                    if (val && val.from) return "<p>" + translate("price.from").replace("[FROM]", val.from) + "</p>";
                    if (val && val.to) return "<p>" + translate("price.to").replace("[TO]", val.to) + "</p>";

                    return "<p>" + translate("price.any") + "</p>";
                },
                cssClass: function (val) {
                    if (val && (val.from > 0 || val.to > 0)) return "success";
                    return "";
                },
                mediaIcon: "icon icon-price"
            });
            this.sortControl = new uiControlTile({
                content: function (val) {
                    return "<p>" + (val ? val.title : globalize.localize("collections.offerFilter.sortPreset.mostrecent")) + "</p>";
                },
                cssClass: function (val) {
                    if (preferences.hasOfferFilterSort(val)) return "success";
                    return "";
                },
                mediaIcon: "icon icon-sort"
            });
            this.imageControl = new uiControlTile({
                content: function (val) {
                    var result = (val ? translate('pictures-only.label') : translate('pictures-only.label'));

                    return "<p>" + result + "</p>";
                    
                },
                cssClass: function (val) {
                    if (val) return "success";
                    return "";
                },
                
                mediaIcon: "icon icon-image"
            });

            this.locationControl.maxLevel = LEVEL_CITY;

            this.registerControl(this.locationControl);
            this.registerControl(this.categoryControl);
            this.registerControl(this.priceControl);
            this.registerControl(this.sortControl);
            this.registerControl(this.imageControl);

            this.locationControl.on("select", function () {
                self.openLocationSelect();
            });
            this.categoryControl.on("select", function () {
                self.openCategorySelect();
            });
            this.categoryControl.on("changed", function () {
                var value = this.value();

                if (value && value.effectiveMetaData && value.effectiveMetaData.isRealestate) {
                    self.locationControl.maxLevel = LEVEL_NEIGHBOURHOOD;
                    var loc = self.locationControl.value();
                    self.locationControl.value(loc);
                } else {
                    // if neighbourhood is selected, then select parent
                    var loc = self.locationControl.value();
                    if (loc && loc.level == LEVEL_NEIGHBOURHOOD) {
                        self.locationControl.value(loc.parent);
                    }
                }

                // self.loadAttributes();
            });

            this.imageControl.on("select", function () {
                this.value(!this.value());
            });

            this.sortControl.on("select", function () {
                self.openSortSelect();
            });
            this.priceControl.on("select", function () {
                self.openPriceSelect();
            });

            this.render();
            this.load();
        },
        renderTemplate: function () {
            return template.offer.searchFilter({
                translate:translate
            });
        },
        mapLocationToApiModel: function (location) {
            var result = {
                withChildren: false
            }
            if (!location) return result;

            var current = location;
            while (current) {
                if (current.level == LEVEL_COUNTRY) {

                }
                if (current.level == LEVEL_REGION) {
                    result.region = current.id;
                }
                if (current.level == LEVEL_CITY) {
                    result.city = current.id;
                }
                if (current.level == LEVEL_NEIGHBOURHOOD) {
                    result.neighbourhood = current.id;
                }
                current = current.parent;
            }

            return result;
        },
        mapCategoryToApiModel: function (category) {
            var result = {
                withChildren: false,
                withParent:false,
                id:category
            }

            return result;
        },
        load: function () {
            var self = this;
            
            var currentLocation = this.filterSettings.location;
            if (!currentLocation) {
                currentLocation = this.preferencesLocation;
            }

            // load location
            if (currentLocation) {
                var apiLocationModel = this.mapLocationToApiModel(currentLocation);
                this.locationControl.loading(true);
                api.collection.location(apiLocationModel).done(function (data) {
                    if (!data) return;

                    var value = self.mapLocationApiResultToModel(data);

                    self.locationControl.value(value);

                }).always(function () {
                    self.locationControl.loading(false);
                });
            } else {
                self.locationControl.value(this.creteDefaultLocation());
            }

            // load category
            if (this.filterSettings.category) {
                var apiCategoryModel = this.mapCategoryToApiModel(this.filterSettings.category);
                this.categoryControl.loading(true);
                api.collection.category(apiCategoryModel).done(function (data) {
                    if (!data) return;

                    var value = self.mapCategoryApiResultToModel(data);

                    self.categoryControl.value(value);

                }).always(function () {
                    self.categoryControl.loading(false);
                });
            } else {
                self.categoryControl.value(this.creteDefaultCategory());
            }
            
            // load sort
            var sort = this.filterSettings.sortPreset || DEFAULT_SORT;
            api.collection.offerFilter.sortPreset(sort).done(function (data) {
                self.sortControl.value(data);
            });
           
            // load price
            this.priceControl.value(this.filterSettings.price || {});

            // load image list
            this.imageControl.value(!!this.filterSettings.withImage);
              
        },
        onpagerenderdone: function () {
            this.imageControl.create($("#filter-image"));
            this.categoryControl.create($("#filter-category"));
            this.locationControl.create($("#filter-location"));
            this.priceControl.create($("#filter-price"));
            this.sortControl.create($("#filter-sort"));
            uiContainers.header.searchText(this.filterSettings.keywords);
        },
        openLocationSelect: function () {
            var self = this;
            uiContainers.header.loading(true);
            uiModal.open({
                location: this.locationControl.value(),
                rootLevelName: translate("location.any"),
                maxLevel: this.locationControl.maxLevel || LEVEL_CITY,
                onSelect: function (val) {
                    self.locationControl.value(val);
                }
            }, ViewLocation, this).always(function () {
                uiContainers.header.loading(false);
            });
        },
        openCategorySelect: function () {
            var self = this;
            uiContainers.header.loading(true);
            uiModal.open({
                category: this.categoryControl.value(),
                rootLevelName: translate("category.any"),
                onSelect: function (val) {
                    self.categoryControl.value(val);
                }
            }, ViewCategory, this).always(function () {
                uiContainers.header.loading(false);
            });
        },
        onBack: function () {
            this.navigateBack();
        },
        initializeFooter: function (o) {
            o.viewNone();
        },

        initializeHeader: function (o) {
            o.viewSaveCancel({
                titleType: "search",
                titleSearchValue: this.filterSettings.keywords,
                titleSearchPlaceholder: translate("keywords.placeholder"),
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translate("search.button"),
                okIcon:"icon-search",
                onOk: $.proxy(function (e) {
                    this.onSave(e);
                }, this),
                onCancel: $.proxy(function (e) {
                    this.navigateBack();
                }, this)
            });
        },
        onSave: function (e) {
            if (e) e.preventDefault();
            this.save();
            this.navigate("offer/search");
        },
        save: function () {
            this.filterSettings.sortPreset = (this.sortControl.value()?this.sortControl.value().id : null);
            this.filterSettings.price = this.priceControl.value();
            this.filterSettings.location = this.locationControl.value();
            if (helper.isLocationEqual(this.filterSettings.location, this.preferencesLocation)) this.filterSettings.location = null;

            this.filterSettings.category = (this.categoryControl.value()||this.creteDefaultCategory()).id;

            this.filterSettings.keywords = uiContainers.header.searchText();
            this.filterSettings.withImage = this.imageControl.value();
            
            api.offer.filterSettings(this.filterSettings);
        },
        openSortSelect: function () {
         
            var self = this;
            var selectedId = (this.sortControl.value()?this.sortControl.value().id : 1);
            var data = function () {
                var deferred = $.Deferred();
                api.collection.offerFilter.sortPreset().done(function (data) {

                    var result = $.map(data, function (o, i) {
                        return {
                            id: o.id,
                            title: o.title,
                            selected: (o.id == selectedId)
                        }
                    });

                    deferred.resolve(result);
                });

                return deferred;
            }
            
            uiModal.openSelectList({
                title: translate("sort.select"),
                list: data,
                onSelect: _.bind(function (data) {
                    this.sortControl.value(data);
                },this)
            },this);
        },
        openPriceSelect: function () {
            var self = this;

            uiModal.open({
                value: this.priceControl.value(),
                onSelect: _.bind(function (val) {
                    this.priceControl.value(val);
                },this)
            },ViewOfferPriceRange, this);
        },
        mapLocationApiResultToModel: function (result) {
            if (!result) return null;

            var items = [];

            if (result.country) {
                result.country.value.name = translate("location.any");
                items.push($.extend(result.country, { level: LEVEL_COUNTRY }));
            }
            if (result.region) items.push($.extend(result.region, { level: LEVEL_REGION }));
            if (result.city) items.push($.extend(result.city, { level: LEVEL_CITY }));
            if (result.neighbourhood) items.push($.extend(result.neighbourhood, { level: LEVEL_NEIGHBOURHOOD }));
            
            var value = null;
            _.each(items, function (o) {
                value = $.extend({
                    level:o.level
                }, { parent: value }, o.value);
            });

            return value;

        },
        mapCategoryApiResultToModel: function (result) {
            if (!result || !result.length) return null;

            var value = result[0].value;
            if (value) {
                helper.applyNodeLevel(value);
            }
            return value;
        },
        onReset: function () {
            uiContainers.header.searchText("");
            this.imageControl.value(false);
            this.sortControl.value(null);
            this.priceControl.value(null);
            this.categoryControl.value(null);
            this.locationControl.value(this.preferencesLocation);

            uiToast.open(translate("reset.success"));
        },
        creteDefaultLocation: function () {
            return {
                id: settings.countryIdIran,
                name:translate("location.any"),
                level: LEVEL_COUNTRY
            };
        },
        creteDefaultCategory: function () {
            return {
                id: null,
                name: translate("category.any"),
                level: 1
            };
        }
    });
    return View;
});