﻿define([
  'jquery',
  'underscore',
  'api',
  'globalize',
  'views/base',
  'template',
  'uiGlobal',
  'cache',
  'helper',
  'uiToast',
  'uiContainers',
  'preferences',
  'uiControlBase',
  'uiControlViewList',
  'uiControlLoader',
  'uiModal',
  'views/offer/view',
  'uiControlViewOfferListItem'
], function ($,
    _,
    api,
    globalize,
    ViewBase,
    template,
    uiGlobal,
    Cache,
    helper,
    uiToast,
    uiContainers,
    preferences,
    uiControlBase,
    uiControlViewList,
    uiControlLoader,
    uiModal,
    ViewOffer,
    uiControlViewOfferListItem) {

    var LEVEL_COUNTRY = 1;
    var LEVEL_REGION = 2;
    var LEVEL_CITY = 3;
    var LEVEL_NEIGHBOURHOOD = 4;

    var translate = globalize.localizeBase("views.offer.search");
    var translateShared = globalize.localizeBase("views.shared");

    var EmptyResult = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-logo icon-6x block\"></i> <%=text %><p><a class=\"btn btn-block btn-primary nowrap\" href=\"<%=buttonLink%>\"><%=buttonText%></a></p></div></div>"),
        getRenderData: function () {
            return {
                text: translate("empty.message"),
                buttonText:  translate("empty.filter.button"),
                buttonLink:"#offer/search-filter"
            }
        }
    });

    var offerListTemplate = _.template("<div class=\"offer-list offer-list-search\"></div>");
    var categoryPathListTemplate = _.template("<div class=\"list\"></div>");

    var View = ViewBase.extend({
        filter: null,
        apiData: null,
        cache: null,
        pageSize: 10,
        currentPage: 1,
        listControl: null,
        shouldEmpty: true,
        breadCrumbModel: null,
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this);

            var self = this;
          //  this.title = translate("title");
            this.filter = preferences.offerFilter() || {};

            this.listControl = new uiControlViewList({
                template: offerListTemplate,
                scrollElement:this.$el,
                enableMore: true,
                createItem: function (item) {
                    return new uiControlViewOfferListItem({
                        data:item
                    });
                },
                more:function(){
                    return self.loadNext();
                },
                hasMore: function (data, list) {
                    var hasMore = false;
                    
                    if (data && list && data.items && data.items) hasMore = (data.items.length > list.length);

                    return hasMore;
                },
                getList: function (data) {
                    if (data && data.items) {
                        return $.map(data.items, function (o, i) {
                            if (i >= self.pageSize) return null;
                            return o;
                        });
                    }
                    return [];
                }
            });
          //  this.registerControl(this.listControl);

            var loader = null;

            this.listControl.on("loadstart", function (data, opt) {
                if (opt && opt.isListEmpty) {
                    loader = new uiControlLoader();
                    self.listControl.destroy();
                    self.$el.empty();
                    self.render(loader);
                }
            }).on("loadend", function (data, opt) {
                
                if (loader) {
                    loader.destroy();
                    loader == null;
                }
                
                if (self && self.listControl && self.listControl.list.length == 0 && self.$el) {
                    self.listControl.destroy();
                    self.$el.empty();

                    var empty = new EmptyResult();
                    self.render(empty);

                } else if (self.listControl && !self.listControl.created() && self.$el) {
                    var container = $("<div />").addClass("content-fixed");
                    self.$el.empty().append(container);
                    self.render(self.listControl, container);
                    
                }
                
            }).on("select", function (data) {
                if (data && data.options && data.options.data) {
                    uiModal.open({
                        offer: data.options.data
                    }, ViewOffer, self);
                }
            }).on("more", function () {
                self.loadNext();
            }).on("loaderror", function (data) {
                self.onError();
            }).on("reload", function () {
                self.reload();
            });
            
            this.title = _.bind(function () {
                var location = preferences.location();
                var title = globalize.localize("views.location.iran");
                if (location && location.name) title = location.name;
                return "<i class=\"icon icon-location\"></i> " + title;
            },this);
            this.cache = new Cache();

            if (this.filter.category) {
                uiContainers.headerSec.view({
                    render: function () {
                        return "<div class=\"content-fixed loading\"><i class=\"icon icon-spinner spin\"></i></div>";
                    }
                });
               
                api.collection.category({
                    id: this.filter.category,
                    withParent: true,
                    withChildren: false
                }).done(function (data) {
                    // render category
                    self.breadCrumbModel = helper.toCategoryParentList(data);
                    self.showCategoryPath(self.breadCrumbModel);
                });
            } else {
                self.breadCrumbModel = null;
                self.showCategoryPath(self.breadCrumbModel);
            }

            this.load(this.currentPage);
        },
        initializeHeaderSec: function (o) {
            this.showCategoryPath(this.breadCrumbModel);
        },
        load: function (page,shouldEmpty) {
            var self = this;

            this.apiData = {
              //  p: page,
              //  pageSize: this.pageSize + 1,
                take:this.pageSize + 1,
                skip:(page - 1)*this.pageSize,
                q: $.trim(this.filter.keywords),
                withImage: this.filter.withImage,
                priceFrom: (this.filter.price && this.filter.price.from) ? this.filter.price.from.toLatinDigits() : null,
                priceTo: (this.filter.price && this.filter.price.to) ? this.filter.price.to.toLatinDigits() : null,
                categoryId:this.filter.category,
                datePreset: this.filter.datePreset,
                sortPreset:this.filter.sortPreset||1
            };
            
            this.apiData = $.extend(this.apiData, this.mapLocationToApiModel(this.filter.location || preferences.location()));

            // get from cache
            if (page == 1) {
                var cacheKey = api.offer.getCacheKey(this.apiData);
                var items = this.cache.get(cacheKey);
                if (items && items.items && items.items.length > 0) {
                    // load view from cache
                    this.listControl.load(items, {
                        empty: true
                    });
                }
            }

            return this.listControl.load(function () {
                return api.offer.get(self.apiData);
            }, {
                empty: (shouldEmpty || self.shouldEmpty)
            }).done(function (data) {
                self.shouldEmpty = false;

                // store to cache
                if (page == 1) self.cache.set(cacheKey, data);
                self.applyScroll();
            });
        },
        showCategoryPath: function (data) {
            if (!data) {
                uiContainers.headerSec.hide();
                return;
            }
            
            var items = [];
            if (data.length > 2) {
                items = [
                    data[0],null,data[data.length-1]
                ]
            } else {
                items = data;
            }

            // click will link to last selected category
            var categoryListId = null;
            var i = data.length - 1;
            
            while (i >= 0) {
                var o = data[i];
                if (o && o.childrenCount) {
                    categoryListId = o.id;
                    break;
                }
                i--;
            }

            var self = this;
            uiContainers.headerSec.view({
                render: function () {
                    var html = "<ul class=\"list-path content-fixed\">" + $.map(items, function (o,i) {
                        if (!o) return "<li>...</li>";
                        if (i == 0) {
                            return "<li><i class=\"icon " + helper.toCategoryCssClass(o)+ "\"></i>" + o.name + "</li>";
                        }
                        return "<li>"+o.name+"</li>";
                    }).join('') + "</ul>";
                    return html;
                },
                onCreate: function (el) {
                    if (el) {
                        el.hammer().on(uiGlobal.event.tap, function (e) {
                            e.preventDefault();
                            e.stopPropagation();
                            if (!categoryListId) {
                                self.navigate("offer/category");
                            } else {
                                self.navigate("offer/category/" + categoryListId);
                            }
                        });
                       
                    }
                }
            });
        },
        reload: function () {
            this.currentPage = 1;
            this.shouldEmpty = true;
            this.filter = api.offer.filterSettings() || {};
            return this.load(this.currentPage,true);
        },
       
        loadNext: function () {
            this.currentPage++;

            return this.load(this.currentPage);
        },
        //onremoving: function () {
        //    uiContainers.header.show();
        //    uiContainers.footer.show();
        //    uiContainers.headerSec.viewNone();
        //},
        onTitleSelect: function () {
            this.openLocationPicker();
        },
        mapLocationToApiModel: function (location) {
            var result = {
            }
            if (!location) return result;

            var current = location;
            while (current) {
                if (current.level == LEVEL_COUNTRY) {

                }
                if (current.level == LEVEL_REGION) {
                    result.regionId = current.id;
                }
                if (current.level == LEVEL_CITY) {
                    result.cityId = current.id;
                }
                if (current.level == LEVEL_NEIGHBOURHOOD) {
                    result.neighbourhoodId = current.id;
                }
                current = current.parent;
            }

            return result;
        }
        
    });
    return View;
});