﻿define([
  'jquery',
  'underscore',
  'views/base',
  'template',
  'models/offer/message',
  'api',
  'uiToast',
  'globalize',
  'tracking',
  'uiControlBase',
  'uiContainers'
], function ($, _, ViewBase, template, Model, api, uiToast, globalize, tracking, uiControlBase, uiContainers) {

    var translate = globalize.localizeBase("views.offer.view-message");
    var translateShared = globalize.localizeBase("views.shared");

    var STATUS_NOT_CONFIRMED = 1;
    var STATUS_ERROR = 2;

    var NotConfirmedResult = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-logo icon-4x block\"></i> <%=text %></div></div>"),
        getRenderData: function () {
            return {
                text: translate("not-confirmed.info")
            }
        }
    });

    var View = ViewBase.extend({
        offer: null,
        validatorSelectorMask: "[name='&']",
        isAuthenticated: false,
        user:null,
        events: {
            "tap [name='sendCopy']": "onSendCopySelect"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this, options);
            this.offer = options.offer;
            this.title = translate("title");
            this.isAuthenticated = api.isAuthenticated();

            var model = new Model();

            this.setModel(model);
            this.render();
        },
        trackPage: function (options) {
            tracking.trackPage("#offer/" + (options&&options.offer? options.offer.id : "") + "/message");
        },
        renderTemplate: function () {
            var model = {
                offer: this.offer,
                message: "",
                sendCopy: false,
                isAuthenticated:this.isAuthenticated,
                translate:translate
            }
            return template.offer.viewMessage(model);
        },
        onpagerenderdone: function () {
            if (this.isAuthenticated) {
                api.me.get().done(_.bind(function (data) {
                    this.user = data;
                    this.$("[data-role='email']").html(data.email);
                },this));
            }
        },
        onSendCopySelect:function(e){
            $("[name='sendCopy']", this.$el).toggleClass("selected");
        },
        save: function () {
            var data = {
                message: $("[name='message']", this.$el).val(),
                sendCopy: $("[name='sendCopy']", this.$el).hasClass("selected"),
                email: (this.user?this.user.email:$("[name='email']", this.$el).val()),
                offerId:this.offer.id
            };
            uiContainers.header.loading(true);
            this.saveModel(data).done(_.bind(function (data) {
                if (data && data.status == STATUS_NOT_CONFIRMED) {
                    this.renderNotConfirmed();
                    return;
                }
                if (data && data.status == STATUS_ERROR) {
                    uiToast.danger(translate("error.info"));
                    return;
                }
                uiToast.success(translate("success.info"));
                this.navigateBack();
            }, this)).fail(_.bind(function (data) {
                uiToast.danger(translate("error.info"));
            }, this)).always(function () {
                uiContainers.header.loading(false);
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },

        initializeHeader: function (o) {
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                onOk: $.proxy(function (e) {
                    this.save();
                }, this),
                onCancel: $.proxy(function (e) {
                    this.navigateBack();
                }, this)
            });
        },
        renderNotConfirmed: function () {
            this.$el.empty();

            uiContainers.header.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                cancelIcon: "icon-cancel",
                okText: translateShared("accept"),
                okIcon: "icon-accept",
                okEnabled:false,
                onCancel: $.proxy(function (e) {
                    this.navigateBack();
                }, this)
            });

            var res = new NotConfirmedResult();
            this.render(res);
        },
       
    });
    return View;
});