﻿define([
  'jquery',
  'underscore',
  'api',
  'helper',
  'uiSwipe',
  'views/base',
  'models/offer',
  'uiModal',
  'uiToast',
  'uiDialog',
  'template',
  'uiGlobal',
  'globalize',
  'uiTimeAgo',
  'uiSocialShare',
  'uiControlBase',
  'libHammer',
  'tracking',
  'uiContainers',
  'views/offer/view-message'
], function ($, _, api, helper, uiSwipe, ViewBase, Model, uiModal, uiToast, uiDialog, template, uiGlobal,
    globalize,
    uiTimeago,
    uiSocialShare,
    uiControlBase,
    Hammer,
    tracking,
    uiContainers,
    ViewMessage) {

    var translate = globalize.localizeBase("views.offer.view");
    var translateShared = globalize.localizeBase("views.shared");

    var NotFoundView = uiControlBase.extend({
        template: _.template("<div class=\"light block-block\"><div class=\"center-center\"><i class=\"icon icon-sad icon-6x block\"></i> <%=text %></div></div>"),
        
        getRenderData: function () {
            return {
                text: translate("404.info")
            }
        }
    });

    var View = ViewBase.extend({
        isFavorite: false,
        elContact: null,
        elFavorite: null,
        elShare: null,
        events: {
            "tap [name='browseMore']": "onBrowseMore"
        },
        initialize: function (options) {
            this.constructor.__super__.initialize.call(this,options);
            
            this.title = translate("title");
            
            if (options.offer) {
                this.setModel(new Model(options.offer));
                this.render();
            } else {
                this.setModel(new Model({
                    id: options.id
                }));
            }
                //this.setModel(new Model({
                //    id: options.id || options.offer.id
                //}));
            
            this.load();
        },
        trackPage: function (options) {            
            tracking.trackPage("#offer/" + (options&&options.offer ? options.offer.id : ""));
        },
        load: function () {
            // fetch record
            var self = this;

            // get from cache

            // load view from cache
            //self.render();

            // load again from service
            this.fetchModel({
                data: {
                    markAsViewed:true
                }
            }, uiContainers.header.get()).done(function () {
                self.render();
            }).fail(_.bind(function (data) {
                uiContainers.footer.hide();
                if (data && data.status == 404) {
                    self.onError(data, NotFoundView);
                } else {
                    self.onError(data);
                }
            },this));
        },
        renderTemplate: function () {
            var model = this.model.toJSON();
            var imageWidth = Math.min(uiGlobal.$body.width(), 992);
            var imageHeight = Math.min(Math.floor(imageWidth*3/4),400);
            
            model.image = function (key) {
                return api.offer.imageUrl(key, {
                    height: imageHeight,
                    width: imageWidth,
                    mode:1
                });
            }
            model.imageDimensions = {
                width: imageWidth,
                height: imageHeight
            };
            model.description = this.nl2br(model.description);
            model.translate = translate;
            model.priceName = (model.category && model.category.effectiveMetaData && model.category.effectiveMetaData.isRental ? translate("rent.label") : translate("price.label"));

            // remove price and price sec from attributes
            model.attributes = helper.prepareOfferAttributeList(model.attributes, [-1, -2]);

            model.format = function (val, f) {
                return globalize.format(val, f);
            };
            model.renderDate = function (val) {
                return uiTimeago.inWords(new Date().getTime() - helper.parseDate(val).getTime());
            };

            model.translate = translate;

            return template.offer.view(model);
        },
        onpagerenderdone: function () {
            var model = this.model.toJSON();
            
            var isMultipleImages = false;
            var initialSlide = 0;
            if (model.images && model.images.length > 1) {
                isMultipleImages = true;
                _.each(model.images, function (o, i) {
                    if (o.primary) initialSlide = i;
                });
            }

            var galleryEl= this.$('.offer-gallery');
            var swipe = galleryEl.data("swiper");
            if (swipe) {
                swipe.destroy();
            }

            swipe = galleryEl.swiper({
                pagination: isMultipleImages?"#offer-gallery-pager":null,
                loop: isMultipleImages
                //cssWidthAndHeight:true
            });

            var id = this.model.get("id");
            if (id) {
                this.isFavorite = api.offer.isFavorite(id);
                this.renderFavorite();
            }

           this.applyScroll();
        },
        onBrowseMore: function (e) {
            e.preventDefault();
           
            var cat = this.model.get("category");
            if (cat) {
                // save category
                var filterSettings = api.offer.filterSettings() || {};
                filterSettings.category = cat.id;
                api.offer.filterSettings(filterSettings);
            }

            tracking.trackEvent("offerView", "browseMore", "click", (this.model ? this.model.id : null));

            // go to offer listing
            this.navigate("offer/search");
        },
        onBack: function () {
            this.navigateBack();
        },
        //initializeHeader: function (o) {
        //    o.viewSaveCancel({
        //        title: this.title,
        //        cancelText: translateShared("close"),
        //        cancelIcon: "icon-cancel",
        //        okEnabled:false,
        //        onOk: $.proxy(function (e) {
        //            this.save();
        //        }, this),
        //        onCancel: $.proxy(function (e) {
        //            this.navigateBack();
        //        }, this)
        //    });
        //},
        initializeFooter: function (o) {
            if (this.elFavorite) {
                this.elFavorite.off(".view");
            }
            if (this.elContact) {
                this.elContact.off(".view");
            }
            if (this.elShare) {
                this.elShare.off(".view");
            }
                      
            data = {
                items: [
                { href: "javascript:void(0);", id: "toggle-favorite", icon: "mark", label: translate("footer.favorite-add"), cssClass: "offer-favorite" },
                { href: "javascript:void(0);", id: "offer-contact", icon: "icon-phone", label: translate("footer.contact"), cssClass: "primary active" },
                { href: "javascript:void(0);", id: "toggle-offer-share", icon: "icon-share", label: translate("footer.share"), cssClass: (uiSocialShare.available() ? "" : " disabled") }
                ]
            };

            o.viewDefault(data);
            
            var el = o.get();

            this.elContact = $("#offer-contact", el);
            this.elFavorite = $("#toggle-favorite", el);
            this.elShare = $("#toggle-offer-share", el);

            var self = this;
            this.elFavorite.hammer({
              //  prevent_default: true
            }).off(uiGlobal.event.tap + ".view").on(uiGlobal.event.tap + ".view", function (e) {
                e.preventDefault();
                self.onFavorite();
            });
            
            this.elShare.hammer({
              //  prevent_default: true
            }).off(uiGlobal.event.tap + ".view").on(uiGlobal.event.tap + ".view", function (e) {
                tracking.trackEvent("offerView", "share", "open");
                
                e.preventDefault();
                var model = self.model.toJSON();
                var imageUrl = null;
                if (model.images) {
                    var image = _.find(model.images, function (o) {
                        return o.primary;
                    });
                    if (image) {
                        imageUrl = api.offer.imageUrl(image.key);
                    }
                }
                var shareMessage = translate("footer.share.message");
                if (model.url) {
                    shareMessage = shareMessage + "\n\n" + model.url;
                }
                uiSocialShare.share(shareMessage, translate("footer.share.title"), imageUrl, model.url || "");
            });
            this.elContact.hammer({
               // prevent_default: true
            }).off(uiGlobal.event.tap + ".view").on(uiGlobal.event.tap + ".view", function (e) {
                e.preventDefault();
                e.stopPropagation();
                tracking.trackEvent("offerView", "contact", "open");
               // uiGlobal.touch(e, function (e) {
                    var model = self.model.toJSON();
                    var methods = [];

                    // disable email contanct method if agent created this offer
                    if (!model.isAgent) {
                        methods.push({ name: translate("footer.contact.send-message"), icon: "icon-message", action: "message" });
                    }
                    if (model && model.mobile) {
                        methods.push({ name: translate("footer.contact.call-mobile").replace("[PHONE]",model.mobile), icon: "icon-mobile", action: "call", value: model.mobile });
                    }
                    if (model && model.landline1) {
                        methods.push({ name: translate("footer.contact.call-landline1").replace("[PHONE]", model.landline1), icon: "icon-phone", action: "call", value: model.landline1 });
                    }
                    if (model && model.landline2) {
                        methods.push({ name: translate("footer.contact.call-landline2").replace("[PHONE]", model.landline2), icon: "icon-phone", action: "call", value: model.landline2 });
                    }

                    if (methods.length == 0) {
                        return;
                    }

                    methods.push({ name: translate("footer.contact.cancel"), cssClass:"btn btn-danger"});
                    uiDialog.options.open(translate("footer.contact.label"), methods).done(function (sel) {
                        if (sel && sel.action) {
                            if (sel.action == 'message') {
                                self.onMessage();
                            }
                            if (sel.action == 'call') {
                                tracking.trackEvent("offerView", "contact", "call");
                                window.location.href = 'tel:' + sel.value;
                            }
                        }
                    });
                //});
                
            });

            this.renderFavorite();
        },
        onMessage: function (e) {
            tracking.trackEvent("offerView", "contact", "message");
            uiModal.open({
                offer:this.model.toJSON()
            }, ViewMessage,this);

        },
        favorite:function(val){
            if (typeof val === "undefined") return this.isFavorite;
            this.isFavorite = val;
            if (val) {
                api.offer.addFavorite(this.model.get("id"));
            } else {
                api.offer.removeFavorite(this.model.get("id"));
            }
        },
        renderFavorite: function () {
            if (this.elFavorite) {
                this.elFavorite.toggleClass("selected", this.isFavorite);
            }
        },
        onFavorite: function () {
            
            if (this.favorite()) {
                tracking.trackEvent("offerView", "favorite", "off", (this.model ? this.model.id : null));
                this.favorite(false);
                uiToast.open(translate("favorite.remove.info"));
            } else {
                tracking.trackEvent("offerView","favorite","on",(this.model ? this.model.id :null));
                this.favorite(true);
                uiToast.open(translate("favorite.add.info"));
            }

            this.renderFavorite();
        },
        onremoving: function () {
            if (this.elFavorite) {
                this.elFavorite.off(".view");
            }
            if (this.elContact) {
                this.elContact.off(".view");
            }
            if (this.elShare) {
                this.elShare.off(".view");
            }

            uiDialog.options.close();
        }
       
    });
    return View;
});