﻿define([
  'jquery',
  'underscore',
  'globalize',
  'views/base',
  'models/register',
  'uiModal',
  'uiToast',
  'template',
  'uiGlobal',
  'uiContainers',
  'views/tos',
  'helper',
  'uiControlBase'
], function ($, _, globalize, ViewBase, Model, uiModal, uiToast, template, uiGlobal,uiContainers, ViewTOS, helper, uiControlBase) {

    var translate = globalize.localizeBase("views.register");
    var translateShared = globalize.localizeBase("views.shared");
    var translateTOS = globalize.localizeBase("views.tos");

    var SuccessResult = uiControlBase.extend({
        _loginButton:null,
        template: _.template("<div class=\"contant-fixed light inset center\"><i class=\"icon icon-logo icon-4x block\"></i>" +
                 "<b><%=title%></b><br /><br />" +
                 "<%=text %> <p><button data-role=\"login\" type=\"button\" class=\"btn btn-block btn-primary\"><i class=\"icon icon-login\"></i><%=buttonText%></button></p></div>"),
        getRenderData: function () {
            return {
                title:translate("confirm-account.info.title"),
                text: helper.nl2br(translate("confirm-account.info")).replace("[EMAIL]","<b class=\"text-big\">"+ this.options.email+"</b>"),
                buttonText: translate("confirm-account.info.login.button")
            }
        },
        onRender: function (el, html, renderData) {
            this._loginButton = $("[data-role='login']", el);
            this._loginButton.on("tap", _.bind(function (e) {
                if (e) e.preventDefault();
                
                if (this.options.onLogin) this.options.onLogin();
            },this));
        },
        onBeforeDestory: function () {
            if (this._loginButton) {
                this._loginButton.off("tap");
                this._loginButton = null;
            }
        },
    });

    var View = ViewBase.extend({
        events: {
            "submit form": "save",
            "tap #tos": "tos"
        },
        initialize: function () {
            this.constructor.__super__.initialize.call(this);

            this.title = translate("title");
            this.setModel(new Model());
            this.render();
        },
        renderTemplate: function () {
            return template.register({
                translate: translate
            });
        },
        save: function (e) {
            if (e) e.preventDefault();
            var self = this;
            var email = this.$("#email").val();
            var res = this.saveModel({
              //  nickName: this.$("#nickName").val(),
                email: email,
                password: this.$("#password").val(),
            }, this.$("form")).done(function (data) {
                uiToast.success(translate("success.info"));
                self.renderSuccess(email);
            });

        },
        renderSuccess: function (email) {
            this.$el.empty();

            uiContainers.footer.viewDefault();

            var res = new SuccessResult({
                email: email,
                onLogin: _.bind(function () {
                    this.navigate("login");
                },this)
            });
            this.render(res);
        },
        onpagerenderdone: function () {
            //   this.$("#nickName").focus();

            var elements = $("[data-href]", this.$el);
            elements.off(".register").hammer({
                prevent_default: true
            }).on(uiGlobal.event.tap + ".register", function (e) {
                e.preventDefault();
                helper.handleNavigation($(this));
            });
        },
        tos: function () {
            uiModal.open({}, ViewTOS, this);
        },
        initializeHeader: function (o) {
        //    o.view()
            o.viewSaveCancel({
                title: this.title,
                cancelText: translateShared("cancel"),
                okEnabled: false,
                onCancel: $.proxy(function () {
                    this.navigate('');
                }, this)
            });
        },
        initializeFooter: function (o) {
            o.viewNone();
        },
        onremoving: function () {
            // destroy current controls
            if (this.$el) {
                $("[data-href]", this.$el).off(".register");
            }
        }
        
    });
    return View;
});